@@ -17,7 +17,6 @@
 """Instance Metadata information."""
 
 import base64
-import json
 import os
 import posixpath
 
@@ -30,12 +29,10 @@ from nova.compute import flavors
 from nova import conductor
 from nova import context
 from nova import network
+from nova import objects
 from nova.objects import base as obj_base
-from nova.objects import block_device as block_device_obj
-from nova.objects import instance as instance_obj
-from nova.objects import security_group as secgroup_obj
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import importutils
+from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
 from nova import utils
@@ -85,6 +82,8 @@ MD_JSON_NAME = "meta_data.json"
 VD_JSON_NAME = "vendor_data.json"
 UD_NAME = "user_data"
 PASS_NAME = "password"
+MIME_TYPE_TEXT_PLAIN = "text/plain"
+MIME_TYPE_APPLICATION_JSON = "application/json"
 
 LOG = logging.getLogger(__name__)
 
@@ -115,14 +114,16 @@ class InstanceMetadata():
         ctxt = context.get_admin_context()
 
         # NOTE(danms): This should be removed after bp:compute-manager-objects
-        if not isinstance(instance, instance_obj.Instance):
+        if not isinstance(instance, obj_base.NovaObject):
             expected = ['metadata', 'system_metadata']
             if 'info_cache' in instance:
                 expected.append('info_cache')
-            instance = instance_obj.Instance._from_db_object(
-                ctxt, instance_obj.Instance(), instance,
+            instance = objects.Instance._from_db_object(
+                ctxt, objects.Instance(), instance,
                 expected_attrs=expected)
 
+        # The default value of mimeType is set to MIME_TYPE_TEXT_PLAIN
+        self.set_mimetype(MIME_TYPE_TEXT_PLAIN)
         self.instance = instance
         self.extra_md = extra_md
 
@@ -134,7 +135,7 @@ class InstanceMetadata():
         self.availability_zone = ec2utils.get_availability_zone_by_host(
                 instance['host'], capi)
 
-        self.security_groups = secgroup_obj.SecurityGroupList.get_by_instance(
+        self.security_groups = objects.SecurityGroupList.get_by_instance(
             ctxt, instance)
 
         self.mappings = _format_instance_mapping(ctxt, instance)
@@ -176,7 +177,7 @@ class InstanceMetadata():
                 'content_path': "/%s/%s" % (CONTENT_DIR, key)}
 
         # 'content' is passed in from the configdrive code in
-        # nova/virt/libvirt/driver.py.  Thats how we get the injected files
+        # nova/virt/libvirt/driver.py.  That's how we get the injected files
         # (personalities) in. AFAIK they're not stored in the db at all,
         # so are not available later (web service metadata time).
         for (path, contents) in content:
@@ -209,6 +210,12 @@ class InstanceMetadata():
         self.route_configuration = RouteConfiguration(path_handlers)
         return self.route_configuration
 
+    def set_mimetype(self, mime_type):
+        self.md_mimetype = mime_type
+
+    def get_mimetype(self):
+        return self.md_mimetype
+
     def get_ec2_metadata(self, version):
         if version == "latest":
             version = VERSIONS[-1]
@@ -319,7 +326,8 @@ class InstanceMetadata():
         if self._check_os_version(GRIZZLY, version):
             metadata['random_seed'] = base64.b64encode(os.urandom(512))
 
-        return json.dumps(metadata)
+        self.set_mimetype(MIME_TYPE_APPLICATION_JSON)
+        return jsonutils.dumps(metadata)
 
     def _handle_content(self, path_tokens):
         if len(path_tokens) == 1:
@@ -352,7 +360,8 @@ class InstanceMetadata():
 
     def _vendor_data(self, version, path):
         if self._check_os_version(HAVANA, version):
-            return json.dumps(self.vddriver.get())
+            self.set_mimetype(MIME_TYPE_APPLICATION_JSON)
+            return jsonutils.dumps(self.vddriver.get())
         raise KeyError(path)
 
     def _check_version(self, required, requested, versions=VERSIONS):
@@ -372,6 +381,9 @@ class InstanceMetadata():
         else:
             path = posixpath.normpath(path)
 
+        # Set default mimeType. It will be modified only if there is a change
+        self.set_mimetype(MIME_TYPE_TEXT_PLAIN)
+
         # fix up requests, prepending /ec2 to anything that does not match
         path_tokens = path.split('/')[1:]
         if path_tokens[0] not in ("ec2", "openstack"):
@@ -391,7 +403,7 @@ class InstanceMetadata():
                 today = timeutils.utcnow().strftime("%Y-%m-%d")
                 versions = [v for v in OPENSTACK_VERSIONS if v <= today]
                 if OPENSTACK_VERSIONS != versions:
-                    LOG.debug(_("future versions %s hidden in version list"),
+                    LOG.debug("future versions %s hidden in version list",
                               [v for v in OPENSTACK_VERSIONS
                                if v not in versions])
                 versions += ["latest"]
@@ -428,7 +440,7 @@ class InstanceMetadata():
                 pass
 
             filepath = os.path.join('ec2', version, 'meta-data.json')
-            yield (filepath, json.dumps(data['meta-data']))
+            yield (filepath, jsonutils.dumps(data['meta-data']))
 
         ALL_OPENSTACK_VERSIONS = OPENSTACK_VERSIONS + ["latest"]
         for version in ALL_OPENSTACK_VERSIONS:
@@ -505,12 +517,12 @@ def get_metadata_by_address(conductor_api, address):
 def get_metadata_by_instance_id(conductor_api, instance_id, address,
                                 ctxt=None):
     ctxt = ctxt or context.get_admin_context()
-    instance = instance_obj.Instance.get_by_uuid(ctxt, instance_id)
+    instance = objects.Instance.get_by_uuid(ctxt, instance_id)
     return InstanceMetadata(instance, address)
 
 
 def _format_instance_mapping(ctxt, instance):
-    bdms = block_device_obj.BlockDeviceMappingList.get_by_instance_uuid(
+    bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
             ctxt, instance.uuid)
     return block_device.instance_block_mapping(instance, bdms)
 
