@@ -29,7 +29,9 @@ from nova.compute import task_states
 from nova.compute import utils as compute_utils
 from nova.compute import vm_states
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova.i18n import _LE
+from nova.i18n import _LW
 from nova.openstack.common import log as logging
 from nova import quota
 
@@ -129,29 +131,63 @@ def status_from_state(vm_state, task_state='default'):
     task_map = _STATE_MAP.get(vm_state, dict(default='UNKNOWN'))
     status = task_map.get(task_state, task_map['default'])
     if status == "UNKNOWN":
-        LOG.error(_("status is UNKNOWN from vm_state=%(vm_state)s "
-                    "task_state=%(task_state)s. Bad upgrade or db "
-                    "corrupted?"),
+        LOG.error(_LE("status is UNKNOWN from vm_state=%(vm_state)s "
+                      "task_state=%(task_state)s. Bad upgrade or db "
+                      "corrupted?"),
                   {'vm_state': vm_state, 'task_state': task_state})
     return status
 
 
-def task_and_vm_state_from_status(status):
-    """Map the server status string to list of vm states and
+def task_and_vm_state_from_status(statuses):
+    """Map the server's multiple status strings to list of vm states and
     list of task states.
     """
     vm_states = set()
     task_states = set()
+    lower_statuses = [status.lower() for status in statuses]
     for state, task_map in _STATE_MAP.iteritems():
         for task_state, mapped_state in task_map.iteritems():
             status_string = mapped_state
-            if status.lower() == status_string.lower():
+            if status_string.lower() in lower_statuses:
                 vm_states.add(state)
                 task_states.add(task_state)
     # Add sort to avoid different order on set in Python 3
     return sorted(vm_states), sorted(task_states)
 
 
+def get_sort_params(input_params, default_key='created_at',
+                    default_dir='desc'):
+    """Retrieves sort keys/directions parameters.
+
+    Processes the parameters to create a list of sort keys and sort directions
+    that correspond to the 'sort_key' and 'sort_dir' parameter values. These
+    sorting parameters can be specified multiple times in order to generate
+    the list of sort keys and directions.
+
+    The input parameters are not modified.
+
+    :param input_params: webob.multidict of request parameters (from
+                         nova.wsgi.Request.params)
+    :param default_key: default sort key value, added to the list if no
+                        'sort_key' parameters are supplied
+    :param default_dir: default sort dir value, added to the list if no
+                        'sort_dir' parameters are supplied
+    :returns: list of sort keys, list of sort dirs
+    """
+    params = input_params.copy()
+    sort_keys = []
+    sort_dirs = []
+    while 'sort_key' in params:
+        sort_keys.append(params.pop('sort_key').strip())
+    while 'sort_dir' in params:
+        sort_dirs.append(params.pop('sort_dir').strip())
+    if len(sort_keys) == 0 and default_key:
+        sort_keys.append(default_key)
+    if len(sort_dirs) == 0 and default_dir:
+        sort_dirs.append(default_dir)
+    return sort_keys, sort_dirs
+
+
 def get_pagination_params(request):
     """Return marker, limit tuple from request.
 
@@ -239,29 +275,6 @@ def get_limit_and_marker(request, max_limit=CONF.osapi_max_limit):
     return limit, marker
 
 
-def limited_by_marker(items, request, max_limit=CONF.osapi_max_limit):
-    """Return a slice of items according to the requested marker and limit."""
-    limit, marker = get_limit_and_marker(request, max_limit)
-
-    limit = min(max_limit, limit)
-    start_index = 0
-    if marker:
-        start_index = -1
-        for i, item in enumerate(items):
-            if 'flavorid' in item:
-                if item['flavorid'] == marker:
-                    start_index = i + 1
-                    break
-            elif item['id'] == marker or item.get('uuid') == marker:
-                start_index = i + 1
-                break
-        if start_index < 0:
-            msg = _('marker [%s] not found') % marker
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-    range_end = start_index + limit
-    return items[start_index:range_end]
-
-
 def get_id_from_href(href):
     """Return the id or uuid portion of a url.
 
@@ -296,9 +309,8 @@ def remove_version_from_href(href):
     new_path = '/'.join(url_parts)
 
     if new_path == parsed_url.path:
-        msg = _('href %s does not contain version') % href
-        LOG.debug(msg)
-        raise ValueError(msg)
+        LOG.debug('href %s does not contain version' % href)
+        raise ValueError(_('href %s does not contain version') % href)
 
     parsed_url = list(parsed_url)
     parsed_url[2] = new_path
@@ -312,8 +324,7 @@ def check_img_metadata_properties_quota(context, metadata):
         QUOTAS.limit_check(context, metadata_items=len(metadata))
     except exception.OverQuota:
         expl = _("Image metadata limit exceeded")
-        raise webob.exc.HTTPRequestEntityTooLarge(explanation=expl,
-                                                headers={'Retry-After': 0})
+        raise webob.exc.HTTPForbidden(explanation=expl)
 
     #  check the key length.
     if isinstance(metadata, dict):
@@ -474,8 +485,8 @@ def check_snapshots_enabled(f):
     @functools.wraps(f)
     def inner(*args, **kwargs):
         if not CONF.allow_instance_snapshots:
-            LOG.warn(_('Rejecting snapshot request, snapshots currently'
-                       ' disabled'))
+            LOG.warn(_LW('Rejecting snapshot request, snapshots currently'
+                         ' disabled'))
             msg = _("Instance snapshots are not permitted at this time.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
         return f(*args, **kwargs)
@@ -571,7 +582,8 @@ class ViewBuilder(object):
         url_parts = list(urlparse.urlsplit(orig_url))
         prefix_parts = list(urlparse.urlsplit(prefix))
         url_parts[0:2] = prefix_parts[0:2]
-        return urlparse.urlunsplit(url_parts)
+        url_parts[2] = prefix_parts[2] + url_parts[2]
+        return urlparse.urlunsplit(url_parts).rstrip('/')
 
     def _update_glance_link_prefix(self, orig_url):
         return self._update_link_prefix(orig_url,
