@@ -18,7 +18,7 @@ import webob
 from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
 from nova.consoleauth import rpcapi as consoleauth_rpcapi
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 
 
 authorize = extensions.extension_authorizer('compute', 'console_auth_tokens')
@@ -59,7 +59,7 @@ class Console_auth_tokens(extensions.ExtensionDescriptor):
     alias = "os-console-auth-tokens"
     namespace = ("http://docs.openstack.org/compute/ext/"
                  "consoles-auth-tokens/api/v2")
-    updated = "2013-08-13T00:00:00+00:00"
+    updated = "2013-08-13T00:00:00Z"
 
     def get_resources(self):
         controller = ConsoleAuthTokensController()
