@@ -15,12 +15,13 @@
 
 import webob.exc
 
+from nova.api.openstack import common
 from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import compute
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 
 
 authorize = extensions.extension_authorizer('compute', 'server_diagnostics')
@@ -37,18 +38,27 @@ class ServerDiagnosticsTemplate(xmlutil.TemplateBuilder):
 
 
 class ServerDiagnosticsController(object):
+    def __init__(self):
+        self.compute_api = compute.API()
+
     @wsgi.serializers(xml=ServerDiagnosticsTemplate)
     def index(self, req, server_id):
         context = req.environ["nova.context"]
         authorize(context)
-        compute_api = compute.API()
         try:
-            instance = compute_api.get(context, server_id, want_objects=True)
-        except exception.NotFound():
-            msg = _("Instance not found")
-            raise webob.exc.HTTPNotFound(explanation=msg)
+            instance = self.compute_api.get(context, server_id,
+                want_objects=True)
+        except exception.InstanceNotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.format_message())
 
-        return compute_api.get_diagnostics(context, instance)
+        try:
+            return self.compute_api.get_diagnostics(context, instance)
+        except exception.InstanceInvalidState as state_error:
+            common.raise_http_conflict_for_instance_invalid_state(state_error,
+                    'get_diagnostics')
+        except NotImplementedError:
+            msg = _("Unable to get diagnostics, functionality not implemented")
+            raise webob.exc.HTTPNotImplemented(explanation=msg)
 
 
 class Server_diagnostics(extensions.ExtensionDescriptor):
@@ -58,11 +68,11 @@ class Server_diagnostics(extensions.ExtensionDescriptor):
     alias = "os-server-diagnostics"
     namespace = ("http://docs.openstack.org/compute/ext/"
                  "server-diagnostics/api/v1.1")
-    updated = "2011-12-21T00:00:00+00:00"
+    updated = "2011-12-21T00:00:00Z"
 
     def get_resources(self):
         parent_def = {'member_name': 'server', 'collection_name': 'servers'}
-        #NOTE(bcwaldon): This should be prefixed with 'os-'
+        # NOTE(bcwaldon): This should be prefixed with 'os-'
         ext = extensions.ResourceExtension('diagnostics',
                                            ServerDiagnosticsController(),
                                            parent=parent_def)
