@@ -19,9 +19,9 @@ from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import compute
 from nova import exception
+from nova.i18n import _
+from nova import objects
 from nova.objects import external_event as external_event_obj
-from nova.objects import instance as instance_obj
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 
 
@@ -82,7 +82,7 @@ class ServerExternalEventsController(wsgi.Controller):
 
         for _event in body_events:
             client_event = dict(_event)
-            event = external_event_obj.InstanceExternalEvent()
+            event = objects.InstanceExternalEvent(context)
 
             try:
                 event.instance_uuid = client_event.pop('server_uuid')
@@ -105,13 +105,13 @@ class ServerExternalEventsController(wsgi.Controller):
             instance = instances.get(event.instance_uuid)
             if not instance:
                 try:
-                    instance = instance_obj.Instance.get_by_uuid(
+                    instance = objects.Instance.get_by_uuid(
                         context, event.instance_uuid)
                     instances[event.instance_uuid] = instance
                 except exception.InstanceNotFound:
-                    LOG.debug(_('Dropping event %(name)s:%(tag)s for unknown '
-                                'instance %(instance_uuid)s'),
-                                dict(event.iteritems()))
+                    LOG.debug('Dropping event %(name)s:%(tag)s for unknown '
+                              'instance %(instance_uuid)s',
+                              dict(event.iteritems()))
                     _event['status'] = 'failed'
                     _event['code'] = 404
                     result = 207
@@ -162,7 +162,7 @@ class Server_external_events(extensions.ExtensionDescriptor):
     alias = "os-server-external-events"
     namespace = ("http://docs.openstack.org/compute/ext/"
                  "server-external-events/api/v2")
-    updated = "2014-02-18T00:00:00-00:00"
+    updated = "2014-02-18T00:00:00Z"
 
     def get_resources(self):
         resource = extensions.ResourceExtension('os-server-external-events',
