@@ -23,11 +23,15 @@ from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 import nova.exception
-from nova.objects import instance as instance_obj
-from nova.objects import instance_group as instance_group_obj
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova.i18n import _LE
+from nova import objects
+from nova.openstack.common import log as logging
 from nova import utils
 
+LOG = logging.getLogger(__name__)
+
+
 # NOTE(russellb) There is one other policy, 'legacy', but we don't allow that
 # being set via the API.  It's only used when a group gets automatically
 # created to support the legacy behavior of the 'group' scheduler hint.
@@ -125,24 +129,13 @@ class ServerGroupXMLDeserializer(wsgi.MetadataXMLDeserializer):
                     policies.append(node.firstChild.nodeValue)
             return policies
 
-    def _extract_members(self, server_group_node):
-        """Marshal the server group members element of a parsed request."""
-        members_node = self.find_first_child_named(server_group_node,
-                                                   'members')
-        if members_node is not None:
-            member_nodes = self.find_children_named(members_node,
-                                                    'member')
-
-            members = []
-            if member_nodes is not None:
-                for node in member_nodes:
-                    members.append(node.firstChild.nodeValue)
-            return members
-
 
 class ServerGroupController(wsgi.Controller):
     """The Server group API controller for the OpenStack API."""
 
+    def __init__(self, ext_mgr):
+        self.ext_mgr = ext_mgr
+
     def _format_server_group(self, context, group):
         # the id field has its value as the uuid of the server group
         # There is no 'uuid' key in server_group seen by clients.
@@ -152,12 +145,14 @@ class ServerGroupController(wsgi.Controller):
         server_group['id'] = group.uuid
         server_group['name'] = group.name
         server_group['policies'] = group.policies or []
-        server_group['metadata'] = group.metadetails or {}
+        # NOTE(danms): This has been exposed to the user, but never used.
+        # Since we can't remove it, just make sure it's always empty.
+        server_group['metadata'] = {}
         members = []
         if group.members:
             # Display the instances that are not deleted.
             filters = {'uuid': group.members, 'deleted': False}
-            instances = instance_obj.InstanceList.get_by_filters(
+            instances = objects.InstanceList.get_by_filters(
                 context, filters=filters)
             members = [instance.uuid for instance in instances]
         server_group['members'] = members
@@ -181,6 +176,11 @@ class ServerGroupController(wsgi.Controller):
             msg = _("Invalid policies: %s") % ', '.join(not_supported)
             raise nova.exception.InvalidInput(reason=msg)
 
+        # Note(wingwj): It doesn't make sense to store duplicate policies.
+        if sorted(set(policies)) != sorted(policies):
+            msg = _("Duplicate policies configured!")
+            raise nova.exception.InvalidInput(reason=msg)
+
     def _validate_input_body(self, body, entity_name):
         if not self.is_valid_body(body, entity_name):
             msg = _("the body is invalid.")
@@ -218,7 +218,7 @@ class ServerGroupController(wsgi.Controller):
         """Return data about the given server group."""
         context = _authorize_context(req)
         try:
-            sg = instance_group_obj.InstanceGroup.get_by_uuid(context, id)
+            sg = objects.InstanceGroup.get_by_uuid(context, id)
         except nova.exception.InstanceGroupNotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.format_message())
         return {'server_group': self._format_server_group(context, sg)}
@@ -227,10 +227,35 @@ class ServerGroupController(wsgi.Controller):
         """Delete an server group."""
         context = _authorize_context(req)
         try:
-            sg = instance_group_obj.InstanceGroup.get_by_uuid(context, id)
-            sg.destroy(context)
+            sg = objects.InstanceGroup.get_by_uuid(context, id)
         except nova.exception.InstanceGroupNotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.format_message())
+
+        quotas = None
+        if self.ext_mgr.is_loaded('os-server-group-quotas'):
+            quotas = objects.Quotas()
+            project_id, user_id = objects.quotas.ids_from_server_group(context,
+                                                                       sg)
+            try:
+                # We have to add the quota back to the user that created
+                # the server group
+                quotas.reserve(context, project_id=project_id,
+                               user_id=user_id, server_groups=-1)
+            except Exception:
+                quotas = None
+                LOG.exception(_LE("Failed to update usages deallocating "
+                                  "server group"))
+
+        try:
+            sg.destroy(context)
+        except nova.exception.InstanceGroupNotFound as e:
+            if quotas:
+                quotas.rollback()
+            raise webob.exc.HTTPNotFound(explanation=e.format_message())
+
+        if quotas:
+            quotas.commit()
+
         return webob.Response(status_int=204)
 
     @wsgi.serializers(xml=ServerGroupsTemplate)
@@ -239,9 +264,9 @@ class ServerGroupController(wsgi.Controller):
         context = _authorize_context(req)
         project_id = context.project_id
         if 'all_projects' in req.GET and context.is_admin:
-            sgs = instance_group_obj.InstanceGroupList.get_all(context)
+            sgs = objects.InstanceGroupList.get_all(context)
         else:
-            sgs = instance_group_obj.InstanceGroupList.get_by_project_id(
+            sgs = objects.InstanceGroupList.get_by_project_id(
                     context, project_id)
         limited_list = common.limited(sgs.objects, req)
         result = [self._format_server_group(context, group)
@@ -259,17 +284,32 @@ class ServerGroupController(wsgi.Controller):
         except nova.exception.InvalidInput as e:
             raise exc.HTTPBadRequest(explanation=e.format_message())
 
+        quotas = None
+        if self.ext_mgr.is_loaded('os-server-group-quotas'):
+            quotas = objects.Quotas()
+            try:
+                quotas.reserve(context, project_id=context.project_id,
+                               user_id=context.user_id, server_groups=1)
+            except nova.exception.OverQuota:
+                msg = _("Quota exceeded, too many server groups.")
+                raise exc.HTTPForbidden(explanation=msg)
+
         vals = body['server_group']
-        sg = instance_group_obj.InstanceGroup()
+        sg = objects.InstanceGroup(context)
         sg.project_id = context.project_id
         sg.user_id = context.user_id
         try:
             sg.name = vals.get('name')
             sg.policies = vals.get('policies')
-            sg.create(context)
+            sg.create()
         except ValueError as e:
+            if quotas:
+                quotas.rollback()
             raise exc.HTTPBadRequest(explanation=e)
 
+        if quotas:
+            quotas.commit()
+
         return {'server_group': self._format_server_group(context, sg)}
 
 
@@ -284,14 +324,14 @@ class Server_groups(extensions.ExtensionDescriptor):
     alias = "os-server-groups"
     namespace = ("http://docs.openstack.org/compute/ext/"
                  "servergroups/api/v2")
-    updated = "2013-06-20T00:00:00+00:00"
+    updated = "2013-06-20T00:00:00Z"
 
     def get_resources(self):
         resources = []
 
         res = extensions.ResourceExtension(
             'os-server-groups',
-            controller=ServerGroupController(),
+            controller=ServerGroupController(self.ext_mgr),
             member_actions={"action": "POST", })
 
         resources.append(res)
