@@ -12,7 +12,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo.config import cfg
 import webob.exc
 
 from nova.api.openstack import extensions
@@ -20,13 +19,11 @@ from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import compute
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova import servicegroup
 from nova import utils
 
 authorize = extensions.extension_authorizer('compute', 'services')
-CONF = cfg.CONF
-CONF.import_opt('service_down_time', 'nova.service')
 
 
 class ServicesIndexTemplate(xmlutil.TemplateBuilder):
@@ -192,7 +189,7 @@ class ServiceController(object):
                     msg = _('The string containing the reason for disabling '
                             'the service contains invalid characters or is '
                             'too long.')
-                    raise webob.exc.HTTPUnprocessableEntity(detail=msg)
+                    raise webob.exc.HTTPBadRequest(explanation=msg)
 
                 status_detail['disabled_reason'] = reason
                 ret_value['service']['disabled_reason'] = reason
@@ -200,7 +197,7 @@ class ServiceController(object):
             msg = _('Invalid attribute in the request')
             if 'host' in body and 'binary' in body:
                 msg = _('Missing disabled reason field')
-            raise webob.exc.HTTPUnprocessableEntity(detail=msg)
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
         try:
             self.host_api.service_update(context, host, binary, status_detail)
@@ -216,7 +213,7 @@ class Services(extensions.ExtensionDescriptor):
     name = "Services"
     alias = "os-services"
     namespace = "http://docs.openstack.org/compute/ext/services/api/v2"
-    updated = "2012-10-28T00:00:00-00:00"
+    updated = "2012-10-28T00:00:00Z"
 
     def get_resources(self):
         resources = []
