@@ -24,8 +24,8 @@ from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import compute
 from nova import exception
-from nova.objects import block_device as block_device_obj
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova import objects
 from nova.openstack.common import log as logging
 from nova.openstack.common import strutils
 from nova.openstack.common import uuidutils
@@ -175,8 +175,8 @@ class VolumeController(wsgi.Controller):
 
         try:
             vol = self.volume_api.get(context, id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
         return {'volume': _translate_volume_detail_view(context, vol)}
 
@@ -189,8 +189,8 @@ class VolumeController(wsgi.Controller):
 
         try:
             self.volume_api.delete(context, id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
         return webob.Response(status_int=202)
 
     @wsgi.serializers(xml=VolumesTemplate)
@@ -221,7 +221,8 @@ class VolumeController(wsgi.Controller):
         authorize(context)
 
         if not self.is_valid_body(body, 'volume'):
-            raise exc.HTTPUnprocessableEntity()
+            msg = _("volume not specified")
+            raise exc.HTTPBadRequest(explanation=msg)
 
         vol = body['volume']
 
@@ -352,15 +353,15 @@ class VolumeAttachmentController(wsgi.Controller):
         volume_id = id
         try:
             instance = self.compute_api.get(context, server_id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
-        bdms = block_device_obj.BlockDeviceMappingList.get_by_instance_uuid(
+        bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
                 context, instance['uuid'])
 
         if not bdms:
-            LOG.debug(_("Instance %s is not attached."), server_id)
-            raise exc.HTTPNotFound()
+            msg = _("Instance %s is not attached.") % server_id
+            raise exc.HTTPNotFound(explanation=msg)
 
         assigned_mountpoint = None
 
@@ -370,8 +371,8 @@ class VolumeAttachmentController(wsgi.Controller):
                 break
 
         if assigned_mountpoint is None:
-            LOG.debug("volume_id not found")
-            raise exc.HTTPNotFound()
+            msg = _("volume_id not found: %s") % volume_id
+            raise exc.HTTPNotFound(explanation=msg)
 
         return {'volumeAttachment': _translate_attachment_detail_view(
             volume_id,
@@ -392,7 +393,8 @@ class VolumeAttachmentController(wsgi.Controller):
         authorize_attach(context, action='create')
 
         if not self.is_valid_body(body, 'volumeAttachment'):
-            raise exc.HTTPUnprocessableEntity()
+            msg = _("volumeAttachment not specified")
+            raise exc.HTTPBadRequest(explanation=msg)
         try:
             volume_id = body['volumeAttachment']['volumeId']
         except KeyError:
@@ -414,8 +416,8 @@ class VolumeAttachmentController(wsgi.Controller):
                                             want_objects=True)
             device = self.compute_api.attach_volume(context, instance,
                                                     volume_id, device)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
         except exception.InstanceIsLocked as e:
             raise exc.HTTPConflict(explanation=e.format_message())
         except exception.InstanceInvalidState as state_error:
@@ -449,7 +451,8 @@ class VolumeAttachmentController(wsgi.Controller):
         authorize_attach(context, action='update')
 
         if not self.is_valid_body(body, 'volumeAttachment'):
-            raise exc.HTTPUnprocessableEntity()
+            msg = _("volumeAttachment not specified")
+            raise exc.HTTPBadRequest(explanation=msg)
 
         old_volume_id = id
         old_volume = self.volume_api.get(context, old_volume_id)
@@ -465,10 +468,10 @@ class VolumeAttachmentController(wsgi.Controller):
         try:
             instance = self.compute_api.get(context, server_id,
                                             want_objects=True)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
-        bdms = block_device_obj.BlockDeviceMappingList.get_by_instance_uuid(
+        bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
                 context, instance.uuid)
         found = False
         try:
@@ -491,7 +494,8 @@ class VolumeAttachmentController(wsgi.Controller):
                     'swap_volume')
 
         if not found:
-            raise exc.HTTPNotFound()
+            msg = _("volume_id not found: %s") % old_volume_id
+            raise exc.HTTPNotFound(explanation=msg)
         else:
             return webob.Response(status_int=202)
 
@@ -505,17 +509,18 @@ class VolumeAttachmentController(wsgi.Controller):
         LOG.audit(_("Detach volume %s"), volume_id, context=context)
 
         try:
-            instance = self.compute_api.get(context, server_id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+            instance = self.compute_api.get(context, server_id,
+                                            want_objects=True)
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
         volume = self.volume_api.get(context, volume_id)
 
-        bdms = block_device_obj.BlockDeviceMappingList.get_by_instance_uuid(
+        bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
                 context, instance['uuid'])
         if not bdms:
-            LOG.debug(_("Instance %s is not attached."), server_id)
-            raise exc.HTTPNotFound()
+            msg = _("Instance %s is not attached.") % server_id
+            raise exc.HTTPNotFound(explanation=msg)
 
         found = False
         try:
@@ -540,7 +545,8 @@ class VolumeAttachmentController(wsgi.Controller):
                     'detach_volume')
 
         if not found:
-            raise exc.HTTPNotFound()
+            msg = _("volume_id not found: %s") % volume_id
+            raise exc.HTTPNotFound(explanation=msg)
         else:
             return webob.Response(status_int=202)
 
@@ -551,10 +557,10 @@ class VolumeAttachmentController(wsgi.Controller):
 
         try:
             instance = self.compute_api.get(context, server_id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
-        bdms = block_device_obj.BlockDeviceMappingList.get_by_instance_uuid(
+        bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
                 context, instance['uuid'])
         limited_list = common.limited(bdms, req)
         results = []
@@ -633,8 +639,8 @@ class SnapshotController(wsgi.Controller):
 
         try:
             vol = self.volume_api.get_snapshot(context, id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
 
         return {'snapshot': _translate_snapshot_detail_view(context, vol)}
 
@@ -647,8 +653,8 @@ class SnapshotController(wsgi.Controller):
 
         try:
             self.volume_api.delete_snapshot(context, id)
-        except exception.NotFound:
-            raise exc.HTTPNotFound()
+        except exception.NotFound as e:
+            raise exc.HTTPNotFound(explanation=e.format_message())
         return webob.Response(status_int=202)
 
     @wsgi.serializers(xml=SnapshotsTemplate)
@@ -678,7 +684,8 @@ class SnapshotController(wsgi.Controller):
         authorize(context)
 
         if not self.is_valid_body(body, 'snapshot'):
-            raise exc.HTTPUnprocessableEntity()
+            msg = _("snapshot not specified")
+            raise exc.HTTPBadRequest(explanation=msg)
 
         snapshot = body['snapshot']
         volume_id = snapshot['volume_id']
@@ -691,7 +698,7 @@ class SnapshotController(wsgi.Controller):
             force = strutils.bool_from_string(force, strict=True)
         except ValueError:
             msg = _("Invalid value '%s' for force.") % force
-            raise exception.InvalidParameterValue(err=msg)
+            raise exc.HTTPBadRequest(explanation=msg)
 
         if force:
             create_func = self.volume_api.create_snapshot_force
@@ -712,7 +719,7 @@ class Volumes(extensions.ExtensionDescriptor):
     name = "Volumes"
     alias = "os-volumes"
     namespace = "http://docs.openstack.org/compute/ext/volumes/api/v1.1"
-    updated = "2011-03-25T00:00:00+00:00"
+    updated = "2011-03-25T00:00:00Z"
 
     def get_resources(self):
         resources = []
