@@ -19,11 +19,13 @@ import webob
 from webob import exc
 
 from nova.api.openstack import common
+from nova.api.openstack.compute.schemas.v3 import attach_interfaces
 from nova.api.openstack import extensions
+from nova.api import validation
 from nova import compute
 from nova import exception
+from nova.i18n import _
 from nova import network
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 
 
@@ -52,11 +54,13 @@ class InterfaceAttachmentController(object):
         self.network_api = network.API()
         super(InterfaceAttachmentController, self).__init__()
 
+    @extensions.expected_errors((404, 501))
     def index(self, req, server_id):
         """Returns the list of interface attachments for a given instance."""
         return self._items(req, server_id,
             entity_maker=_translate_interface_attachment_view)
 
+    @extensions.expected_errors(404)
     def show(self, req, server_id, id):
         """Return data about the given interface attachment."""
         context = req.environ['nova.context']
@@ -73,9 +77,11 @@ class InterfaceAttachmentController(object):
         if port_info['port']['device_id'] != server_id:
             raise exc.HTTPNotFound()
 
-        return {'interface_attachment': _translate_interface_attachment_view(
+        return {'interfaceAttachment': _translate_interface_attachment_view(
                 port_info['port'])}
 
+    @extensions.expected_errors((400, 404, 409, 500, 501))
+    @validation.schema(attach_interfaces.create)
     def create(self, req, server_id, body):
         """Attach an interface to an instance."""
         context = req.environ['nova.context']
@@ -85,7 +91,7 @@ class InterfaceAttachmentController(object):
         port_id = None
         req_ip = None
         if body:
-            attachment = body['interface_attachment']
+            attachment = body['interfaceAttachment']
             network_id = attachment.get('net_id', None)
             port_id = attachment.get('port_id', None)
             try:
@@ -106,25 +112,27 @@ class InterfaceAttachmentController(object):
             vif = self.compute_api.attach_interface(context,
                 instance, network_id, port_id, req_ip)
         except (exception.PortNotFound,
+                exception.FixedIpAlreadyInUse,
                 exception.PortInUse,
                 exception.NetworkDuplicated,
                 exception.NetworkAmbiguous,
                 exception.NetworkNotFound) as e:
             raise exc.HTTPBadRequest(explanation=e.format_message())
+        except exception.InstanceIsLocked as e:
+            raise exc.HTTPConflict(explanation=e.format_message())
         except NotImplementedError as e:
             raise webob.exc.HTTPNotImplemented(explanation=e.format_message())
         except exception.InterfaceAttachFailed as e:
             LOG.exception(e)
             raise webob.exc.HTTPInternalServerError(
                 explanation=e.format_message())
+        except exception.InstanceInvalidState as state_error:
+            common.raise_http_conflict_for_instance_invalid_state(state_error,
+                    'attach_interface')
 
         return self.show(req, server_id, vif['id'])
 
-    def update(self, req, server_id, id, body):
-        """Update a interface attachment.  We don't currently support this."""
-        msg = _("Attachments update is not supported")
-        raise exc.HTTPNotImplemented(explanation=msg)
-
+    @extensions.expected_errors((404, 409, 501))
     def delete(self, req, server_id, id):
         """Detach an interface from an instance."""
         context = req.environ['nova.context']
@@ -139,8 +147,13 @@ class InterfaceAttachmentController(object):
                 instance, port_id=port_id)
         except exception.PortNotFound:
             raise exc.HTTPNotFound()
+        except exception.InstanceIsLocked as e:
+            raise exc.HTTPConflict(explanation=e.format_message())
         except NotImplementedError as e:
             raise webob.exc.HTTPNotImplemented(explanation=e.format_message())
+        except exception.InstanceInvalidState as state_error:
+            common.raise_http_conflict_for_instance_invalid_state(state_error,
+                    'detach_interface')
 
         return webob.Response(status_int=202)
 
@@ -164,7 +177,7 @@ class InterfaceAttachmentController(object):
         ports = data.get('ports', [])
         results = [entity_maker(port) for port in ports]
 
-        return {'interface_attachments': results}
+        return {'interfaceAttachments': results}
 
 
 class AttachInterfaces(extensions.V3APIExtensionBase):
@@ -175,7 +188,7 @@ class AttachInterfaces(extensions.V3APIExtensionBase):
     version = 1
 
     def get_resources(self):
-        res = [extensions.ResourceExtension(ALIAS,
+        res = [extensions.ResourceExtension('os-interface',
                                             InterfaceAttachmentController(),
                                             parent=dict(
                                                 member_name='server',
