@@ -22,19 +22,17 @@ import six
 from webob import exc
 
 from nova.api.openstack import common
+from nova.api.openstack.compute.schemas.v3 import cells
 from nova.api.openstack import extensions
-from nova.api.openstack import wsgi
+from nova.api import validation
 from nova.cells import rpcapi as cells_rpcapi
 from nova.compute import api as compute
 from nova import exception
-from nova.openstack.common.gettextutils import _
-from nova.openstack.common import log as logging
+from nova.i18n import _
 from nova.openstack.common import strutils
-from nova.openstack.common import timeutils
 from nova import rpc
 
 
-LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 CONF.import_opt('name', 'nova.cells.opts', group='cells')
 CONF.import_opt('capabilities', 'nova.cells.opts', group='cells')
@@ -171,13 +169,18 @@ class CellsController(object):
             raise exc.HTTPNotFound(explanation=e.format_message())
         return dict(cell=_scrub_cell(cell))
 
+    # NOTE(gmann): Returns 200 for backwards compatibility but should be 204
+    # as this operation complete the deletion of aggregate resource and return
+    # no response body.
     @extensions.expected_errors((403, 404, 501))
     @common.check_cells_enabled
-    @wsgi.response(204)
     def delete(self, req, id):
         """Delete a child or parent cell entry.  'id' is a cell name."""
         context = req.environ['nova.context']
+
         authorize(context)
+        authorize(context, action="delete")
+
         try:
             num_deleted = self.cells_rpcapi.cell_delete(context, id)
         except exception.CellsUpdateUnsupported as e:
@@ -186,24 +189,6 @@ class CellsController(object):
             raise exc.HTTPNotFound(
                 explanation=_("Cell %s doesn't exist.") % id)
 
-    def _validate_cell_name(self, cell_name):
-        """Validate cell name is not empty and doesn't contain '!' or '.'."""
-        if not cell_name:
-            msg = _("Cell name cannot be empty")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
-        if '!' in cell_name or '.' in cell_name:
-            msg = _("Cell name cannot contain '!' or '.'")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
-
-    def _validate_cell_type(self, cell_type):
-        """Validate cell_type is 'parent' or 'child'."""
-        if cell_type not in ['parent', 'child']:
-            msg = _("Cell type must be 'parent' or 'child'")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
-
     def _normalize_cell(self, cell, existing=None):
         """Normalize input cell data.  Normalizations include:
 
@@ -213,7 +198,6 @@ class CellsController(object):
 
         # Start with the cell type conversion
         if 'type' in cell:
-            self._validate_cell_type(cell['type'])
             cell['is_parent'] = cell['type'] == 'parent'
             del cell['type']
         # Avoid cell type being overwritten to 'child'
@@ -248,23 +232,20 @@ class CellsController(object):
         # Now set the transport URL
         cell['transport_url'] = str(transport_url)
 
+    # NOTE(gmann): Returns 200 for backwards compatibility but should be 201
+    # as this operation complete the creation of aggregates resource when
+    # returning a response.
     @extensions.expected_errors((400, 403, 501))
     @common.check_cells_enabled
-    @wsgi.response(201)
+    @validation.schema(cells.create)
     def create(self, req, body):
         """Create a child cell entry."""
         context = req.environ['nova.context']
+
         authorize(context)
-        if 'cell' not in body:
-            msg = _("No cell information in request")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
+        authorize(context, action="create")
+
         cell = body['cell']
-        if 'name' not in cell:
-            msg = _("No cell name in request")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
-        self._validate_cell_name(cell['name'])
         self._normalize_cell(cell)
         try:
             cell = self.cells_rpcapi.cell_create(context, cell)
@@ -274,18 +255,17 @@ class CellsController(object):
 
     @extensions.expected_errors((400, 403, 404, 501))
     @common.check_cells_enabled
+    @validation.schema(cells.update)
     def update(self, req, id, body):
         """Update a child cell entry.  'id' is the cell name to update."""
         context = req.environ['nova.context']
+
         authorize(context)
-        if 'cell' not in body:
-            msg = _("No cell information in request")
-            LOG.error(msg)
-            raise exc.HTTPBadRequest(explanation=msg)
+        authorize(context, action="update")
+
         cell = body['cell']
         cell.pop('id', None)
-        if 'name' in cell:
-            self._validate_cell_name(cell['name'])
+
         try:
             # NOTE(Vek): There is a race condition here if multiple
             #            callers are trying to update the cell
@@ -305,31 +285,24 @@ class CellsController(object):
             raise exc.HTTPForbidden(explanation=e.format_message())
         return dict(cell=_scrub_cell(cell))
 
+    # NOTE(gmann): Returns 200 for backwards compatibility but should be 204
+    # as this operation complete the sync instance info and return
+    # no response body.
     @extensions.expected_errors((400, 501))
     @common.check_cells_enabled
-    @wsgi.response(204)
+    @validation.schema(cells.sync_instances)
     def sync_instances(self, req, body):
         """Tell all cells to sync instance info."""
         context = req.environ['nova.context']
+
         authorize(context)
+        authorize(context, action="sync_instances")
+
         project_id = body.pop('project_id', None)
         deleted = body.pop('deleted', False)
         updated_since = body.pop('updated_since', None)
-        if body:
-            msg = _("Only 'updated_since', 'project_id' and 'deleted' are "
-                    "understood.")
-            raise exc.HTTPBadRequest(explanation=msg)
         if isinstance(deleted, six.string_types):
-            try:
-                deleted = strutils.bool_from_string(deleted, strict=True)
-            except ValueError as err:
-                raise exc.HTTPBadRequest(explanation=str(err))
-        if updated_since:
-            try:
-                timeutils.parse_isotime(updated_since)
-            except ValueError:
-                msg = _('Invalid changes-since value')
-                raise exc.HTTPBadRequest(explanation=msg)
+            deleted = strutils.bool_from_string(deleted, strict=True)
         self.cells_rpcapi.sync_instances(context, project_id=project_id,
                 updated_since=updated_since, deleted=deleted)
 
