@@ -14,6 +14,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import re
+
 import webob
 
 from nova.api.openstack import common
@@ -23,7 +25,7 @@ from nova.api.openstack import wsgi
 from nova.api import validation
 from nova import compute
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 
 ALIAS = "os-console-output"
 authorize = extensions.extension_authorizer('compute', "v3:" + ALIAS)
@@ -35,26 +37,41 @@ class ConsoleOutputController(wsgi.Controller):
         self.compute_api = compute.API()
 
     @extensions.expected_errors((400, 404, 409, 501))
-    @wsgi.action('get_console_output')
+    @wsgi.action('os-getConsoleOutput')
     @validation.schema(console_output.get_console_output)
     def get_console_output(self, req, id, body):
         """Get text console output."""
         context = req.environ['nova.context']
         authorize(context)
 
-        instance = common.get_instance(self.compute_api, context, id)
-        length = body['get_console_output'].get('length')
+        instance = common.get_instance(self.compute_api, context, id,
+                                       want_objects=True)
+        length = body['os-getConsoleOutput'].get('length')
+        # TODO(cyeoh): In a future API update accept a length of -1
+        # as meaning unlimited length (convert to None)
 
         try:
             output = self.compute_api.get_console_output(context,
                                                          instance,
                                                          length)
+        # NOTE(cyeoh): This covers race conditions where the instance is
+        # deleted between common.get_instance and get_console_output
+        # being called
+        except exception.InstanceNotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.format_message())
         except exception.InstanceNotReady as e:
             raise webob.exc.HTTPConflict(explanation=e.format_message())
         except NotImplementedError:
             msg = _("Unable to get console log, functionality not implemented")
             raise webob.exc.HTTPNotImplemented(explanation=msg)
 
+        # XML output is not correctly escaped, so remove invalid characters
+        # NOTE(cyeoh): We don't support XML output with V2.1, but for
+        # backwards compatibility reasons we continue to filter the output
+        # We should remove this in the future
+        remove_re = re.compile('[\x00-\x08\x0B-\x1F]')
+        output = remove_re.sub('', output)
+
         return {'output': output}
 
 
