@@ -16,17 +16,15 @@
 import os.path
 
 import webob
-from webob import exc
 
 from nova.api.openstack import common
+from nova.api.openstack.compute.schemas.v3 import create_backup
 from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
+from nova.api import validation
 from nova import compute
 from nova import exception
-from nova.openstack.common.gettextutils import _
-from nova.openstack.common import log as logging
 
-LOG = logging.getLogger(__name__)
 ALIAS = "os-create-backup"
 authorize = extensions.extension_authorizer('compute', "v3:" + ALIAS)
 
@@ -36,8 +34,9 @@ class CreateBackupController(wsgi.Controller):
         super(CreateBackupController, self).__init__(*args, **kwargs)
         self.compute_api = compute.API()
 
-    @extensions.expected_errors((400, 404, 409, 413))
-    @wsgi.action('create_backup')
+    @extensions.expected_errors((400, 403, 404, 409))
+    @wsgi.action('createBackup')
+    @validation.schema(create_backup.create_backup)
     def _create_backup(self, req, id, body):
         """Backup a server instance.
 
@@ -51,39 +50,16 @@ class CreateBackupController(wsgi.Controller):
         """
         context = req.environ["nova.context"]
         authorize(context)
-        entity = body["create_backup"]
+        entity = body["createBackup"]
 
-        try:
-            image_name = entity["name"]
-            backup_type = entity["backup_type"]
-            rotation = entity["rotation"]
-
-        except KeyError as missing_key:
-            msg = _("create_backup entity requires %s attribute") % missing_key
-            raise exc.HTTPBadRequest(explanation=msg)
-
-        except TypeError:
-            msg = _("Malformed create_backup entity")
-            raise exc.HTTPBadRequest(explanation=msg)
-
-        try:
-            rotation = int(rotation)
-        except ValueError:
-            msg = _("create_backup attribute 'rotation' must be an integer")
-            raise exc.HTTPBadRequest(explanation=msg)
-        if rotation < 0:
-            msg = _("create_backup attribute 'rotation' must be greater "
-                    "than or equal to zero")
-            raise exc.HTTPBadRequest(explanation=msg)
+        image_name = entity["name"]
+        backup_type = entity["backup_type"]
+        rotation = int(entity["rotation"])
 
         props = {}
         metadata = entity.get('metadata', {})
         common.check_img_metadata_properties_quota(context, metadata)
-        try:
-            props.update(metadata)
-        except ValueError:
-            msg = _("Invalid metadata")
-            raise exc.HTTPBadRequest(explanation=msg)
+        props.update(metadata)
 
         instance = common.get_instance(self.compute_api, context, id,
                                        want_objects=True)
@@ -93,7 +69,9 @@ class CreateBackupController(wsgi.Controller):
                     backup_type, rotation, extra_properties=props)
         except exception.InstanceInvalidState as state_error:
             common.raise_http_conflict_for_instance_invalid_state(state_error,
-                    'create_backup')
+                    'createBackup')
+        except exception.InvalidRequest as e:
+            raise webob.exc.HTTPBadRequest(explanation=e.format_message())
 
         resp = webob.Response(status_int=202)
 
@@ -111,7 +89,6 @@ class CreateBackup(extensions.V3APIExtensionBase):
 
     name = "CreateBackup"
     alias = ALIAS
-    namespace = "http://docs.openstack.org/compute/ext/%s/api/v3" % ALIAS
     version = 1
 
     def get_controller_extensions(self):
