@@ -12,9 +12,11 @@
 
 import webob
 
+from nova.api.openstack.compute.schemas.v3 import flavor_manage
 from nova.api.openstack.compute.views import flavors as flavors_view
 from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
+from nova.api import validation
 from nova.compute import flavors
 from nova import exception
 
@@ -44,29 +46,31 @@ class FlavorManageController(wsgi.Controller):
 
         flavors.destroy(flavor['name'])
 
-        return webob.Response(status_int=204)
+        # NOTE(oomichi): Return 202 for backwards compatibility but should be
+        # 204 as this operation complete the deletion of aggregate resource and
+        # return no response body.
+        return webob.Response(status_int=202)
 
-    @wsgi.response(201)
+    # NOTE(oomichi): Return 200 for backwards compatibility but should be 201
+    # as this operation complete the creation of flavor resource.
     @wsgi.action("create")
-    @extensions.expected_errors((400, 409))
+    @extensions.expected_errors((400, 409, 500))
+    @validation.schema(flavor_manage.create)
     def _create(self, req, body):
         context = req.environ['nova.context']
         authorize(context)
 
-        if not self.is_valid_body(body, 'flavor'):
-            raise webob.exc.HTTPBadRequest('Invalid request body ')
-
         vals = body['flavor']
 
-        name = vals.get('name')
+        name = vals['name']
         flavorid = vals.get('id')
-        memory = vals.get('ram')
-        vcpus = vals.get('vcpus')
-        root_gb = vals.get('disk')
-        ephemeral_gb = vals.get('ephemeral', 0)
+        memory = vals['ram']
+        vcpus = vals['vcpus']
+        root_gb = vals['disk']
+        ephemeral_gb = vals.get('OS-FLV-EXT-DATA:ephemeral', 0)
         swap = vals.get('swap', 0)
-        rxtx_factor = vals.get('os-flavor-rxtx:rxtx_factor', 1.0)
-        is_public = vals.get('flavor-access:is_public', True)
+        rxtx_factor = vals.get('rxtx_factor', 1.0)
+        is_public = vals.get('os-flavor-access:is_public', True)
 
         try:
             flavor = flavors.create(name, memory, vcpus, root_gb,
@@ -74,15 +78,15 @@ class FlavorManageController(wsgi.Controller):
                                     flavorid=flavorid, swap=swap,
                                     rxtx_factor=rxtx_factor,
                                     is_public=is_public)
-            if not flavor['is_public']:
-                flavors.add_flavor_access(flavor['flavorid'],
-                                          context.project_id, context)
+            # NOTE(gmann): For backward compatibility, non public flavor
+            # access is not being added for created tenant. Ref -bug/1209101
             req.cache_db_flavor(flavor)
         except (exception.FlavorExists,
                 exception.FlavorIdExists) as err:
             raise webob.exc.HTTPConflict(explanation=err.format_message())
-        except exception.InvalidInput as exc:
-            raise webob.exc.HTTPBadRequest(explanation=exc.format_message())
+        except exception.FlavorCreateFailed as err:
+            raise webob.exc.HTTPInternalServerError(explanation=
+                err.format_message())
 
         return self._view_builder.show(req, flavor)
 
