@@ -27,16 +27,10 @@ CONF.import_opt('enabled', 'nova.api.openstack', group='osapi_v3')
 
 LINKS = {
    'v2.0': {
-       'pdf': 'http://docs.openstack.org/'
-               'api/openstack-compute/2/os-compute-devguide-2.pdf',
-       'wadl': 'http://docs.openstack.org/'
-               'api/openstack-compute/2/wadl/os-compute-2.wadl'
+       'html': 'http://docs.openstack.org/'
     },
-   'v3.0': {
-       'pdf': 'http://docs.openstack.org/'
-               'api/openstack-compute/3/os-compute-devguide-3.pdf',
-       'wadl': 'http://docs.openstack.org/'
-               'api/openstack-compute/3/wadl/os-compute-3.wadl'
+   'v2.1': {
+       'html': 'http://docs.openstack.org/'
     },
 }
 
@@ -49,13 +43,8 @@ VERSIONS = {
         "links": [
             {
                 "rel": "describedby",
-                "type": "application/pdf",
-                "href": LINKS['v2.0']['pdf'],
-            },
-            {
-                "rel": "describedby",
-                "type": "application/vnd.sun.wadl+xml",
-                "href": LINKS['v2.0']['wadl'],
+                "type": "text/html",
+                "href": LINKS['v2.0']['html'],
             },
         ],
         "media-types": [
@@ -69,26 +58,21 @@ VERSIONS = {
             }
         ],
     },
-    "v3.0": {
-        "id": "v3.0",
+    "v2.1": {
+        "id": "v2.1",
         "status": "EXPERIMENTAL",
         "updated": "2013-07-23T11:33:21Z",
         "links": [
             {
                 "rel": "describedby",
-                "type": "application/pdf",
-                "href": LINKS['v3.0']['pdf'],
-            },
-            {
-                "rel": "describedby",
-                "type": "application/vnd.sun.wadl+xml",
-                "href": LINKS['v3.0']['wadl'],
+                "type": "text/html",
+                "href": LINKS['v2.1']['html'],
             },
         ],
         "media-types": [
             {
                 "base": "application/json",
-                "type": "application/vnd.openstack.compute+json;version=3",
+                "type": "application/vnd.openstack.compute+json;version=2.1",
             }
         ],
     }
@@ -237,7 +221,7 @@ class Versions(wsgi.Resource):
     def __init__(self):
         super(Versions, self).__init__(None)
         if not CONF.osapi_v3.enabled:
-            del VERSIONS["v3.0"]
+            del VERSIONS["v2.1"]
 
     @wsgi.serializers(xml=VersionsTemplate,
                       atom=VersionsAtomSerializer)
