@@ -16,6 +16,7 @@
 
 from nova.api.openstack import common
 from nova.image import glance
+from nova.openstack.common import timeutils
 
 
 class ViewBuilder(common.ViewBuilder):
@@ -122,10 +123,10 @@ class ViewBuilder(common.ViewBuilder):
                          str(identifier)])
 
     @staticmethod
-    def _format_date(date_string):
-        """Return standard format for given date."""
-        if date_string is not None:
-            return date_string.strftime('%Y-%m-%dT%H:%M:%SZ')
+    def _format_date(dt):
+        """Return standard format for a given datetime object."""
+        if dt is not None:
+            return timeutils.isotime(dt)
 
     @staticmethod
     def _get_status(image):
@@ -146,20 +147,3 @@ class ViewBuilder(common.ViewBuilder):
             "saving": 50,
             "active": 100,
         }.get(image.get("status"), 0)
-
-
-class ViewBuilderV3(ViewBuilder):
-
-    def _get_bookmark_link(self, request, identifier, collection_name):
-        """Create a URL that refers to a specific resource."""
-        if collection_name == "images":
-            glance_url = glance.generate_image_url(identifier)
-            return self._update_glance_link_prefix(glance_url)
-        else:
-            raise NotImplementedError
-            # NOTE(cyeoh) The V3 version of _get_bookmark_link should
-            # only ever be called with images as the
-            # collection_name. The images API has been removed in the
-            # V3 API and the V3 version of the view only exists for
-            # the servers view to be able to generate the appropriate
-            # bookmark link for the image of the instance.
