@@ -26,7 +26,8 @@ import nova.api.openstack
 from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova.i18n import _LW
 from nova.openstack.common import importutils
 from nova.openstack.common import log as logging
 import nova.policy
@@ -55,7 +56,7 @@ class ExtensionDescriptor(object):
     namespace = None
 
     # The timestamp when the extension was last updated, e.g.,
-    # '2011-01-22T13:25:27-06:00'
+    # '2011-01-22T19:25:27Z'
     updated = None
 
     def __init__(self, ext_mgr):
@@ -174,7 +175,7 @@ class ExtensionsController(wsgi.Resource):
 class ExtensionManager(object):
     """Load extensions from the configured extension path.
 
-    See nova/tests/api/openstack/volume/extensions/foxinsocks.py or an
+    See nova/tests/api/openstack/compute/extensions/foxinsocks.py or an
     example extension implementation.
 
     """
@@ -194,8 +195,6 @@ class ExtensionManager(object):
             return
 
         alias = ext.alias
-        LOG.audit(_('Loaded extension: %s'), alias)
-
         if alias in self.extensions:
             raise exception.NovaException("Found duplicate extension: %s"
                                           % alias)
@@ -233,12 +232,12 @@ class ExtensionManager(object):
     def _check_extension(self, extension):
         """Checks for required methods in extension objects."""
         try:
-            LOG.debug(_('Ext name: %s'), extension.name)
-            LOG.debug(_('Ext alias: %s'), extension.alias)
-            LOG.debug(_('Ext description: %s'),
+            LOG.debug('Ext name: %s', extension.name)
+            LOG.debug('Ext alias: %s', extension.alias)
+            LOG.debug('Ext description: %s',
                       ' '.join(extension.__doc__.strip().split()))
-            LOG.debug(_('Ext namespace: %s'), extension.namespace)
-            LOG.debug(_('Ext updated: %s'), extension.updated)
+            LOG.debug('Ext namespace: %s', extension.namespace)
+            LOG.debug('Ext updated: %s', extension.updated)
         except AttributeError as ex:
             LOG.exception(_("Exception loading extension: %s"), unicode(ex))
             return False
@@ -254,7 +253,7 @@ class ExtensionManager(object):
         expected to call the register() method at least once.
         """
 
-        LOG.debug(_("Loading extension %s"), ext_factory)
+        LOG.debug("Loading extension %s", ext_factory)
 
         if isinstance(ext_factory, six.string_types):
             # Load the factory
@@ -263,7 +262,7 @@ class ExtensionManager(object):
             factory = ext_factory
 
         # Call it
-        LOG.debug(_("Calling extension factory %s"), ext_factory)
+        LOG.debug("Calling extension factory %s", ext_factory)
         factory(self)
 
     def _load_extensions(self):
@@ -275,8 +274,8 @@ class ExtensionManager(object):
             try:
                 self.load_extension(ext_factory)
             except Exception as exc:
-                LOG.warn(_('Failed to load extension %(ext_factory)s: '
-                           '%(exc)s'),
+                LOG.warn(_LW('Failed to load extension %(ext_factory)s: '
+                             '%(exc)s'),
                          {'ext_factory': ext_factory, 'exc': exc})
 
 
@@ -394,18 +393,26 @@ def extension_authorizer(api_name, extension_name):
     return core_authorizer('%s_extension' % api_name, extension_name)
 
 
-def soft_extension_authorizer(api_name, extension_name):
-    hard_authorize = extension_authorizer(api_name, extension_name)
+def soft_authorizer(hard_authorizer, api_name, extension_name):
+    hard_authorize = hard_authorizer(api_name, extension_name)
 
     def authorize(context, action=None):
         try:
             hard_authorize(context, action=action)
             return True
-        except exception.NotAuthorized:
+        except exception.Forbidden:
             return False
     return authorize
 
 
+def soft_extension_authorizer(api_name, extension_name):
+    return soft_authorizer(extension_authorizer, api_name, extension_name)
+
+
+def soft_core_authorizer(api_name, extension_name):
+    return soft_authorizer(core_authorizer, api_name, extension_name)
+
+
 def check_compute_policy(context, action, target, scope='compute'):
     _action = '%s:%s' % (scope, action)
     nova.policy.enforce(context, _action, target)
