@@ -14,16 +14,16 @@
 #    under the License.
 
 import os.path
-
-from lxml import etree
-import six
 from xml.dom import minidom
 from xml.parsers import expat
 from xml import sax
 from xml.sax import expatreader
 
+from lxml import etree
+import six
+
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova import utils
 
 
@@ -411,7 +411,7 @@ class TemplateElement(object):
         # We have fully rendered the element; return it
         return elem
 
-    def render(self, parent, obj, patches=[], nsmap=None):
+    def render(self, parent, obj, patches=None, nsmap=None):
         """Render an object.
 
         Renders an object against this template node.  Returns a list
@@ -428,6 +428,7 @@ class TemplateElement(object):
                       the etree.Element instances.
         """
 
+        patches = patches or []
         # First, get the datum we're rendering
         data = None if obj is None else self.selector(obj)
 
@@ -989,7 +990,7 @@ def safe_minidom_parse_string(xml_string):
         return minidom.parseString(xml_string, parser=ProtectedExpatParser())
     except (sax.SAXParseException, ValueError,
             expat.ExpatError, LookupError) as e:
-        #NOTE(Vijaya Erukala): XML input such as
+        # NOTE(Vijaya Erukala): XML input such as
         #                      <?xml version="1.0" encoding="TF-8"?>
         #                      raises LookupError: unknown encoding: TF-8
-        raise exception.MalformedRequestBody(reason=str(e))
+        raise exception.MalformedRequestBody(reason=six.text_type(e))
