@@ -14,32 +14,16 @@
 #    under the License.
 
 import base64
-import re
 
+import rfc3986
 import six
 
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _get_path_validator_regex():
-    # rfc3986 path validator regex from
-    # http://jmrware.com/articles/2009/uri_regexp/URI_regex.html
-    pchar = "([A-Za-z0-9\-._~!$&'()*+,;=:@]|%[0-9A-Fa-f]{2})"
-    path = "((/{pchar}*)*|"
-    path += "/({pchar}+(/{pchar}*)*)?|"
-    path += "{pchar}+(/{pchar}*)*|"
-    path += "{pchar}+(/{pchar}*)*|)"
-    path = path.format(pchar=pchar)
-    return re.compile(path)
-
-
-VALIDATE_PATH_RE = _get_path_validator_regex()
-
-
 def validate_str(max_length=None):
 
     def _do(val):
@@ -70,7 +54,9 @@ def validate_url_path(val):
     if not validate_str()(val):
         return False
 
-    return VALIDATE_PATH_RE.match(val).end() == len(val)
+    uri = rfc3986.URIReference(None, None, val, None, None)
+
+    return uri.path_is_valid() and val.startswith('/')
 
 
 def validate_image_path(val):
@@ -128,8 +114,8 @@ def validate(args, validator):
         assert callable(f)
 
         if not f(args[key]):
-            LOG.debug(_("%(key)s with value %(value)s failed"
-                        " validator %(name)s"),
+            LOG.debug("%(key)s with value %(value)s failed"
+                      " validator %(name)s",
                       {'key': key, 'value': args[key], 'name': f.__name__})
             return False
     return True
