@@ -19,15 +19,17 @@ CellState Manager
 import copy
 import datetime
 import functools
+import time
 
 from oslo.config import cfg
+from oslo.db import exception as db_exc
 
 from nova.cells import rpc_driver
 from nova import context
 from nova.db import base
 from nova import exception
+from nova.i18n import _
 from nova.openstack.common import fileutils
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
@@ -36,14 +38,14 @@ from nova import rpc
 from nova import utils
 
 cell_state_manager_opts = [
-        cfg.IntOpt('db_check_interval',
-                default=60,
-                help='Interval, in seconds, for getting fresh cell '
-                   'information from the database.'),
-        cfg.StrOpt('cells_config',
-                   help='Configuration file from which to read cells '
-                   'configuration.  If given, overrides reading cells '
-                   'from the database.'),
+    cfg.IntOpt('db_check_interval',
+               default=60,
+               help='Interval, in seconds, for getting fresh cell '
+               'information from the database.'),
+    cfg.StrOpt('cells_config',
+               help='Configuration file from which to read cells '
+               'configuration.  If given, overrides reading cells '
+               'from the database.'),
 ]
 
 
@@ -53,7 +55,6 @@ CONF = cfg.CONF
 CONF.import_opt('name', 'nova.cells.opts', group='cells')
 CONF.import_opt('reserve_percent', 'nova.cells.opts', group='cells')
 CONF.import_opt('mute_child_interval', 'nova.cells.opts', group='cells')
-#CONF.import_opt('capabilities', 'nova.cells.opts', group='cells')
 CONF.register_opts(cell_state_manager_opts, group='cells')
 
 
@@ -166,7 +167,17 @@ class CellStateManager(base.Base):
         self.child_cells = {}
         self.last_cell_db_check = datetime.datetime.min
 
-        self._cell_data_sync(force=True)
+        attempts = 0
+        while True:
+            try:
+                self._cell_data_sync(force=True)
+                break
+            except db_exc.DBError as e:
+                attempts += 1
+                if attempts > 120:
+                    raise
+                LOG.exception(_('DB error: %s') % e)
+                time.sleep(30)
 
         my_cell_capabs = {}
         for cap in CONF.cells.capabilities:
@@ -425,7 +436,7 @@ class CellStateManagerDB(CellStateManager):
                       of whether it's time to do so.
         """
         if force or self._time_to_sync():
-            LOG.debug(_("Updating cell cache from db."))
+            LOG.debug("Updating cell cache from db.")
             self.last_cell_db_check = timeutils.utcnow()
             ctxt = context.get_admin_context()
             db_cells = self.db.cell_get_all(ctxt)
@@ -465,7 +476,7 @@ class CellStateManagerFile(CellStateManager):
                                                     force_reload=force)
 
         if reloaded:
-            LOG.debug(_("Updating cell cache from config file."))
+            LOG.debug("Updating cell cache from config file.")
             self.cells_config_data = jsonutils.loads(data)
             self._refresh_cells_from_dict(self.cells_config_data)
 
