@@ -20,6 +20,7 @@ import sys
 from oslo.config import cfg
 
 from nova import config
+from nova import objects
 from nova.openstack.common import log as logging
 from nova.openstack.common.report import guru_meditation_report as gmr
 from nova import service
@@ -35,6 +36,7 @@ def main():
     config.parse_args(sys.argv)
     logging.setup('nova')
     utils.monkey_patch()
+    objects.register_all()
 
     gmr.TextGuruMeditation.setup_autorun(version)
 
