@@ -20,6 +20,7 @@ import sys
 from oslo.config import cfg
 
 from nova import config
+from nova import objects
 from nova.openstack.common import log as logging
 from nova.openstack.common.report import guru_meditation_report as gmr
 from nova import service
@@ -31,6 +32,7 @@ CONF = cfg.CONF
 def main():
     config.parse_args(sys.argv)
     logging.setup("nova")
+    objects.register_all()
 
     gmr.TextGuruMeditation.setup_autorun(version)
 
