@@ -22,23 +22,28 @@ from __future__ import print_function
 
 import os
 import sys
+import traceback
 
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import importutils
+from oslo.config import cfg
 
-from nova.cmd import common as cmd_common
 from nova.conductor import rpcapi as conductor_rpcapi
-import nova.conf
 from nova import config
 from nova import context
+import nova.db.api
+from nova import exception
+from nova.i18n import _
 from nova.network import rpcapi as network_rpcapi
 from nova import objects
 from nova.objects import base as objects_base
+from nova.openstack.common import importutils
+from nova.openstack.common import jsonutils
+from nova.openstack.common import log as logging
 from nova import rpc
 
-CONF = nova.conf.CONF
+CONF = cfg.CONF
+CONF.import_opt('host', 'nova.netconf')
+CONF.import_opt('network_manager', 'nova.service')
+CONF.import_opt('use_local', 'nova.conductor.api', group='conductor')
 LOG = logging.getLogger(__name__)
 
 
@@ -60,7 +65,7 @@ def del_lease(mac, ip_address):
     """Called when a lease expires."""
     api = network_rpcapi.NetworkAPI()
     api.release_fixed_ip(context.get_admin_context(), ip_address,
-                         CONF.host, mac)
+                         CONF.host)
 
 
 def init_leases(network_id):
@@ -72,23 +77,18 @@ def init_leases(network_id):
 
 
 def add_action_parsers(subparsers):
-    subparsers.add_parser('init')
+    parser = subparsers.add_parser('init')
 
     # NOTE(cfb): dnsmasq always passes mac, and ip. hostname
     #            is passed if known. We don't care about
     #            hostname, but argparse will complain if we
     #            do not accept it.
-    actions = {
-        'add': add_lease,
-        'del': del_lease,
-        'old': old_lease,
-    }
-    for action, func in actions.items():
+    for action in ['add', 'del', 'old']:
         parser = subparsers.add_parser(action)
         parser.add_argument('mac')
         parser.add_argument('ip')
         parser.add_argument('hostname', nargs='?', default='')
-        parser.set_defaults(func=func)
+        parser.set_defaults(func=globals()[action + '_lease'])
 
 
 CONF.register_cli_opt(
@@ -98,39 +98,48 @@ CONF.register_cli_opt(
                       handler=add_action_parsers))
 
 
+def block_db_access():
+    class NoDB(object):
+        def __getattr__(self, attr):
+            return self
+
+        def __call__(self, *args, **kwargs):
+            stacktrace = "".join(traceback.format_stack())
+            LOG.error(_('No db access allowed in nova-dhcpbridge: %s'),
+                      stacktrace)
+            raise exception.DBNotAllowed('nova-dhcpbridge')
+
+    nova.db.api.IMPL = NoDB()
+
+
 def main():
     """Parse environment and arguments and call the appropriate action."""
     config.parse_args(sys.argv,
         default_config_files=jsonutils.loads(os.environ['CONFIG_FILE']))
 
-    logging.setup(CONF, "nova")
+    logging.setup("nova")
     global LOG
     LOG = logging.getLogger('nova.dhcpbridge')
-
-    if CONF.action.name == 'old':
-        # NOTE(sdague): old is the most frequent message sent, and
-        # it's a noop. We should just exit immediately otherwise we
-        # can stack up a bunch of requests in dnsmasq. A SIGHUP seems
-        # to dump this list, so actions queued up get lost.
-        return
-
     objects.register_all()
 
-    cmd_common.block_db_access('nova-dhcpbridge')
-    objects_base.NovaObject.indirection_api = conductor_rpcapi.ConductorAPI()
+    if not CONF.conductor.use_local:
+        block_db_access()
+        objects_base.NovaObject.indirection_api = \
+            conductor_rpcapi.ConductorAPI()
 
-    if CONF.action.name in ['add', 'del']:
-        LOG.debug("Called '%(action)s' for mac '%(mac)s' with IP '%(ip)s'",
-                  {"action": CONF.action.name,
-                   "mac": CONF.action.mac,
-                   "ip": CONF.action.ip})
+    if CONF.action.name in ['add', 'del', 'old']:
+        msg = (_("Called '%(action)s' for mac '%(mac)s' with ip '%(ip)s'") %
+               {"action": CONF.action.name,
+                "mac": CONF.action.mac,
+                "ip": CONF.action.ip})
+        LOG.debug(msg)
         CONF.action.func(CONF.action.mac, CONF.action.ip)
     else:
         try:
             network_id = int(os.environ.get('NETWORK_ID'))
         except TypeError:
-            LOG.error("Environment variable 'NETWORK_ID' must be set.")
-            return 1
+            LOG.error(_("Environment variable 'NETWORK_ID' must be set."))
+            return(1)
 
         print(init_leases(network_id))
 
