@@ -25,8 +25,9 @@ from nova.conductor import rpcapi as conductor_rpcapi
 from nova import config
 import nova.db.api
 from nova import exception
+from nova.i18n import _
+from nova import objects
 from nova.objects import base as objects_base
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 from nova.openstack.common.report import guru_meditation_report as gmr
 from nova import service
@@ -57,6 +58,7 @@ def main():
     config.parse_args(sys.argv)
     logging.setup("nova")
     utils.monkey_patch()
+    objects.register_all()
 
     gmr.TextGuruMeditation.setup_autorun(version)
 
