@@ -17,12 +17,14 @@
 Claim objects for use with resource tracking.
 """
 
+from nova import context
 from nova import exception
-from nova.objects import instance as instance_obj
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova import objects
+from nova.objects import base as obj_base
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
-from nova.pci import pci_request
+from nova.virt import hardware
 
 
 LOG = logging.getLogger(__name__)
@@ -33,6 +35,7 @@ class NopClaim(object):
 
     def __init__(self, migration=None):
         self.migration = migration
+        self.claimed_numa_topology = None
 
     @property
     def disk_gb(self):
@@ -42,10 +45,6 @@ class NopClaim(object):
     def memory_mb(self):
         return 0
 
-    @property
-    def vcpus(self):
-        return 0
-
     def __enter__(self):
         return self
 
@@ -57,8 +56,8 @@ class NopClaim(object):
         pass
 
     def __str__(self):
-        return "[Claim: %d MB memory, %d GB disk, %d VCPUS]" % (self.memory_mb,
-                self.disk_gb, self.vcpus)
+        return "[Claim: %d MB memory, %d GB disk]" % (self.memory_mb,
+                self.disk_gb)
 
 
 class Claim(NopClaim):
@@ -72,25 +71,27 @@ class Claim(NopClaim):
     correct decisions with respect to host selection.
     """
 
-    def __init__(self, instance, tracker, resources, overhead=None,
+    def __init__(self, context, instance, tracker, resources, overhead=None,
                  limits=None):
         super(Claim, self).__init__()
         # Stash a copy of the instance at the current point of time
-        if isinstance(instance, instance_obj.Instance):
+        if isinstance(instance, obj_base.NovaObject):
             self.instance = instance.obj_clone()
         else:
             # This does not use copy.deepcopy() because it could be
             # a sqlalchemy model, and it's best to make sure we have
             # the primitive form.
             self.instance = jsonutils.to_primitive(instance)
+        self._numa_topology_loaded = False
         self.tracker = tracker
 
         if not overhead:
             overhead = {'memory_mb': 0}
 
         self.overhead = overhead
+        self.context = context
 
-        # Check claim at constuctor to avoid mess code
+        # Check claim at constructor to avoid mess code
         # Raise exception ComputeResourcesUnavailable if claim failed
         self._claim_test(resources, limits)
 
@@ -103,15 +104,29 @@ class Claim(NopClaim):
         return self.instance['memory_mb'] + self.overhead['memory_mb']
 
     @property
-    def vcpus(self):
-        return self.instance['vcpus']
+    def numa_topology(self):
+        if self._numa_topology_loaded:
+            return self._numa_topology
+        else:
+            if isinstance(self.instance, obj_base.NovaObject):
+                self._numa_topology = self.instance.numa_topology
+            else:
+                try:
+                    self._numa_topology = (
+                        objects.InstanceNUMATopology.get_by_instance_uuid(
+                            context.get_admin_context(), self.instance['uuid'])
+                        )
+                except exception.NumaTopologyNotFound:
+                    self._numa_topology = None
+            self._numa_topology_loaded = True
+            return self._numa_topology
 
     def abort(self):
         """Compute operation requiring claimed resources has failed or
         been aborted.
         """
-        LOG.debug(_("Aborting claim: %s") % self, instance=self.instance)
-        self.tracker.abort_instance_claim(self.instance)
+        LOG.debug("Aborting claim: %s" % self, instance=self.instance)
+        self.tracker.abort_instance_claim(self.context, self.instance)
 
     def _claim_test(self, resources, limits=None):
         """Test if this claim can be satisfied given available resources and
@@ -130,18 +145,21 @@ class Claim(NopClaim):
         # unlimited:
         memory_mb_limit = limits.get('memory_mb')
         disk_gb_limit = limits.get('disk_gb')
-        vcpu_limit = limits.get('vcpu')
+        numa_topology_limit = limits.get('numa_topology')
+        if numa_topology_limit:
+            numa_topology_limit = hardware.VirtNUMALimitTopology.from_json(
+                numa_topology_limit)
 
         msg = _("Attempting claim: memory %(memory_mb)d MB, disk %(disk_gb)d "
-                "GB, VCPUs %(vcpus)d")
-        params = {'memory_mb': self.memory_mb, 'disk_gb': self.disk_gb,
-                  'vcpus': self.vcpus}
+                "GB")
+        params = {'memory_mb': self.memory_mb, 'disk_gb': self.disk_gb}
         LOG.audit(msg % params, instance=self.instance)
 
         reasons = [self._test_memory(resources, memory_mb_limit),
                    self._test_disk(resources, disk_gb_limit),
-                   self._test_cpu(resources, vcpu_limit),
+                   self._test_numa_topology(resources, numa_topology_limit),
                    self._test_pci()]
+        reasons = reasons + self._test_ext_resources(limits)
         reasons = [r for r in reasons if r is not None]
         if len(reasons) > 0:
             raise exception.ComputeResourcesUnavailable(reason=
@@ -168,22 +186,37 @@ class Claim(NopClaim):
         return self._test(type_, unit, total, used, requested, limit)
 
     def _test_pci(self):
-        pci_requests = pci_request.get_instance_pci_requests(self.instance)
+        pci_requests = objects.InstancePCIRequests.get_by_instance_uuid(
+            self.context, self.instance['uuid'])
 
-        if pci_requests:
+        if pci_requests.requests:
             can_claim = self.tracker.pci_tracker.stats.support_requests(
-                pci_requests)
+                pci_requests.requests)
             if not can_claim:
                 return _('Claim pci failed.')
 
-    def _test_cpu(self, resources, limit):
-        type_ = _("CPUs")
-        unit = "VCPUs"
-        total = resources['vcpus']
-        used = resources['vcpus_used']
-        requested = self.vcpus
+    def _test_ext_resources(self, limits):
+        return self.tracker.ext_resources_handler.test_resources(
+            self.instance, limits)
 
-        return self._test(type_, unit, total, used, requested, limit)
+    def _test_numa_topology(self, resources, limit):
+        host_topology = resources.get('numa_topology')
+        requested_topology = (self.numa_topology and
+                                self.numa_topology.topology_from_obj())
+        if host_topology:
+            host_topology = hardware.VirtNUMAHostTopology.from_json(
+                    host_topology)
+            instance_topology = (
+                    hardware.VirtNUMAHostTopology.fit_instance_to_host(
+                        host_topology, requested_topology,
+                        limits_topology=limit))
+            if requested_topology and not instance_topology:
+                return (_("Requested instance NUMA topology cannot fit "
+                          "the given host NUMA topology"))
+            elif instance_topology:
+                self.claimed_numa_topology = (
+                        objects.InstanceNUMATopology.obj_from_topology(
+                            instance_topology))
 
     def _test(self, type_, unit, total, used, requested, limit):
         """Test if the given type of resource needed for a claim can be safely
@@ -219,11 +252,14 @@ class ResizeClaim(Claim):
     """Claim used for holding resources for an incoming resize/migration
     operation.
     """
-    def __init__(self, instance, instance_type, tracker, resources,
-                 overhead=None, limits=None):
+    def __init__(self, context, instance, instance_type, image_meta, tracker,
+                 resources, overhead=None, limits=None):
+        self.context = context
         self.instance_type = instance_type
-        super(ResizeClaim, self).__init__(instance, tracker, resources,
-                                          overhead=overhead, limits=limits)
+        self.image_meta = image_meta
+        super(ResizeClaim, self).__init__(context, instance, tracker,
+                                          resources, overhead=overhead,
+                                          limits=limits)
         self.migration = None
 
     @property
@@ -236,21 +272,33 @@ class ResizeClaim(Claim):
         return self.instance_type['memory_mb'] + self.overhead['memory_mb']
 
     @property
-    def vcpus(self):
-        return self.instance_type['vcpus']
+    def numa_topology(self):
+        instance_topology = hardware.VirtNUMAInstanceTopology.get_constraints(
+                    self.instance_type, self.image_meta)
+        if instance_topology:
+            return objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology)
 
     def _test_pci(self):
-        pci_requests = pci_request.get_instance_pci_requests(
-            self.instance, 'new_')
-        if pci_requests:
+        pci_requests = objects.InstancePCIRequests.\
+                       get_by_instance_uuid_and_newness(
+                           self.context, self.instance['uuid'], True)
+        if pci_requests.requests:
             claim = self.tracker.pci_tracker.stats.support_requests(
-                pci_requests)
+                pci_requests.requests)
             if not claim:
                 return _('Claim pci failed.')
 
+    def _test_ext_resources(self, limits):
+        return self.tracker.ext_resources_handler.test_resources(
+            self.instance_type, limits)
+
     def abort(self):
         """Compute operation requiring claimed resources has failed or
         been aborted.
         """
-        LOG.debug(_("Aborting claim: %s") % self, instance=self.instance)
-        self.tracker.drop_resize_claim(self.instance, self.instance_type)
+        LOG.debug("Aborting claim: %s" % self, instance=self.instance)
+        self.tracker.drop_resize_claim(
+            self.context,
+            self.instance, instance_type=self.instance_type,
+            image_meta=self.image_meta)
