@@ -19,6 +19,7 @@ import string
 import traceback
 
 from oslo.config import cfg
+from oslo.utils import encodeutils
 
 from nova import block_device
 from nova.compute import flavors
@@ -64,7 +65,19 @@ def exception_to_dict(fault):
     # NOTE(dripton) The message field in the database is limited to 255 chars.
     # MySQL silently truncates overly long messages, but PostgreSQL throws an
     # error if we don't truncate it.
-    u_message = unicode(message)[:255]
+    b_message = encodeutils.safe_encode(message)[:255]
+
+    # NOTE(chaochin) UTF-8 character byte size varies from 1 to 6. If
+    # truncating a long byte string to 255, the last character may be
+    # cut in the middle, so that UnicodeDecodeError will occur when
+    # converting it back to unicode.
+    decode_ok = False
+    while not decode_ok:
+        try:
+            u_message = encodeutils.safe_decode(b_message)
+            decode_ok = True
+        except UnicodeDecodeError:
+            b_message = b_message[:-1]
 
     fault_dict = dict(exception=fault)
     fault_dict["message"] = u_message
