@@ -16,21 +16,45 @@
 
 import socket
 
-from oslo_log import log as logging
+from oslo.config import cfg
 import six.moves
 
-import nova.conf
 from nova import exception
+from nova.i18n import _LW
+from nova.openstack.common import log as logging
 from nova import utils
 
 LOG = logging.getLogger(__name__)
 
 ALLOCATED_PORTS = set()  # in-memory set of already allocated ports
 SERIAL_LOCK = 'serial-lock'
+DEFAULT_PORT_RANGE = '10000:20000'
 
-CONF = nova.conf.CONF
+serial_opts = [
+    cfg.BoolOpt('enabled',
+                default=False,
+                help='Enable serial console related features'),
+    cfg.StrOpt('port_range',
+                default=DEFAULT_PORT_RANGE,
+                help='Range of TCP ports to use for serial ports '
+                     'on compute hosts'),
+    cfg.StrOpt('base_url',
+               default='ws://127.0.0.1:6083/',
+               help='Location of serial console proxy.'),
+    cfg.StrOpt('listen',
+               default='127.0.0.1',
+               help=('IP address on which instance serial console '
+                     'should listen')),
+    cfg.StrOpt('proxyclient_address',
+               default='127.0.0.1',
+               help=('The address to which proxy clients '
+                     '(like nova-serialproxy) should connect')),
+    ]
 
-# TODO(sahid): Add a method to initialize ALLOCATED_PORTS with the
+CONF = cfg.CONF
+CONF.register_opts(serial_opts, group='serial_console')
+
+# TODO(sahid): Add a method to initialize ALOCATED_PORTS with the
 # already binded TPC port(s). (cf from danpb: list all running guests and
 # query the XML in libvirt driver to find out the TCP port(s) it uses).
 
@@ -53,7 +77,7 @@ def acquire_port(host):
             ALLOCATED_PORTS.add((host, port))
             return port
         except exception.SocketPortInUseException as e:
-            LOG.warning(e.format_message())
+            LOG.warn(e)
 
     raise exception.SocketPortRangeExhaustedException(host=host)
 
@@ -66,17 +90,17 @@ def release_port(host, port):
 
 def _get_port_range():
     config_range = CONF.serial_console.port_range
-
-    start, stop = map(int, config_range.split(':'))
-    if start >= stop:
-        default_port_range = nova.conf.serial_console.DEFAULT_PORT_RANGE
-        LOG.warning("serial_console.port_range should be in the "
-                    "format <start>:<stop> and start < stop, "
-                    "Given value %(port_range)s is invalid. "
-                    "Taking the default port range %(default)s.",
-                    {'port_range': config_range,
-                     'default': default_port_range})
-        start, stop = map(int, default_port_range.split(':'))
+    try:
+        start, stop = map(int, config_range.split(':'))
+        if start >= stop:
+            raise ValueError
+    except ValueError:
+        LOG.warn(_LW("serial_console.port_range should be <num>:<num>. "
+                     "Given value %(port_range)s could not be parsed. "
+                     "Taking the default port range %(default)s."),
+                 {'port_range': config_range,
+                  'default': DEFAULT_PORT_RANGE})
+        start, stop = map(int, DEFAULT_PORT_RANGE.split(':'))
     return start, stop
 
 
