@@ -30,9 +30,10 @@ from oslo.config import cfg
 
 from nova.api.ec2 import ec2utils
 import nova.cert.rpcapi
+from nova.compute import arch
 from nova import exception
+from nova.i18n import _, _LE
 from nova.image import glance
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 from nova.openstack.common import processutils
 from nova import utils
@@ -160,7 +161,7 @@ class S3ImageService(object):
         return self._translate_uuid_to_id(context, image)
 
     def detail(self, context, **kwargs):
-        #NOTE(bcwaldon): sort asc to make sure we assign lower ids
+        # NOTE(bcwaldon): sort asc to make sure we assign lower ids
         # to older images
         kwargs.setdefault('sort_dir', 'asc')
         images = self.service.detail(context, **kwargs)
@@ -215,9 +216,13 @@ class S3ImageService(object):
             ramdisk_id = None
 
         try:
-            arch = manifest.find('machine_configuration/architecture').text
+            guestarch = manifest.find(
+                'machine_configuration/architecture').text
         except Exception:
-            arch = 'x86_64'
+            guestarch = arch.X86_64
+
+        if not arch.is_valid(guestarch):
+            raise exception.InvalidArchitectureName(arch=guestarch)
 
         # NOTE(yamahata):
         # EC2 ec2-budlne-image --block-device-mapping accepts
@@ -242,7 +247,7 @@ class S3ImageService(object):
             mappings = []
 
         properties = metadata['properties']
-        properties['architecture'] = arch
+        properties['architecture'] = guestarch
 
         def _translate_dependent_image_id(image_key, image_id):
             image_uuid = ec2utils.ec2_id_to_glance_id(context, image_id)
@@ -264,7 +269,7 @@ class S3ImageService(object):
                          'properties': properties})
         metadata['properties']['image_state'] = 'pending'
 
-        #TODO(bcwaldon): right now, this removes user-defined ids.
+        # TODO(bcwaldon): right now, this removes user-defined ids.
         # We need to re-enable this.
         metadata.pop('id', None)
 
@@ -328,8 +333,8 @@ class S3ImageService(object):
                                 shutil.copyfileobj(part, combined)
 
                 except Exception:
-                    LOG.exception(_("Failed to download %(image_location)s "
-                                    "to %(image_path)s"), log_vars)
+                    LOG.exception(_LE("Failed to download %(image_location)s "
+                                      "to %(image_path)s"), log_vars)
                     _update_image_state(context, image_uuid, 'failed_download')
                     return
 
@@ -345,8 +350,8 @@ class S3ImageService(object):
                     self._decrypt_image(context, enc_filename, encrypted_key,
                                         encrypted_iv, dec_filename)
                 except Exception:
-                    LOG.exception(_("Failed to decrypt %(image_location)s "
-                                    "to %(image_path)s"), log_vars)
+                    LOG.exception(_LE("Failed to decrypt %(image_location)s "
+                                      "to %(image_path)s"), log_vars)
                     _update_image_state(context, image_uuid, 'failed_decrypt')
                     return
 
@@ -356,8 +361,8 @@ class S3ImageService(object):
                     unz_filename = self._untarzip_image(image_path,
                                                         dec_filename)
                 except Exception:
-                    LOG.exception(_("Failed to untar %(image_location)s "
-                                    "to %(image_path)s"), log_vars)
+                    LOG.exception(_LE("Failed to untar %(image_location)s "
+                                      "to %(image_path)s"), log_vars)
                     _update_image_state(context, image_uuid, 'failed_untar')
                     return
 
@@ -366,8 +371,8 @@ class S3ImageService(object):
                     with open(unz_filename) as image_file:
                         _update_image_data(context, image_uuid, image_file)
                 except Exception:
-                    LOG.exception(_("Failed to upload %(image_location)s "
-                                    "to %(image_path)s"), log_vars)
+                    LOG.exception(_LE("Failed to upload %(image_location)s "
+                                      "to %(image_path)s"), log_vars)
                     _update_image_state(context, image_uuid, 'failed_upload')
                     return
 
