@@ -29,9 +29,9 @@ this class.
 import array
 
 from nova import exception
+from nova.i18n import _
 from nova.keymgr import key
 from nova.keymgr import key_mgr
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 from nova.openstack.common import uuidutils
 from nova import utils
@@ -72,10 +72,10 @@ class MockKeyManager(key_mgr.KeyManager):
         """Creates a key.
 
         This implementation returns a UUID for the created key. A
-        NotAuthorized exception is raised if the specified context is None.
+        Forbidden exception is raised if the specified context is None.
         """
         if ctxt is None:
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         key = self._generate_key(**kwargs)
         return self.store_key(ctxt, key)
@@ -90,7 +90,7 @@ class MockKeyManager(key_mgr.KeyManager):
     def store_key(self, ctxt, key, **kwargs):
         """Stores (i.e., registers) a key with the key manager."""
         if ctxt is None:
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         key_id = self._generate_key_id()
         self.keys[key_id] = key
@@ -99,7 +99,7 @@ class MockKeyManager(key_mgr.KeyManager):
 
     def copy_key(self, ctxt, key_id, **kwargs):
         if ctxt is None:
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         copied_key_id = self._generate_key_id()
         self.keys[copied_key_id] = self.keys[key_id]
@@ -110,21 +110,21 @@ class MockKeyManager(key_mgr.KeyManager):
         """Retrieves the key identified by the specified id.
 
         This implementation returns the key that is associated with the
-        specified UUID. A NotAuthorized exception is raised if the specified
+        specified UUID. A Forbidden exception is raised if the specified
         context is None; a KeyError is raised if the UUID is invalid.
         """
         if ctxt is None:
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         return self.keys[key_id]
 
     def delete_key(self, ctxt, key_id, **kwargs):
         """Deletes the key identified by the specified id.
 
-        A NotAuthorized exception is raised if the context is None and a
+        A Forbidden exception is raised if the context is None and a
         KeyError is raised if the UUID is invalid.
         """
         if ctxt is None:
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         del self.keys[key_id]
