@@ -23,8 +23,8 @@ import time
 from oslo.config import cfg
 
 from nova import exception
+from nova.i18n import _
 from nova.network import dns_driver
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 from nova import utils
 
