@@ -19,8 +19,8 @@ import tempfile
 from oslo.config import cfg
 
 from nova import exception
+from nova.i18n import _
 from nova.network import dns_driver
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 
 CONF = cfg.CONF
@@ -45,7 +45,7 @@ class MiniDNS(dns_driver.DNSDriver):
         else:
             self.tempdir = tempfile.mkdtemp()
             self.filename = os.path.join(self.tempdir, "dnstest.txt")
-        LOG.debug(_('minidns file is |%s|'), self.filename)
+        LOG.debug('minidns file is |%s|', self.filename)
 
         if not os.path.exists(self.filename):
             f = open(self.filename, "w+")
