@@ -19,7 +19,6 @@ from nova.objects import base
 from nova.objects import fields
 
 
-@base.NovaObjectRegistry.register
 class Agent(base.NovaPersistentObject, base.NovaObject):
     VERSION = '1.0'
 
@@ -36,7 +35,7 @@ class Agent(base.NovaPersistentObject, base.NovaObject):
     @staticmethod
     def _from_db_object(context, agent, db_agent):
         for name in agent.fields:
-            setattr(agent, name, db_agent[name])
+            agent[name] = db_agent[name]
         agent._context = context
         agent.obj_reset_changes()
         return agent
@@ -50,32 +49,34 @@ class Agent(base.NovaPersistentObject, base.NovaObject):
         return cls._from_db_object(context, objects.Agent(), db_agent)
 
     @base.remotable
-    def create(self):
+    def create(self, context):
         updates = self.obj_get_changes()
         if 'id' in updates:
             raise exception.ObjectActionError(action='create',
                                               reason='Already Created')
-        db_agent = db.agent_build_create(self._context, updates)
-        self._from_db_object(self._context, self, db_agent)
+        db_agent = db.agent_build_create(context, updates)
+        self._from_db_object(context, self, db_agent)
 
     @base.remotable
-    def destroy(self):
-        db.agent_build_destroy(self._context, self.id)
+    def destroy(self, context):
+        db.agent_build_destroy(context, self.id)
 
     @base.remotable
-    def save(self):
+    def save(self, context):
         updates = self.obj_get_changes()
-        db.agent_build_update(self._context, self.id, updates)
+        db.agent_build_update(context, self.id, updates)
         self.obj_reset_changes()
 
 
-@base.NovaObjectRegistry.register
 class AgentList(base.ObjectListBase, base.NovaObject):
     VERSION = '1.0'
 
     fields = {
         'objects': fields.ListOfObjectsField('Agent'),
         }
+    child_versions = {
+        '1.0': '1.0',
+        }
 
     @base.remotable_classmethod
     def get_all(cls, context, hypervisor=None):
