@@ -10,19 +10,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from nova.db import api as db
+from nova import db
 from nova.objects import base
 from nova.objects import fields
 
 
-# TODO(stephenfin): Remove after W, once we no longer need to worry about the
-# versioned notification framework using this
-@base.NovaObjectRegistry.register
 class BandwidthUsage(base.NovaPersistentObject, base.NovaObject):
     # Version 1.0: Initial version
     # Version 1.1: Add use_slave to get_by_instance_uuid_and_mac
-    # Version 1.2: Add update_cells to create
-    VERSION = '1.2'
+    VERSION = '1.1'
 
     fields = {
         'instance_uuid': fields.UUIDField(),
@@ -38,25 +34,16 @@ class BandwidthUsage(base.NovaPersistentObject, base.NovaObject):
     @staticmethod
     def _from_db_object(context, bw_usage, db_bw_usage):
         for field in bw_usage.fields:
-            if field == 'instance_uuid':
-                setattr(bw_usage, field, db_bw_usage['uuid'])
-            else:
-                setattr(bw_usage, field, db_bw_usage[field])
+            bw_usage[field] = db_bw_usage[field]
         bw_usage._context = context
         bw_usage.obj_reset_changes()
         return bw_usage
 
-    @staticmethod
-    @db.select_db_reader_mode
-    def _db_bw_usage_get(context, uuid, start_period, mac, use_slave=False):
-        return db.bw_usage_get(context, uuid=uuid, start_period=start_period,
-                               mac=mac)
-
     @base.serialize_args
     @base.remotable_classmethod
     def get_by_instance_uuid_and_mac(cls, context, instance_uuid, mac,
                                      start_period=None, use_slave=False):
-        db_bw_usage = cls._db_bw_usage_get(context, uuid=instance_uuid,
+        db_bw_usage = db.bw_usage_get(context, uuid=instance_uuid,
                                       start_period=start_period, mac=mac,
                                       use_slave=use_slave)
         if db_bw_usage:
@@ -64,39 +51,31 @@ class BandwidthUsage(base.NovaPersistentObject, base.NovaObject):
 
     @base.serialize_args
     @base.remotable
-    def create(self, uuid, mac, bw_in, bw_out, last_ctr_in,
-               last_ctr_out, start_period=None, last_refreshed=None,
-               update_cells=True):
+    def create(self, context, uuid, mac, bw_in, bw_out, last_ctr_in,
+               last_ctr_out, start_period=None, last_refreshed=None):
         db_bw_usage = db.bw_usage_update(
-            self._context, uuid, mac, start_period, bw_in, bw_out,
+            context, uuid, mac, start_period, bw_in, bw_out,
             last_ctr_in, last_ctr_out, last_refreshed=last_refreshed)
 
-        self._from_db_object(self._context, self, db_bw_usage)
+        self._from_db_object(context, self, db_bw_usage)
 
 
-# TODO(stephenfin): Remove after W, once we no longer need to worry about the
-# versioned notification framework using this
-@base.NovaObjectRegistry.register
 class BandwidthUsageList(base.ObjectListBase, base.NovaObject):
     # Version 1.0: Initial version
     # Version 1.1: Add use_slave to get_by_uuids
-    # Version 1.2: BandwidthUsage <= version 1.2
-    VERSION = '1.2'
+    VERSION = '1.1'
     fields = {
         'objects': fields.ListOfObjectsField('BandwidthUsage'),
     }
-
-    @staticmethod
-    @db.select_db_reader_mode
-    def _db_bw_usage_get_by_uuids(context, uuids, start_period,
-                                  use_slave=False):
-        return db.bw_usage_get_by_uuids(context, uuids=uuids,
-                                        start_period=start_period)
+    child_versions = {
+        '1.0': '1.0',
+        '1.1': '1.1',
+    }
 
     @base.serialize_args
     @base.remotable_classmethod
     def get_by_uuids(cls, context, uuids, start_period=None, use_slave=False):
-        db_bw_usages = cls._db_bw_usage_get_by_uuids(context, uuids=uuids,
+        db_bw_usages = db.bw_usage_get_by_uuids(context, uuids=uuids,
                                                 start_period=start_period,
                                                 use_slave=use_slave)
         return base.obj_make_list(context, cls(), BandwidthUsage, db_bw_usages)
