@@ -16,7 +16,9 @@
 
 import collections
 import copy
+import datetime
 import functools
+import traceback
 
 import netaddr
 from oslo import messaging
@@ -24,9 +26,11 @@ import six
 
 from nova import context
 from nova import exception
+from nova.i18n import _, _LE
+from nova import objects
 from nova.objects import fields
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
+from nova.openstack.common import timeutils
 from nova.openstack.common import versionutils
 
 
@@ -56,6 +60,9 @@ def make_class_properties(cls):
             if name not in cls.fields:
                 cls.fields[name] = field
     for name, field in cls.fields.iteritems():
+        if not isinstance(field, fields.Field):
+            raise exception.ObjectFieldInvalid(
+                field=name, objname=cls.obj_name())
 
         def getter(self, name=name):
             attrname = get_attrname(name)
@@ -64,14 +71,22 @@ def make_class_properties(cls):
             return getattr(self, attrname)
 
         def setter(self, value, name=name, field=field):
+            attrname = get_attrname(name)
+            field_value = field.coerce(self, name, value)
+            if field.read_only and hasattr(self, attrname):
+                # Note(yjiang5): _from_db_object() may iterate
+                # every field and write, no exception in such situation.
+                if getattr(self, attrname) != field_value:
+                    raise exception.ReadOnlyFieldError(field=name)
+                else:
+                    return
+
             self._changed_fields.add(name)
             try:
-                return setattr(self, get_attrname(name),
-                               field.coerce(self, name, value))
+                return setattr(self, attrname, field_value)
             except Exception:
                 attr = "%s.%s" % (self.obj_name(), name)
-                LOG.exception(_('Error setting %(attr)s') %
-                              {'attr': attr})
+                LOG.exception(_LE('Error setting %(attr)s'), {'attr': attr})
                 raise
 
         setattr(cls, name, property(getter, setter))
@@ -86,12 +101,40 @@ class NovaObjectMetaclass(type):
 
     def __init__(cls, names, bases, dict_):
         if not hasattr(cls, '_obj_classes'):
-            # This will be set in the 'NovaObject' class.
+            # This means this is a base class using the metaclass. I.e.,
+            # the 'NovaObject' class.
             cls._obj_classes = collections.defaultdict(list)
+            return
+
+        def _vers_tuple(obj):
+            return tuple([int(x) for x in obj.VERSION.split(".")])
+
+        # Add the subclass to NovaObject._obj_classes. If the
+        # same version already exists, replace it. Otherwise,
+        # keep the list with newest version first.
+        make_class_properties(cls)
+        obj_name = cls.obj_name()
+        for i, obj in enumerate(cls._obj_classes[obj_name]):
+            if cls.VERSION == obj.VERSION:
+                cls._obj_classes[obj_name][i] = cls
+                # Update nova.objects with this newer class.
+                setattr(objects, obj_name, cls)
+                break
+            if _vers_tuple(cls) > _vers_tuple(obj):
+                # Insert before.
+                cls._obj_classes[obj_name].insert(i, cls)
+                if i == 0:
+                    # Later version than we've seen before. Update
+                    # nova.objects.
+                    setattr(objects, obj_name, cls)
+                break
         else:
-            # Add the subclass to NovaObject._obj_classes
-            make_class_properties(cls)
-            cls._obj_classes[cls.obj_name()].append(cls)
+            cls._obj_classes[obj_name].append(cls)
+            # Either this is the first time we've seen the object or it's
+            # an older version than anything we'e seen. Update nova.objects
+            # only if it's the first time we've seen this object name.
+            if not hasattr(objects, obj_name):
+                setattr(objects, obj_name, cls)
 
 
 # These are decorators that mark an object's method as remotable.
@@ -113,6 +156,10 @@ def remotable_classmethod(fn):
             if isinstance(result, NovaObject):
                 result._context = context
         return result
+
+    # NOTE(danms): Make this discoverable
+    wrapper.remotable = True
+    wrapper.original_fn = fn
     return classmethod(wrapper)
 
 
@@ -143,12 +190,21 @@ def remotable(fn):
             for key, value in updates.iteritems():
                 if key in self.fields:
                     field = self.fields[key]
-                    self[key] = field.from_primitive(self, key, value)
+                    # NOTE(ndipanov): Since NovaObjectSerializer will have
+                    # deserialized any object fields into objects already,
+                    # we do not try to deserialize them again here.
+                    if isinstance(value, NovaObject):
+                        self[key] = value
+                    else:
+                        self[key] = field.from_primitive(self, key, value)
             self.obj_reset_changes()
             self._changed_fields = set(updates.get('obj_what_changed', []))
             return result
         else:
             return fn(self, ctxt, *args, **kwargs)
+
+    wrapper.remotable = True
+    wrapper.original_fn = fn
     return wrapper
 
 
@@ -185,6 +241,15 @@ class NovaObject(object):
         for key in kwargs.keys():
             self[key] = kwargs[key]
 
+    def __repr__(self):
+        return '%s(%s)' % (
+            self.obj_name(),
+            ','.join(['%s=%s' % (name,
+                                 (self.obj_attr_is_set(name) and
+                                  field.stringify(getattr(self, name)) or
+                                  '<?>'))
+                      for name, field in sorted(self.fields.items())]))
+
     @classmethod
     def obj_name(cls):
         """Return a canonical name for this object which will be used over
@@ -200,25 +265,24 @@ class NovaObject(object):
                         '%(objtype)s') % dict(objtype=objname))
             raise exception.UnsupportedObjectError(objtype=objname)
 
-        latest = None
+        # NOTE(comstud): If there's not an exact match, return the highest
+        # compatible version. The objects stored in the class are sorted
+        # such that highest version is first, so only set compatible_match
+        # once below.
         compatible_match = None
+
         for objclass in cls._obj_classes[objname]:
             if objclass.VERSION == objver:
                 return objclass
-
-            version_bits = tuple([int(x) for x in objclass.VERSION.split(".")])
-            if latest is None:
-                latest = version_bits
-            elif latest < version_bits:
-                latest = version_bits
-
-            if versionutils.is_compatible(objver, objclass.VERSION):
+            if (not compatible_match and
+                    versionutils.is_compatible(objver, objclass.VERSION)):
                 compatible_match = objclass
 
         if compatible_match:
             return compatible_match
 
-        latest_ver = '%i.%i' % latest
+        # As mentioned above, latest version is always first in the list.
+        latest_ver = cls._obj_classes[objname][0].VERSION
         raise exception.IncompatibleObjectVersion(objname=objname,
                                                   objver=objver,
                                                   supported=latest_ver)
@@ -279,13 +343,25 @@ class NovaObject(object):
         This is responsible for taking the primitive representation of
         an object and making it suitable for the given target_version.
         This may mean converting the format of object attributes, removing
-        attributes that have been added since the target version, etc.
+        attributes that have been added since the target version, etc. In
+        general:
+
+        - If a new version of an object adds a field, this routine
+          should remove it for older versions.
+        - If a new version changed or restricted the format of a field, this
+          should convert it back to something a client knowing only of the
+          older version will tolerate.
+        - If an object that this object depends on is bumped, then this
+          object should also take a version bump. Then, this routine should
+          backlevel the dependent object (by calling its obj_make_compatible())
+          if the requested version of this object is older than the version
+          where the new dependent object was added.
 
         :param:primitive: The result of self.obj_to_primitive()
         :param:target_version: The version string requested by the recipient
-                               of the object.
-        :param:raises: nova.exception.UnsupportedObjectError if conversion
-                       is not possible for some reason.
+        of the object
+        :raises: nova.exception.UnsupportedObjectError if conversion
+        is not possible for some reason
         """
         pass
 
@@ -458,6 +534,12 @@ class ObjectListBase(object):
     # requested of the list object.
     child_versions = {}
 
+    def __init__(self, *args, **kwargs):
+        super(ObjectListBase, self).__init__(*args, **kwargs)
+        if 'objects' not in kwargs:
+            self.objects = []
+            self._changed_fields.discard('objects')
+
     def __iter__(self):
         """List iterator interface."""
         return iter(self.objects)
@@ -492,18 +574,6 @@ class ObjectListBase(object):
     def sort(self, cmp=None, key=None, reverse=False):
         self.objects.sort(cmp=cmp, key=key, reverse=reverse)
 
-    def _attr_objects_to_primitive(self):
-        """Serialization of object list."""
-        return [x.obj_to_primitive() for x in self.objects]
-
-    def _attr_objects_from_primitive(self, value):
-        """Deserialization of object list."""
-        objects = []
-        for entity in value:
-            obj = NovaObject.obj_from_primitive(entity, context=self._context)
-            objects.append(obj)
-        return objects
-
     def obj_make_compatible(self, primitive, target_version):
         primitives = primitive['objects']
         child_target_version = self.child_versions.get(target_version, '1.0')
@@ -533,8 +603,8 @@ class NovaObjectSerializer(messaging.NoOpSerializer):
     @property
     def conductor(self):
         if not hasattr(self, '_conductor'):
-            from nova.conductor import api as conductor_api
-            self._conductor = conductor_api.API()
+            from nova import conductor
+            self._conductor = conductor.API()
         return self._conductor
 
     def _process_object(self, context, objprim):
@@ -554,15 +624,19 @@ class NovaObjectSerializer(messaging.NoOpSerializer):
                   items from values having had action applied.
         """
         iterable = values.__class__
-        if iterable == set:
+        if issubclass(iterable, dict):
+            return iterable(**dict((k, action_fn(context, v))
+                            for k, v in six.iteritems(values)))
+        else:
             # NOTE(danms): A set can't have an unhashable value inside, such as
             # a dict. Convert sets to tuples, which is fine, since we can't
             # send them over RPC anyway.
-            iterable = tuple
-        return iterable([action_fn(context, value) for value in values])
+            if iterable == set:
+                iterable = tuple
+            return iterable([action_fn(context, value) for value in values])
 
     def serialize_entity(self, context, entity):
-        if isinstance(entity, (tuple, list, set)):
+        if isinstance(entity, (tuple, list, set, dict)):
             entity = self._process_iterable(context, self.serialize_entity,
                                             entity)
         elif (hasattr(entity, 'obj_to_primitive') and
@@ -573,7 +647,7 @@ class NovaObjectSerializer(messaging.NoOpSerializer):
     def deserialize_entity(self, context, entity):
         if isinstance(entity, dict) and 'nova_object.name' in entity:
             entity = self._process_object(context, entity)
-        elif isinstance(entity, (tuple, list, set)):
+        elif isinstance(entity, (tuple, list, set, dict)):
             entity = self._process_iterable(context, self.deserialize_entity,
                                             entity)
         return entity
@@ -621,3 +695,27 @@ def obj_make_list(context, list_obj, item_cls, db_list, **extra_args):
     list_obj._context = context
     list_obj.obj_reset_changes()
     return list_obj
+
+
+def serialize_args(fn):
+    """Decorator that will do the arguments serialization before remoting."""
+    def wrapper(obj, *args, **kwargs):
+        for kw in kwargs:
+            value_arg = kwargs.get(kw)
+            if kw == 'exc_val' and value_arg:
+                kwargs[kw] = str(value_arg)
+            elif kw == 'exc_tb' and (
+                    not isinstance(value_arg, six.string_types) and value_arg):
+                kwargs[kw] = ''.join(traceback.format_tb(value_arg))
+            elif isinstance(value_arg, datetime.datetime):
+                kwargs[kw] = timeutils.isotime(value_arg)
+        if hasattr(fn, '__call__'):
+            return fn(obj, *args, **kwargs)
+        # NOTE(danms): We wrap a descriptor, so use that protocol
+        return fn.__get__(None, obj)(*args, **kwargs)
+
+    # NOTE(danms): Make this discoverable
+    wrapper.remotable = getattr(fn, 'remotable', False)
+    wrapper.original_fn = fn
+    return (functools.wraps(fn)(wrapper) if hasattr(fn, '__call__')
+            else classmethod(wrapper))
