@@ -14,6 +14,7 @@
 
 from nova import db
 from nova import exception
+from nova import objects
 from nova.objects import base
 from nova.objects import fields
 
@@ -22,7 +23,10 @@ OPTIONAL_FIELDS = ['extra_specs', 'projects']
 
 
 class Flavor(base.NovaPersistentObject, base.NovaObject):
-    VERSION = '1.0'
+    # Version 1.0: Initial version
+    # Version 1.1: Added save_projects(), save_extra_specs(), removed
+    #              remoteable from save()
+    VERSION = '1.1'
 
     fields = {
         'id': fields.IntegerField(),
@@ -41,6 +45,11 @@ class Flavor(base.NovaPersistentObject, base.NovaObject):
         'projects': fields.ListOfStringsField(),
         }
 
+    def __init__(self, *args, **kwargs):
+        super(Flavor, self).__init__(*args, **kwargs)
+        self._orig_extra_specs = {}
+        self._orig_projects = {}
+
     @staticmethod
     def _from_db_object(context, flavor, db_flavor, expected_attrs=None):
         if expected_attrs is None:
@@ -78,23 +87,60 @@ class Flavor(base.NovaPersistentObject, base.NovaObject):
 
         self._load_projects()
 
+    def obj_reset_changes(self, fields=None):
+        super(Flavor, self).obj_reset_changes(fields=fields)
+        if fields is None or 'extra_specs' in fields:
+            self._orig_extra_specs = (dict(self.extra_specs)
+                                      if self.obj_attr_is_set('extra_specs')
+                                      else {})
+        if fields is None or 'projects' in fields:
+            self._orig_projects = (list(self.projects)
+                                   if self.obj_attr_is_set('projects')
+                                   else [])
+
+    def obj_what_changed(self):
+        changes = super(Flavor, self).obj_what_changed()
+        if ('extra_specs' in self and
+            self.extra_specs != self._orig_extra_specs):
+            changes.add('extra_specs')
+        if 'projects' in self and self.projects != self._orig_projects:
+            changes.add('projects')
+        return changes
+
+    @classmethod
+    def _obj_from_primitive(cls, context, objver, primitive):
+        self = super(Flavor, cls)._obj_from_primitive(context, objver,
+                                                      primitive)
+        changes = self.obj_what_changed()
+        if 'extra_specs' not in changes:
+            # This call left extra_specs "clean" so update our tracker
+            self._orig_extra_specs = (dict(self.extra_specs)
+                                      if self.obj_attr_is_set('extra_specs')
+                                      else {})
+        if 'projects' not in changes:
+            # This call left projects "clean" so update our tracker
+            self._orig_projects = (list(self.projects)
+                                   if self.obj_attr_is_set('projects')
+                                   else [])
+        return self
+
     @base.remotable_classmethod
     def get_by_id(cls, context, id):
         db_flavor = db.flavor_get(context, id)
-        return cls._from_db_object(context, cls(), db_flavor,
+        return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
     @base.remotable_classmethod
     def get_by_name(cls, context, name):
         db_flavor = db.flavor_get_by_name(context, name)
-        return cls._from_db_object(context, cls(), db_flavor,
+        return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
     @base.remotable_classmethod
     def get_by_flavor_id(cls, context, flavor_id, read_deleted=None):
         db_flavor = db.flavor_get_by_flavor_id(context, flavor_id,
                                                read_deleted)
-        return cls._from_db_object(context, cls(), db_flavor,
+        return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
     @base.remotable
@@ -129,7 +175,43 @@ class Flavor(base.NovaPersistentObject, base.NovaObject):
                              expected_attrs=expected_attrs)
 
     @base.remotable
-    def save(self, context):
+    def save_projects(self, context, to_add=None, to_delete=None):
+        """Add or delete projects.
+
+        :param:to_add: A list of projects to add
+        :param:to_delete: A list of projects to remove
+        """
+
+        to_add = to_add if to_add is not None else []
+        to_delete = to_delete if to_delete is not None else []
+
+        for project_id in to_add:
+            db.flavor_access_add(context, self.flavorid, project_id)
+        for project_id in to_delete:
+            db.flavor_access_remove(context, self.flavorid, project_id)
+        self.obj_reset_changes(['projects'])
+
+    @base.remotable
+    def save_extra_specs(self, context, to_add=None, to_delete=None):
+        """Add or delete extra_specs.
+
+        :param:to_add: A dict of new keys to add/update
+        :param:to_delete: A list of keys to remove
+        """
+
+        to_add = to_add if to_add is not None else []
+        to_delete = to_delete if to_delete is not None else []
+
+        if to_add:
+            db.flavor_extra_specs_update_or_create(context, self.flavorid,
+                                                   to_add)
+
+        for key in to_delete:
+            db.flavor_extra_specs_delete(context, self.flavorid, key)
+        self.obj_reset_changes(['extra_specs'])
+
+    def save(self):
+        context = self._context
         updates = self.obj_get_changes()
         projects = updates.pop('projects', None)
         extra_specs = updates.pop('extra_specs', None)
@@ -137,25 +219,29 @@ class Flavor(base.NovaPersistentObject, base.NovaObject):
             raise exception.ObjectActionError(
                 action='save', reason='read-only fields were changed')
 
-        if extra_specs:
-            db.flavor_extra_specs_update_or_create(context, self.flavorid,
-                                                   extra_specs)
+        if extra_specs is not None:
+            deleted_keys = (set(self._orig_extra_specs.keys()) -
+                            set(extra_specs.keys()))
+            added_keys = self.extra_specs
+        else:
+            added_keys = deleted_keys = None
 
-        # NOTE(danms): This could be much simpler and more efficient
-        # with a smarter flavor_access_update() method in db_api.
         if projects is not None:
-            current_projects = [x['project_id']
-                                for x in db.flavor_access_get_by_flavor_id(
-                                    context, self.flavorid)]
-            for project_id in projects:
-                if project_id not in current_projects:
-                    db.flavor_access_add(context, self.flavorid, project_id)
-                else:
-                    current_projects.remove(project_id)
-            for condemned_project_id in current_projects:
-                db.flavor_access_remove(context, self.flavorid,
-                                        condemned_project_id)
-        self.obj_reset_changes()
+            deleted_projects = set(self._orig_projects) - set(projects)
+            added_projects = set(projects) - set(self._orig_projects)
+        else:
+            added_projects = deleted_projects = None
+
+        # NOTE(danms): The first remotable method we call will reset
+        # our of the original values for projects and extra_specs. Thus,
+        # we collect the added/deleted lists for both above and /then/
+        # call these methods to update them.
+
+        if added_keys or deleted_keys:
+            self.save_extra_specs(context, self.extra_specs, deleted_keys)
+
+        if added_projects or deleted_projects:
+            self.save_projects(context, added_projects, deleted_projects)
 
     @base.remotable
     def destroy(self, context):
@@ -163,13 +249,14 @@ class Flavor(base.NovaPersistentObject, base.NovaObject):
 
 
 class FlavorList(base.ObjectListBase, base.NovaObject):
-    VERSION = '1.0'
+    VERSION = '1.1'
 
     fields = {
         'objects': fields.ListOfObjectsField('Flavor'),
         }
     child_versions = {
         '1.0': '1.0',
+        '1.1': '1.1',
         }
 
     @base.remotable_classmethod
@@ -179,5 +266,5 @@ class FlavorList(base.ObjectListBase, base.NovaObject):
                                        filters=filters, sort_key=sort_key,
                                        sort_dir=sort_dir, limit=limit,
                                        marker=marker)
-        return base.obj_make_list(context, cls(), Flavor, db_flavors,
-                                  expected_attrs=['extra_specs'])
+        return base.obj_make_list(context, cls(context), objects.Flavor,
+                                  db_flavors, expected_attrs=['extra_specs'])
