@@ -12,8 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from nova.compute import utils as compute_utils
 from nova import db
+from nova import objects
 from nova.objects import base
 from nova.objects import fields
 from nova.openstack.common import timeutils
@@ -109,6 +109,9 @@ class InstanceActionList(base.ObjectListBase, base.NovaObject):
 
 
 class InstanceActionEvent(base.NovaPersistentObject, base.NovaObject):
+    # Version 1.0: Initial version
+    # Version 1.1: event_finish_with_failure decorated with serialize_args
+    VERSION = '1.1'
     fields = {
         'id': fields.IntegerField(),
         'event': fields.StringField(nullable=True),
@@ -127,6 +130,29 @@ class InstanceActionEvent(base.NovaPersistentObject, base.NovaObject):
         event.obj_reset_changes()
         return event
 
+    @staticmethod
+    def pack_action_event_start(context, instance_uuid, event_name):
+        values = {'event': event_name,
+                  'instance_uuid': instance_uuid,
+                  'request_id': context.request_id,
+                  'start_time': timeutils.utcnow()}
+        return values
+
+    @staticmethod
+    def pack_action_event_finish(context, instance_uuid, event_name,
+                                 exc_val=None, exc_tb=None):
+        values = {'event': event_name,
+                  'instance_uuid': instance_uuid,
+                  'request_id': context.request_id,
+                  'finish_time': timeutils.utcnow()}
+        if exc_tb is None:
+            values['result'] = 'Success'
+        else:
+            values['result'] = 'Error'
+            values['message'] = exc_val
+            values['traceback'] = exc_tb
+        return values
+
     @base.remotable_classmethod
     def get_by_id(cls, context, action_id, event_id):
         db_event = db.action_event_get_by_id(context, action_id, event_id)
@@ -134,19 +160,19 @@ class InstanceActionEvent(base.NovaPersistentObject, base.NovaObject):
 
     @base.remotable_classmethod
     def event_start(cls, context, instance_uuid, event_name, want_result=True):
-        values = compute_utils.pack_action_event_start(context, instance_uuid,
-                                                       event_name)
+        values = cls.pack_action_event_start(context, instance_uuid,
+                                             event_name)
         db_event = db.action_event_start(context, values)
         if want_result:
             return cls._from_db_object(context, cls(), db_event)
 
+    @base.serialize_args
     @base.remotable_classmethod
     def event_finish_with_failure(cls, context, instance_uuid, event_name,
                                   exc_val=None, exc_tb=None, want_result=None):
-        values = compute_utils.pack_action_event_finish(context, instance_uuid,
-                                                        event_name,
-                                                        exc_val=exc_val,
-                                                        exc_tb=exc_tb)
+        values = cls.pack_action_event_finish(context, instance_uuid,
+                                              event_name, exc_val=exc_val,
+                                              exc_tb=exc_tb)
         db_event = db.action_event_finish(context, values)
         if want_result:
             return cls._from_db_object(context, cls(), db_event)
@@ -161,11 +187,9 @@ class InstanceActionEvent(base.NovaPersistentObject, base.NovaObject):
 
     @base.remotable
     def finish_with_failure(self, context, exc_val, exc_tb):
-        values = compute_utils.pack_action_event_finish(context,
-                                                        self.instance_uuid,
-                                                        self.event,
-                                                        exc_val=exc_val,
-                                                        exc_tb=exc_tb)
+        values = self.pack_action_event_finish(context, self.instance_uuid,
+                                               self.event, exc_val=exc_val,
+                                               exc_tb=exc_tb)
         db_event = db.action_event_finish(context, values)
         self._from_db_object(context, self, db_event)
 
@@ -180,10 +204,11 @@ class InstanceActionEventList(base.ObjectListBase, base.NovaObject):
         }
     child_versions = {
         '1.0': '1.0',
+        '1.1': '1.1',
         }
 
     @base.remotable_classmethod
     def get_by_action(cls, context, action_id):
         db_events = db.action_events_get(context, action_id)
-        return base.obj_make_list(context, cls(), InstanceActionEvent,
-                                  db_events)
+        return base.obj_make_list(context, cls(context),
+                                  objects.InstanceActionEvent, db_events)
