@@ -18,7 +18,7 @@ import collections
 
 from nova.compute import task_states
 from nova.compute import vm_states
-from nova import context
+from nova import context as nova_context
 from nova import exception
 from nova.i18n import _
 from nova import objects
@@ -41,7 +41,7 @@ class PciDevTracker(object):
     information is updated to DB when devices information is changed.
     """
 
-    def __init__(self, node_id=None):
+    def __init__(self, context, node_id=None):
         """Create a pci device tracker.
 
         If a node_id is passed in, it will fetch pci devices information
@@ -250,22 +250,6 @@ class PciDevTracker(object):
                 for dev in devs:
                     self._free_device(dev)
 
-    def set_compute_node_id(self, node_id):
-        """Set the compute node id that this object is tracking for.
-
-        In current resource tracker implementation, the
-        compute_node entry is created in the last step of
-        update_available_resoruces, thus we have to lazily set the
-        compute_node_id at that time.
-        """
-
-        if self.node_id and self.node_id != node_id:
-            raise exception.PciTrackerInvalidNodeId(node_id=self.node_id,
-                                                    new_node_id=node_id)
-        self.node_id = node_id
-        for dev in self.pci_devs:
-            dev.compute_node_id = node_id
-
 
 def get_instance_pci_devs(inst, request_id=None):
     """Get the devices allocated to one or all requests for an instance.
@@ -279,7 +263,7 @@ def get_instance_pci_devs(inst, request_id=None):
     if isinstance(inst, objects.Instance):
         pci_devices = inst.pci_devices
     else:
-        ctxt = context.get_admin_context()
+        ctxt = nova_context.get_admin_context()
         pci_devices = objects.PciDeviceList.get_by_instance_uuid(
             ctxt, inst['uuid'])
     return [device for device in pci_devices if
