@@ -16,6 +16,9 @@
 
 from oslo.config import cfg
 
+from nova import exception
+from nova.i18n import _
+from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.pci import pci_devspec
 
@@ -46,8 +49,26 @@ class PciHostDevicesWhiteList(object):
         """Parse and validate the pci whitelist from the nova config."""
         specs = []
         for jsonspec in whitelists:
-            spec = pci_devspec.PciDeviceSpec(jsonspec)
-            specs.append(spec)
+            try:
+                dev_spec = jsonutils.loads(jsonspec)
+            except ValueError:
+                raise exception.PciConfigInvalidWhitelist(
+                          reason=_("Invalid entry: '%s'") % jsonspec)
+            if isinstance(dev_spec, dict):
+                dev_spec = [dev_spec]
+            elif not isinstance(dev_spec, list):
+                raise exception.PciConfigInvalidWhitelist(
+                          reason=_("Invalid entry: '%s'; "
+                                   "Expecting list or dict") % jsonspec)
+
+            for ds in dev_spec:
+                if not isinstance(ds, dict):
+                    raise exception.PciConfigInvalidWhitelist(
+                              reason=_("Invalid entry: '%s'; "
+                                       "Expecting dict") % ds)
+
+                spec = pci_devspec.PciDeviceSpec(ds)
+                specs.append(spec)
 
         return specs
 
