@@ -41,18 +41,20 @@ class IsolatedHostsFilter(filters.BaseHostFilter):
 
     def host_passes(self, host_state, filter_properties):
         """Result Matrix with 'restrict_isolated_hosts_to_isolated_images' set
-        to True:
-                     | isolated_image | non_isolated_image
-        -------------+----------------+-------------------
-        iso_host     |    True        |     False
-        non_iso_host |    False       |      True
+        to True::
+
+        |                | isolated_image | non_isolated_image
+        |   -------------+----------------+-------------------
+        |   iso_host     |    True        |     False
+        |   non_iso_host |    False       |      True
 
         Result Matrix with 'restrict_isolated_hosts_to_isolated_images' set
-        to False:
-                     | isolated_image | non_isolated_image
-        -------------+----------------+-------------------
-        iso_host     |    True        |      True
-        non_iso_host |    False       |      True
+        to False::
+
+        |                | isolated_image | non_isolated_image
+        |   -------------+----------------+-------------------
+        |   iso_host     |    True        |      True
+        |   non_iso_host |    False       |      True
 
         """
         # If the configuration does not list any hosts, the filter will always
