@@ -21,7 +21,7 @@ from oslo.config import cfg
 
 from nova import conductor
 from nova import context
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _, _LE
 from nova.openstack.common import log as logging
 from nova.openstack.common import memorycache
 from nova.openstack.common import timeutils
@@ -72,7 +72,7 @@ class MemcachedDriver(api.ServiceGroupDriver):
     def get_all(self, group_id):
         """Returns ALL members of the given group
         """
-        LOG.debug(_('Memcached_Driver: get_all members of the %s group') %
+        LOG.debug('Memcached_Driver: get_all members of the %s group',
                   group_id)
         rs = []
         ctxt = context.get_admin_context()
@@ -84,7 +84,6 @@ class MemcachedDriver(api.ServiceGroupDriver):
 
     def _report_state(self, service):
         """Update the state of this service in the datastore."""
-        ctxt = context.get_admin_context()
         try:
             key = "%(topic)s:%(host)s" % service.service_ref
             # memcached has data expiration time capability.
@@ -103,4 +102,4 @@ class MemcachedDriver(api.ServiceGroupDriver):
         except Exception:  # pylint: disable=W0702
             if not getattr(service, 'model_disconnected', False):
                 service.model_disconnected = True
-                LOG.exception(_('model server went away'))
+                LOG.exception(_LE('model server went away'))
