@@ -34,12 +34,10 @@ import webob
 
 from nova.api import auth
 from nova.api import ec2
-from nova.api.ec2 import apirequest
 from nova.api.ec2 import ec2utils
 from nova import block_device
 from nova import context
 from nova import exception
-from nova.openstack.common import timeutils
 from nova.openstack.common import versionutils
 from nova import test
 from nova.tests import matchers
@@ -96,7 +94,7 @@ class FakeHttplibConnection(object):
         pass
 
 
-class XmlConversionTestCase(test.TestCase):
+class XmlConversionTestCase(test.NoDBTestCase):
     """Unit test api xml conversion."""
     def test_number_conversion(self):
         conv = ec2utils._try_convert
@@ -127,7 +125,7 @@ class XmlConversionTestCase(test.TestCase):
         self.assertEqual(conv(''), '')
 
 
-class Ec2utilsTestCase(test.TestCase):
+class Ec2utilsTestCase(test.NoDBTestCase):
     def test_ec2_id_to_id(self):
         self.assertEqual(ec2utils.ec2_id_to_id('i-0000001e'), 30)
         self.assertEqual(ec2utils.ec2_id_to_id('ami-1d'), 29)
@@ -281,21 +279,6 @@ class ApiEc2TestCase(test.TestCase):
             self.ec2.new_http_connection(host, is_secure).AndReturn(self.http)
         return self.http
 
-    def test_return_valid_isoformat(self):
-        """Ensure that the ec2 api returns datetime in xs:dateTime
-           (which apparently isn't datetime.isoformat())
-           NOTE(ken-pepple): https://bugs.launchpad.net/nova/+bug/721297
-        """
-        conv = apirequest._database_to_isoformat
-        # sqlite database representation with microseconds
-        time_to_convert = timeutils.parse_strtime("2011-02-21 20:14:10.634276",
-                                                  "%Y-%m-%d %H:%M:%S.%f")
-        self.assertEqual(conv(time_to_convert), '2011-02-21T20:14:10.634Z')
-        # mysqlite database representation
-        time_to_convert = timeutils.parse_strtime("2011-02-21 19:56:18",
-                                                  "%Y-%m-%d %H:%M:%S")
-        self.assertEqual(conv(time_to_convert), '2011-02-21T19:56:18.000Z')
-
     def test_xmlns_version_matches_request_version(self):
         self.expect_http(api_version='2010-10-30')
         self.mox.ReplayAll()
@@ -398,26 +381,26 @@ class ApiEc2TestCase(test.TestCase):
             (True, "test name", bad_amazon_ec2),
             (False, bad_strict_ec2, "test desc"),
         ]
-        for test in test_raise:
+        for t in test_raise:
             self.expect_http()
             self.mox.ReplayAll()
-            self.flags(ec2_strict_validation=test[0])
+            self.flags(ec2_strict_validation=t[0])
             self.assertRaises(boto_exc.EC2ResponseError,
                               self.ec2.create_security_group,
-                              test[1],
-                              test[2])
+                              t[1],
+                              t[2])
         test_accept = [
             (False, bad_amazon_ec2, "test desc"),
             (False, "test name", bad_amazon_ec2),
         ]
-        for test in test_accept:
+        for t in test_accept:
             self.expect_http()
             self.mox.ReplayAll()
-            self.flags(ec2_strict_validation=test[0])
-            self.ec2.create_security_group(test[1], test[2])
+            self.flags(ec2_strict_validation=t[0])
+            self.ec2.create_security_group(t[1], t[2])
             self.expect_http()
             self.mox.ReplayAll()
-            self.ec2.delete_security_group(test[1])
+            self.ec2.delete_security_group(t[1])
 
     def test_group_name_valid_length_security_group(self):
         """Test that we sanely handle invalid security group names.
