@@ -30,10 +30,11 @@ from nova.compute import utils as compute_utils
 from nova import context
 from nova import db
 from nova import exception
-from nova.objects import instance as instance_obj
+from nova import objects
 from nova import test
 from nova.tests import cast_as_call
 from nova.tests import fake_network
+from nova.tests import fake_notifier
 from nova.tests import fake_utils
 from nova.tests.image import fake
 from nova.tests import matchers
@@ -88,12 +89,12 @@ class CinderCloudTestCase(test.TestCase):
     def setUp(self):
         super(CinderCloudTestCase, self).setUp()
         ec2utils.reset_cache()
-        vol_tmpdir = self.useFixture(fixtures.TempDir()).path
+        self.useFixture(fixtures.TempDir()).path
         fake_utils.stub_out_utils_spawn_n(self.stubs)
         self.flags(compute_driver='nova.virt.fake.FakeDriver',
                    volume_api_class='nova.tests.fake_volume.API')
 
-        def fake_show(meh, context, id):
+        def fake_show(meh, context, id, **kwargs):
             return {'id': id,
                     'name': 'fake_name',
                     'container_format': 'ami',
@@ -126,6 +127,12 @@ class CinderCloudTestCase(test.TestCase):
         # Short-circuit the conductor service
         self.flags(use_local=True, group='conductor')
 
+        # Stub out the notification service so we use the no-op serializer
+        # and avoid lazy-load traces with the wrap_exception decorator in
+        # the compute service.
+        fake_notifier.stub_notifier(self.stubs)
+        self.addCleanup(fake_notifier.reset)
+
         # set up services
         self.conductor = self.start_service('conductor',
                 manager=CONF.conductor.manager)
@@ -264,6 +271,48 @@ class CinderCloudTestCase(test.TestCase):
         self.cloud.delete_snapshot(self.context, snap['snapshotId'])
         self.cloud.delete_volume(self.context, volume1_id)
 
+    def test_volume_status_of_attaching_volume(self):
+        """Test the volume's status in response when attaching a volume."""
+        vol1 = self.cloud.create_volume(self.context,
+                                        size=1,
+                                        name='test-ls',
+                                        description='test volume ls')
+        self.assertEqual('available', vol1['status'])
+
+        kwargs = {'image_id': 'ami-1',
+                  'instance_type': CONF.default_flavor,
+                  'max_count': 1}
+        ec2_instance_id = self._run_instance(**kwargs)
+        resp = self.cloud.attach_volume(self.context,
+                                        vol1['volumeId'],
+                                        ec2_instance_id,
+                                        '/dev/sde')
+        # Here,the status should be 'attaching',but it can be 'attached' in
+        # unittest scenario if the attach action is very fast.
+        self.assertIn(resp['status'], ('attaching', 'attached'))
+
+    def test_volume_status_of_detaching_volume(self):
+        """Test the volume's status in response when detaching a volume."""
+        vol1 = self.cloud.create_volume(self.context,
+                                        size=1,
+                                        name='test-ls',
+                                        description='test volume ls')
+        self.assertEqual('available', vol1['status'])
+        vol1_uuid = ec2utils.ec2_vol_id_to_uuid(vol1['volumeId'])
+        kwargs = {'image_id': 'ami-1',
+                  'instance_type': CONF.default_flavor,
+                  'max_count': 1,
+                  'block_device_mapping': [{'device_name': '/dev/sdb',
+                                            'volume_id': vol1_uuid,
+                                            'delete_on_termination': True}]}
+        self._run_instance(**kwargs)
+        resp = self.cloud.detach_volume(self.context,
+                                        vol1['volumeId'])
+
+        # Here,the status should be 'detaching',but it can be 'detached' in
+        # unittest scenario if the detach action is very fast.
+        self.assertIn(resp['status'], ('detaching', 'detached'))
+
     def test_describe_snapshots(self):
         # Makes sure describe_snapshots works and filters results.
         availability_zone = 'zone1:host1'
@@ -416,71 +465,83 @@ class CinderCloudTestCase(test.TestCase):
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
         sys_meta = flavors.save_flavor_info(
             {}, flavors.get_flavor(1))
-        inst1 = db.instance_create(self.context,
+        inst0 = db.instance_create(self.context,
                                   {'image_ref': image_uuid,
                                    'instance_type_id': 1,
                                    'root_device_name': '/dev/sdb1',
                                    'system_metadata': sys_meta})
-        inst2 = db.instance_create(self.context,
+        inst1 = db.instance_create(self.context,
                                   {'image_ref': image_uuid,
                                    'instance_type_id': 1,
                                    'root_device_name': '/dev/sdc1',
                                    'system_metadata': sys_meta})
+        inst2 = db.instance_create(self.context,
+                                  {'image_ref': '',
+                                   'instance_type_id': 1,
+                                   'root_device_name': '/dev/vda',
+                                   'system_metadata': sys_meta})
 
-        instance_uuid = inst1['uuid']
+        instance0_uuid = inst0['uuid']
         mappings0 = [
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb1',
              'snapshot_id': '1',
              'volume_id': '2'},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb2',
              'volume_id': '3',
              'volume_size': 1},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb3',
              'delete_on_termination': True,
              'snapshot_id': '4',
              'volume_id': '5'},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb4',
              'delete_on_termination': False,
              'snapshot_id': '6',
              'volume_id': '7'},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb5',
              'snapshot_id': '8',
              'volume_id': '9',
              'volume_size': 0},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb6',
              'snapshot_id': '10',
              'volume_id': '11',
              'volume_size': 1},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb7',
              'no_device': True},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb8',
              'virtual_name': 'swap'},
-            {'instance_uuid': instance_uuid,
+            {'instance_uuid': instance0_uuid,
              'device_name': '/dev/sdb9',
              'virtual_name': 'ephemeral3'}]
+        instance2_uuid = inst2['uuid']
+        mappings2 = [
+            {'instance_uuid': instance2_uuid,
+             'device_name': 'vda',
+             'snapshot_id': '1',
+             'volume_id': '21'}]
 
-        volumes = self._block_device_mapping_create(instance_uuid, mappings0)
-        return (inst1, inst2, volumes)
+        volumes0 = self._block_device_mapping_create(instance0_uuid, mappings0)
+        volumes2 = self._block_device_mapping_create(instance2_uuid, mappings2)
+        return ((inst0, inst1, inst2), (volumes0, [], volumes2))
 
-    def _tearDownBlockDeviceMapping(self, inst1, inst2, volumes):
-        for vol in volumes:
-            self.volume_api.delete(self.context, vol['id'])
-        for uuid in (inst1['uuid'], inst2['uuid']):
+    def _tearDownBlockDeviceMapping(self, instances, volumes):
+        for vols in volumes:
+            for vol in vols:
+                self.volume_api.delete(self.context, vol['id'])
+        for instance in instances:
             for bdm in db.block_device_mapping_get_all_by_instance(
-                    self.context, uuid):
+                    self.context, instance['uuid']):
                 db.block_device_mapping_destroy(self.context, bdm['id'])
-        db.instance_destroy(self.context, inst2['uuid'])
-        db.instance_destroy(self.context, inst1['uuid'])
+            db.instance_destroy(self.context, instance['uuid'])
 
-    _expected_instance_bdm1 = {
+    _expected_instance_bdm0 = {
         'instanceId': 'i-00000001',
         'rootDeviceName': '/dev/sdb1',
         'rootDeviceType': 'ebs'}
@@ -515,33 +576,38 @@ class CinderCloudTestCase(test.TestCase):
          'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
                  'volumeId': 'vol-0000000b', }}]
-        # NOTE(yamahata): swap/ephemeral device case isn't supported yet.
+    # NOTE(yamahata): swap/ephemeral device case isn't supported yet.
 
-    _expected_instance_bdm2 = {
+    _expected_instance_bdm1 = {
         'instanceId': 'i-00000002',
         'rootDeviceName': '/dev/sdc1',
         'rootDeviceType': 'instance-store'}
 
+    _expected_instance_bdm2 = {
+        'instanceId': 'i-00000003',
+        'rootDeviceName': '/dev/vda',
+        'rootDeviceType': 'ebs'}
+
     def test_format_instance_bdm(self):
-        (inst1, inst2, volumes) = self._setUpBlockDeviceMapping()
+        (instances, volumes) = self._setUpBlockDeviceMapping()
 
         result = {}
-        self.cloud._format_instance_bdm(self.context, inst1['uuid'],
+        self.cloud._format_instance_bdm(self.context, instances[0]['uuid'],
                                         '/dev/sdb1', result)
         self.assertThat(
-            {'rootDeviceType': self._expected_instance_bdm1['rootDeviceType']},
+            {'rootDeviceType': self._expected_instance_bdm0['rootDeviceType']},
             matchers.IsSubDictOf(result))
         self._assertEqualBlockDeviceMapping(
             self._expected_block_device_mapping0, result['blockDeviceMapping'])
 
         result = {}
-        self.cloud._format_instance_bdm(self.context, inst2['uuid'],
+        self.cloud._format_instance_bdm(self.context, instances[1]['uuid'],
                                         '/dev/sdc1', result)
         self.assertThat(
-            {'rootDeviceType': self._expected_instance_bdm2['rootDeviceType']},
+            {'rootDeviceType': self._expected_instance_bdm1['rootDeviceType']},
             matchers.IsSubDictOf(result))
 
-        self._tearDownBlockDeviceMapping(inst1, inst2, volumes)
+        self._tearDownBlockDeviceMapping(instances, volumes)
 
     def _assertInstance(self, instance_id):
         ec2_instance_id = ec2utils.id_to_ec2_id(instance_id)
@@ -568,21 +634,26 @@ class CinderCloudTestCase(test.TestCase):
         """Make sure describe_instances works with root_device_name and
         block device mappings
         """
-        (inst1, inst2, volumes) = self._setUpBlockDeviceMapping()
+        (instances, volumes) = self._setUpBlockDeviceMapping()
 
-        result = self._assertInstance(inst1['id'])
+        result = self._assertInstance(instances[0]['id'])
         self.assertThat(
-            self._expected_instance_bdm1,
+            self._expected_instance_bdm0,
             matchers.IsSubDictOf(result))
         self._assertEqualBlockDeviceMapping(
             self._expected_block_device_mapping0, result['blockDeviceMapping'])
 
-        result = self._assertInstance(inst2['id'])
+        result = self._assertInstance(instances[1]['id'])
+        self.assertThat(
+            self._expected_instance_bdm1,
+            matchers.IsSubDictOf(result))
+
+        result = self._assertInstance(instances[2]['id'])
         self.assertThat(
             self._expected_instance_bdm2,
             matchers.IsSubDictOf(result))
 
-        self._tearDownBlockDeviceMapping(inst1, inst2, volumes)
+        self._tearDownBlockDeviceMapping(instances, volumes)
 
     def _setUpImageSet(self, create_volumes_and_snapshots=False):
         self.flags(max_local_block_devices=-1)
@@ -637,7 +708,7 @@ class CinderCloudTestCase(test.TestCase):
                 'mappings': mappings2,
                 'block_device_mapping': block_device_mapping2}}
 
-        def fake_show(meh, context, image_id):
+        def fake_show(meh, context, image_id, **kwargs):
             _images = [copy.deepcopy(image1), copy.deepcopy(image2)]
             for i in _images:
                 if str(i['id']) == str(image_id):
@@ -773,8 +844,7 @@ class CinderCloudTestCase(test.TestCase):
 
         self.assertEqual(len(vols), 2)
         for vol in vols:
-            self.assertTrue(str(vol['id']) == str(vol1_uuid) or
-                str(vol['id']) == str(vol2_uuid))
+            self.assertIn(str(vol['id']), [str(vol1_uuid), str(vol2_uuid)])
             if str(vol['id']) == str(vol1_uuid):
                 self.volume_api.attach(self.context, vol['id'],
                                        instance_uuid, '/dev/sdb')
@@ -805,15 +875,13 @@ class CinderCloudTestCase(test.TestCase):
         vols = [v for v in vols if v['instance_uuid'] == instance_uuid]
         self.assertEqual(len(vols), 2)
         for vol in vols:
-            self.assertTrue(str(vol['id']) == str(vol1_uuid) or
-                            str(vol['id']) == str(vol2_uuid))
-            self.assertTrue(vol['mountpoint'] == '/dev/sdb' or
-                            vol['mountpoint'] == '/dev/sdc')
+            self.assertIn(str(vol['id']), [str(vol1_uuid), str(vol2_uuid)])
+            self.assertIn(vol['mountpoint'], ['/dev/sdb', '/dev/sdc'])
             self.assertEqual(vol['instance_uuid'], instance_uuid)
             self.assertEqual(vol['status'], "in-use")
             self.assertEqual(vol['attach_status'], "attached")
 
-        #Here we puke...
+        # Here we puke...
         self.cloud.terminate_instances(self.context, [ec2_instance_id])
 
         admin_ctxt = context.get_admin_context(read_deleted="no")
@@ -856,9 +924,7 @@ class CinderCloudTestCase(test.TestCase):
         vol = self.volume_api.get(self.context, vol2_uuid)
         self._assert_volume_detached(vol)
 
-        inst_obj = instance_obj.Instance.get_by_uuid(self.context,
-                                                         instance_uuid)
-        instance = db.instance_get_by_uuid(self.context, instance_uuid)
+        inst_obj = objects.Instance.get_by_uuid(self.context, instance_uuid)
         self.cloud.compute_api.attach_volume(self.context,
                                              inst_obj,
                                              volume_id=vol2_uuid,
@@ -871,7 +937,7 @@ class CinderCloudTestCase(test.TestCase):
         self._assert_volume_attached(vol2, instance_uuid, '/dev/sdc')
 
         self.cloud.compute_api.detach_volume(self.context,
-                                             instance, vol1)
+                                             inst_obj, vol1)
 
         vol1 = self.volume_api.get(self.context, vol1_uuid)
         self._assert_volume_detached(vol1)
@@ -953,7 +1019,7 @@ class CinderCloudTestCase(test.TestCase):
 
             self._assert_volume_attached(vol, instance_uuid, mountpoint)
 
-        #Just make sure we found them
+        # Just make sure we found them
         self.assertTrue(vol1_id)
         self.assertTrue(vol2_id)
 
