@@ -411,6 +411,26 @@ class BareMetalNodesTest(test.NoDBTestCase):
         self.assertEqual(expected_output, res_dict)
         mock_list.assert_called_once_with(detail=True)
 
+    @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'list')
+    def test_index_ironic_missing_properties(self, mock_list):
+        CONF.set_override('compute_driver', 'nova.virt.ironic.driver')
+
+        properties = {'cpus': 2}
+        node = ironic_utils.get_test_node(properties=properties)
+        mock_list.return_value = [node]
+
+        res_dict = self.controller.index(self.request)
+        expected_output = {'nodes':
+                            [{'memory_mb': 0,
+                             'host': 'IRONIC MANAGED',
+                             'disk_gb': 0,
+                             'interfaces': [],
+                             'task_state': None,
+                             'id': node.uuid,
+                             'cpus': properties['cpus']}]}
+        self.assertEqual(expected_output, res_dict)
+        mock_list.assert_called_once_with(detail=True)
+
     @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'list_ports')
     @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'get')
     def test_show_ironic(self, mock_get, mock_list_ports):
@@ -436,6 +456,31 @@ class BareMetalNodesTest(test.NoDBTestCase):
         mock_get.assert_called_once_with(node.uuid)
         mock_list_ports.assert_called_once_with(node.uuid)
 
+    @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'list_ports')
+    @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'get')
+    def test_show_ironic_no_properties(self, mock_get, mock_list_ports):
+        CONF.set_override('compute_driver', 'nova.virt.ironic.driver')
+
+        properties = {}
+        node = ironic_utils.get_test_node(properties=properties)
+        port = ironic_utils.get_test_port()
+        mock_get.return_value = node
+        mock_list_ports.return_value = [port]
+
+        res_dict = self.controller.show(self.request, node.uuid)
+        expected_output = {'node':
+                            {'memory_mb': 0,
+                             'instance_uuid': None,
+                             'host': 'IRONIC MANAGED',
+                             'disk_gb': 0,
+                             'interfaces': [{'address': port.address}],
+                             'task_state': None,
+                             'id': node.uuid,
+                             'cpus': 0}}
+        self.assertEqual(expected_output, res_dict)
+        mock_get.assert_called_once_with(node.uuid)
+        mock_list_ports.assert_called_once_with(node.uuid)
+
     @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'list_ports')
     @mock.patch.object(FAKE_IRONIC_CLIENT.node, 'get')
     def test_show_ironic_no_interfaces(self, mock_get, mock_list_ports):
