@@ -20,6 +20,7 @@ from nova.api.openstack.compute.contrib import extended_server_attributes
 from nova import compute
 from nova import db
 from nova import exception
+from nova import objects
 from nova.objects import instance as instance_obj
 from nova.openstack.common import jsonutils
 from nova import test
@@ -36,8 +37,8 @@ UUID3 = '00000000-0000-0000-0000-000000000003'
 
 def fake_compute_get(*args, **kwargs):
     fields = instance_obj.INSTANCE_DEFAULT_FIELDS
-    return instance_obj.Instance._from_db_object(
-        args[1], instance_obj.Instance(),
+    return objects.Instance._from_db_object(
+        args[1], objects.Instance(),
         fakes.stub_instance(1, uuid=UUID3, host="host-fake",
                             node="node-fake"), fields)
 
@@ -49,29 +50,28 @@ def fake_compute_get_all(*args, **kwargs):
     ]
     fields = instance_obj.INSTANCE_DEFAULT_FIELDS
     return instance_obj._make_instance_list(args[1],
-                                            instance_obj.InstanceList(),
+                                            objects.InstanceList(),
                                             db_list, fields)
 
 
-class ExtendedServerAttributesTest(test.TestCase):
+class ExtendedServerAttributesTestV21(test.TestCase):
     content_type = 'application/json'
     prefix = 'OS-EXT-SRV-ATTR:'
+    fake_url = '/v2/fake'
 
     def setUp(self):
-        super(ExtendedServerAttributesTest, self).setUp()
+        super(ExtendedServerAttributesTestV21, self).setUp()
         fakes.stub_out_nw_api(self.stubs)
         self.stubs.Set(compute.api.API, 'get', fake_compute_get)
         self.stubs.Set(compute.api.API, 'get_all', fake_compute_get_all)
         self.stubs.Set(db, 'instance_get_by_uuid', fake_compute_get)
-        self.flags(
-            osapi_compute_extension=[
-                'nova.api.openstack.compute.contrib.select_extensions'],
-            osapi_compute_ext_list=['Extended_server_attributes'])
 
     def _make_request(self, url):
         req = webob.Request.blank(url)
         req.headers['Accept'] = self.content_type
-        res = req.get_response(fakes.wsgi_app(init_only=('servers',)))
+        res = req.get_response(
+            fakes.wsgi_app_v21(init_only=('servers',
+                                          'os-extended-server-attributes')))
         return res
 
     def _get_server(self, body):
@@ -88,7 +88,7 @@ class ExtendedServerAttributesTest(test.TestCase):
                          node)
 
     def test_show(self):
-        url = '/v2/fake/servers/%s' % UUID3
+        url = self.fake_url + '/servers/%s' % UUID3
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -98,7 +98,7 @@ class ExtendedServerAttributesTest(test.TestCase):
                                 instance_name=NAME_FMT % 1)
 
     def test_detail(self):
-        url = '/v2/fake/servers/detail'
+        url = self.fake_url + '/servers/detail'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -114,13 +114,29 @@ class ExtendedServerAttributesTest(test.TestCase):
             raise exception.InstanceNotFound(instance_id='fake')
 
         self.stubs.Set(compute.api.API, 'get', fake_compute_get)
-        url = '/v2/fake/servers/70f6db34-de8d-4fbd-aafb-4065bdfa6115'
+        url = self.fake_url + '/servers/70f6db34-de8d-4fbd-aafb-4065bdfa6115'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 404)
 
 
-class ExtendedServerAttributesXmlTest(ExtendedServerAttributesTest):
+class ExtendedServerAttributesTestV2(ExtendedServerAttributesTestV21):
+
+    def setUp(self):
+        super(ExtendedServerAttributesTestV2, self).setUp()
+        self.flags(
+            osapi_compute_extension=[
+                'nova.api.openstack.compute.contrib.select_extensions'],
+            osapi_compute_ext_list=['Extended_server_attributes'])
+
+    def _make_request(self, url):
+        req = webob.Request.blank(url)
+        req.headers['Accept'] = self.content_type
+        res = req.get_response(fakes.wsgi_app(init_only=('servers',)))
+        return res
+
+
+class ExtendedServerAttributesXmlTest(ExtendedServerAttributesTestV2):
     content_type = 'application/xml'
     ext = extended_server_attributes
     prefix = '{%s}' % ext.Extended_server_attributes.namespace
