@@ -53,6 +53,7 @@ def network_api_get_floating_ips_by_project(self, context):
              'address': '10.10.10.10',
              'pool': 'nova',
              'fixed_ip': {'address': '10.0.0.1',
+                          'instance_uuid': FAKE_UUID,
                           'instance': {'uuid': FAKE_UUID}}},
             {'id': 2,
              'pool': 'nova', 'interface': 'eth0',
@@ -223,7 +224,6 @@ class FloatingIpTest(test.TestCase):
                                                     floating_ip_address)
         # NOTE(vish): network_get uses the id not the address
         floating_ip = db.floating_ip_get(self.context, floating_ip['id'])
-        self.controller._normalize_ip(floating_ip)
         view = floating_ips._translate_floating_ip_view(floating_ip)
         self.assertIn('floating_ip', view)
         self.assertTrue(view['floating_ip']['id'])
@@ -234,7 +234,6 @@ class FloatingIpTest(test.TestCase):
     def test_translate_floating_ip_view_dict(self):
         floating_ip = {'id': 0, 'address': '10.0.0.10', 'pool': 'nova',
                        'fixed_ip': None}
-        self.controller._normalize_ip(floating_ip)
         view = floating_ips._translate_floating_ip_view(floating_ip)
         self.assertIn('floating_ip', view)
 
@@ -318,6 +317,7 @@ class FloatingIpTest(test.TestCase):
         def get_floating_ip(self, context, id):
             return {'id': 1, 'address': '10.10.10.10', 'pool': 'nova',
                     'fixed_ip': {'address': '10.0.0.1',
+                                 'instance_uuid': FAKE_UUID,
                                  'instance': {'uuid': FAKE_UUID}}}
 
         self.stubs.Set(network.api.API, "get_floating_ip", get_floating_ip)
@@ -377,6 +377,34 @@ class FloatingIpTest(test.TestCase):
         self.assertIn('No more floating ips in pool non_existent_pool',
                       ex.explanation)
 
+    @mock.patch('nova.network.api.API.allocate_floating_ip',
+                side_effect=exception.FloatingIpLimitExceeded())
+    def test_floating_ip_allocate_over_quota(self, allocate_mock):
+        req = fakes.HTTPRequest.blank('/v2/fake/os-floating-ips')
+        ex = self.assertRaises(webob.exc.HTTPForbidden,
+                               self.controller.create, req)
+
+        self.assertIn('IP allocation over quota', ex.explanation)
+
+    @mock.patch('nova.network.api.API.allocate_floating_ip',
+                side_effect=exception.FloatingIpLimitExceeded())
+    def test_floating_ip_allocate_quota_exceed_in_pool(self, allocate_mock):
+        req = fakes.HTTPRequest.blank('/v2/fake/os-floating-ips')
+        ex = self.assertRaises(webob.exc.HTTPForbidden,
+            self.controller.create, req, {'pool': 'non_existent_pool'})
+
+        self.assertIn('IP allocation over quota in pool non_existent_pool.',
+                      ex.explanation)
+
+    @mock.patch('nova.network.api.API.allocate_floating_ip',
+                side_effect=exception.FloatingIpPoolNotFound())
+    def test_floating_ip_create_with_unknown_pool(self, allocate_mock):
+        req = fakes.HTTPRequest.blank('/v2/fake/os-floating-ips')
+        ex = self.assertRaises(webob.exc.HTTPNotFound,
+            self.controller.create, req, {'pool': 'non_existent_pool'})
+
+        self.assertIn('Floating ip pool not found.', ex.explanation)
+
     def test_floating_ip_allocate(self):
         def fake1(*args, **kwargs):
             pass
@@ -418,10 +446,9 @@ class FloatingIpTest(test.TestCase):
 
         req = fakes.HTTPRequest.blank('/v2/fake/servers/test_inst/action')
         rsp = self.manager._add_floating_ip(req, 'test_inst', body)
-        self.assertTrue(rsp.status_int == 202)
+        self.assertEqual(202, rsp.status_int)
 
     def test_floating_ip_associate_invalid_instance(self):
-        fixed_address = '192.168.1.100'
 
         def fake_get(self, context, id, expected_attrs=None,
                      want_objects=False):
@@ -453,13 +480,14 @@ class FloatingIpTest(test.TestCase):
 
         req = fakes.HTTPRequest.blank('/v2/fake/servers/test_inst/action')
         rsp = self.manager._add_floating_ip(req, 'test_inst', body)
-        self.assertTrue(rsp.status_int == 202)
+        self.assertEqual(202, rsp.status_int)
 
     def test_associate_not_allocated_floating_ip_to_instance(self):
         def fake_associate_floating_ip(self, context, instance,
                               floating_address, fixed_address,
                               affect_auto_assigned=False):
-            raise exception.NotAuthorized()
+            raise exception.FloatingIpNotFoundForAddress(
+                address=floating_address)
         self.stubs.Set(network.api.API, "associate_floating_ip",
                        fake_associate_floating_ip)
         floating_ip = '10.10.10.11'
@@ -474,6 +502,15 @@ class FloatingIpTest(test.TestCase):
         self.assertEqual(res_dict['itemNotFound']['message'],
                        "floating ip not found")
 
+    @mock.patch.object(network.api.API, 'associate_floating_ip',
+                       side_effect=exception.Forbidden)
+    def test_associate_floating_ip_forbidden(self, associate_mock):
+        body = dict(addFloatingIp=dict(address='10.10.10.11'))
+        req = fakes.HTTPRequest.blank('/v2/fake/servers/test_inst/action')
+        self.assertRaises(webob.exc.HTTPForbidden,
+                          self.manager._add_floating_ip, req, 'test_inst',
+                          body)
+
     def test_floating_ip_disassociate(self):
         def get_instance_by_floating_ip_addr(self, context, address):
             if address == '10.10.10.10':
@@ -486,7 +523,7 @@ class FloatingIpTest(test.TestCase):
 
         req = fakes.HTTPRequest.blank('/v2/fake/servers/test_inst/action')
         rsp = self.manager._remove_floating_ip(req, 'test_inst', body)
-        self.assertTrue(rsp.status_int == 202)
+        self.assertEqual(202, rsp.status_int)
 
     def test_floating_ip_disassociate_missing(self):
         body = dict(removeFloatingIp=dict(address='10.10.10.10'))
@@ -597,7 +634,7 @@ class FloatingIpTest(test.TestCase):
                 return 'test_inst'
 
         def network_api_disassociate(self, context, instance, address):
-            raise exception.NotAuthorized()
+            raise exception.Forbidden()
 
         self.stubs.Set(network.api.API, "get_floating_ip_by_address",
                        fake_get_floating_ip_addr_auto_assigned)
@@ -715,7 +752,7 @@ class ExtendedFloatingIpTest(test.TestCase):
 
         req = fakes.HTTPRequest.blank('/v2/fake/servers/test_inst/action')
         rsp = self.manager._add_floating_ip(req, 'test_inst', body)
-        self.assertTrue(rsp.status_int == 202)
+        self.assertEqual(202, rsp.status_int)
 
     def test_extended_floating_ip_associate_fixed_not_allocated(self):
         def fake_associate_floating_ip(*args, **kwargs):
