@@ -72,12 +72,12 @@ def fake_detail(*args, **kwargs):
     return IMAGES
 
 
-class ImageSizeTest(test.NoDBTestCase):
+class ImageSizeTestV21(test.NoDBTestCase):
     content_type = 'application/json'
     prefix = 'OS-EXT-IMG-SIZE'
 
     def setUp(self):
-        super(ImageSizeTest, self).setUp()
+        super(ImageSizeTestV21, self).setUp()
         self.stubs.Set(glance.GlanceImageService, 'show', fake_show)
         self.stubs.Set(glance.GlanceImageService, 'detail', fake_detail)
         self.flags(osapi_compute_extension=['nova.api.openstack.compute'
@@ -86,9 +86,15 @@ class ImageSizeTest(test.NoDBTestCase):
     def _make_request(self, url):
         req = webob.Request.blank(url)
         req.headers['Accept'] = self.content_type
-        res = req.get_response(fakes.wsgi_app())
+        res = req.get_response(self._get_app())
         return res
 
+    def _get_app(self):
+        return fakes.wsgi_app_v3()
+
+    def _get_url(self):
+        return '/v3'
+
     def _get_image(self, body):
         return jsonutils.loads(body).get('image')
 
@@ -99,7 +105,7 @@ class ImageSizeTest(test.NoDBTestCase):
         self.assertEqual(image.get('%s:size' % self.prefix), size)
 
     def test_show(self):
-        url = '/v2/fake/images/1'
+        url = self._get_url() + '/images/1'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -107,7 +113,7 @@ class ImageSizeTest(test.NoDBTestCase):
         self.assertImageSize(image, 12345678)
 
     def test_detail(self):
-        url = '/v2/fake/images/detail'
+        url = self._get_url() + '/images/detail'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -116,7 +122,15 @@ class ImageSizeTest(test.NoDBTestCase):
         self.assertImageSize(images[1], 87654321)
 
 
-class ImageSizeXmlTest(ImageSizeTest):
+class ImageSizeTestV2(ImageSizeTestV21):
+    def _get_app(self):
+        return fakes.wsgi_app()
+
+    def _get_url(self):
+        return '/v2/fake'
+
+
+class ImageSizeXmlTest(ImageSizeTestV2):
     content_type = 'application/xml'
     prefix = '{%s}' % image_size.Image_size.namespace
 
