@@ -19,8 +19,8 @@ from lxml import etree
 from nova.api.openstack.compute.contrib import migrations
 from nova import context
 from nova import exception
+from nova import objects
 from nova.objects import base
-from nova.objects import migration
 from nova.openstack.common.fixture import moxstubout
 from nova import test
 
@@ -61,8 +61,8 @@ fake_migrations = [
 
 migrations_obj = base.obj_make_list(
     'fake-context',
-    migration.MigrationList(),
-    migration.Migration,
+    objects.MigrationList(),
+    objects.Migration,
     fake_migrations)
 
 
@@ -87,9 +87,9 @@ class MigrationsTestCase(test.NoDBTestCase):
             'migrations': migrations.output(migrations_obj)}
 
         for mig in migrations_in_progress['migrations']:
-            self.assertTrue('id' in mig)
-            self.assertTrue('deleted' not in mig)
-            self.assertTrue('deleted_at' not in mig)
+            self.assertIn('id', mig)
+            self.assertNotIn('deleted', mig)
+            self.assertNotIn('deleted_at', mig)
 
         filters = {'host': 'host1', 'status': 'migrating',
                    'cell_name': 'ChildCell'}
