@@ -27,9 +27,11 @@ from nova import compute
 from nova import context
 import nova.db
 from nova import exception
+from nova.network import model
 from nova.network import neutronv2
 from nova.network.neutronv2 import api as neutron_api
 from nova.network.security_group import neutron_driver
+from nova.objects import instance as instance_obj
 from nova.openstack.common import jsonutils
 from nova import test
 from nova.tests.api.openstack.compute.contrib import test_security_groups
@@ -68,7 +70,7 @@ class TestNeutronSecurityGroups(
         return net
 
     def _create_port(self, **kwargs):
-        body = {'port': {}}
+        body = {'port': {'binding:vnic_type': model.VNIC_TYPE_NORMAL}}
         fields = ['security_groups', 'device_id', 'network_id',
                   'port_security_enabled']
         for field in fields:
@@ -90,6 +92,10 @@ class TestNeutronSecurityGroups(
         # Neutron's security group description field is optional.
         pass
 
+    def test_create_security_group_with_empty_description(self):
+        # Neutron's security group description field is optional.
+        pass
+
     def test_create_security_group_with_blank_name(self):
         # Neutron's security group name field is optional.
         pass
@@ -141,7 +147,7 @@ class TestNeutronSecurityGroups(
         self._create_port(
             network_id=net['network']['id'], security_groups=[sg['id']],
             device_id=test_security_groups.FAKE_UUID1)
-        expected = [{'rules': [], 'tenant_id': 'fake_tenant', 'id': sg['id'],
+        expected = [{'rules': [], 'tenant_id': 'fake', 'id': sg['id'],
                     'name': 'test', 'description': 'test-description'}]
         self.stubs.Set(nova.db, 'instance_get_by_uuid',
                        test_security_groups.return_server_by_uuid)
@@ -174,16 +180,16 @@ class TestNeutronSecurityGroups(
     def test_delete_security_group_in_use(self):
         sg = self._create_sg_template().get('security_group')
         self._create_network()
-        fake_instance = {'project_id': 'fake_tenant',
-                         'availability_zone': 'zone_one',
-                         'info_cache': {'network_info': []},
-                         'security_groups': [],
-                         'uuid': str(uuid.uuid4()),
-                         'display_name': 'test_instance'}
+        db_inst = fakes.stub_instance(id=1, nw_cache=[], security_groups=[])
+        _context = context.get_admin_context()
+        instance = instance_obj.Instance._from_db_object(
+            _context, instance_obj.Instance(), db_inst,
+            expected_attrs=instance_obj.INSTANCE_DEFAULT_FIELDS)
         neutron = neutron_api.API()
-        neutron.allocate_for_instance(context.get_admin_context(),
-                                      fake_instance,
-                                      security_groups=[sg['id']])
+        with mock.patch.object(nova.db, 'instance_get_by_uuid',
+                               return_value=db_inst):
+            neutron.allocate_for_instance(_context, instance,
+                                          security_groups=[sg['id']])
 
         req = fakes.HTTPRequest.blank('/v2/fake/os-security-groups/%s'
                                       % sg['id'])
@@ -302,8 +308,8 @@ class TestNeutronSecurityGroups(
 
     def test_get_raises_no_unique_match_error(self):
 
-        def fake_find_resourceid_by_name_or_id(client, param, name):
-
+        def fake_find_resourceid_by_name_or_id(client, param, name,
+                                               project_id=None):
             raise n_exc.NeutronClientNoUniqueMatch()
 
         self.stubs.Set(neutronv20, 'find_resourceid_by_name_or_id',
@@ -671,7 +677,7 @@ class MockClient(object):
             msg = 'Security Group name great than 255'
             raise n_exc.NeutronClientException(message=msg, status_code=401)
         ret = {'name': s.get('name'), 'description': s.get('description'),
-               'tenant_id': 'fake_tenant', 'security_group_rules': [],
+               'tenant_id': 'fake', 'security_group_rules': [],
                'id': str(uuid.uuid4())}
 
         self._fake_security_groups[ret['id']] = ret
@@ -710,7 +716,9 @@ class MockClient(object):
                'device_id': p.get('device_id', str(uuid.uuid4())),
                'admin_state_up': p.get('admin_state_up', True),
                'security_groups': p.get('security_groups', []),
-               'network_id': p.get('network_id')}
+               'network_id': p.get('network_id'),
+               'binding:vnic_type':
+                   p.get('binding:vnic_type') or model.VNIC_TYPE_NORMAL}
 
         network = self._fake_networks[p['network_id']]
         if 'port_security_enabled' in p:
@@ -814,8 +822,14 @@ class MockClient(object):
         return {'security_groups': ret}
 
     def list_networks(self, **_params):
-        return {'networks':
-                [network for network in self._fake_networks.values()]}
+        # neutronv2/api.py _get_available_networks calls this assuming
+        # search_opts filter "shared" is implemented and not ignored
+        shared = _params.get("shared", None)
+        if shared:
+            return {'networks': []}
+        else:
+            return {'networks':
+                 [network for network in self._fake_networks.values()]}
 
     def list_ports(self, **_params):
         ret = []
