@@ -19,7 +19,7 @@ import mox
 from oslo.config import cfg
 import webob
 
-from nova.api.openstack.compute.contrib import security_groups
+from nova.api.openstack.compute.contrib import security_groups as secgroups_v2
 from nova.api.openstack import wsgi
 from nova.api.openstack import xmlutil
 from nova import compute
@@ -27,6 +27,7 @@ from nova.compute import power_state
 from nova import context as context_maker
 import nova.db
 from nova import exception
+from nova import objects
 from nova.objects import instance as instance_obj
 from nova.openstack.common import jsonutils
 from nova import quota
@@ -120,14 +121,17 @@ def return_server_nonexistent(context, server_id, columns_to_join=None):
     raise exception.InstanceNotFound(instance_id=server_id)
 
 
+# NOTE(oomichi): v2.1 API does not support security group management (create/
+# update/delete a security group). We don't need to test this class against
+# v2.1 API.
 class TestSecurityGroups(test.TestCase):
     def setUp(self):
         super(TestSecurityGroups, self).setUp()
 
-        self.controller = security_groups.SecurityGroupController()
+        self.controller = secgroups_v2.SecurityGroupController()
         self.server_controller = (
-            security_groups.ServerSecurityGroupController())
-        self.manager = security_groups.SecurityGroupActionController()
+            secgroups_v2.ServerSecurityGroupController())
+        self.manager = secgroups_v2.SecurityGroupActionController()
 
         # This needs to be done here to set fake_id because the derived
         # class needs to be called first if it wants to set
@@ -176,6 +180,21 @@ class TestSecurityGroups(test.TestCase):
 
         self._assert_no_security_groups_reserved(req.environ['nova.context'])
 
+    def test_create_security_group_with_empty_description(self):
+        sg = security_group_template()
+        sg['description'] = ""
+
+        req = fakes.HTTPRequest.blank('/v2/fake/os-security-groups')
+        try:
+            self.controller.create(req, {'security_group': sg})
+            self.fail('Should have raised BadRequest exception')
+        except webob.exc.HTTPBadRequest as exc:
+            self.assertEqual('description has a minimum character requirement'
+                             ' of 1.', exc.explanation)
+        except exception.InvalidInput as exc:
+            self.fail('Should have raised BadRequest exception instead of')
+        self._assert_no_security_groups_reserved(req.environ['nova.context'])
+
     def test_create_security_group_with_blank_name(self):
         sg = security_group_template(name='')
 
@@ -286,8 +305,7 @@ class TestSecurityGroups(test.TestCase):
             self.assertEqual(res_dict['security_group']['name'], name)
 
         sg = security_group_template()
-        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
-                          self.controller.create,
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
                           req, {'security_group': sg})
 
     def test_get_security_group_list(self):
@@ -329,7 +347,7 @@ class TestSecurityGroups(test.TestCase):
         groups.append(sg)
         # An expected rule here needs to be created as the api returns
         # different attributes on the rule for a response than what was
-        # passed in. For exmaple:
+        # passed in. For example:
         #  "cidr": "0.0.0.0/0" ->"ip_range": {"cidr": "0.0.0.0/0"}
         expected_rule = security_group_rule_template(
             ip_range={'cidr': '10.2.3.124/24'}, parent_group_id=1,
@@ -789,11 +807,14 @@ class TestSecurityGroups(test.TestCase):
         self.manager._removeSecurityGroup(req, '1', body)
 
 
+# NOTE(oomichi): v2.1 API does not support security group management (create/
+# update/delete a security group). We don't need to test this class against
+# v2.1 API.
 class TestSecurityGroupRules(test.TestCase):
     def setUp(self):
         super(TestSecurityGroupRules, self).setUp()
 
-        self.controller = security_groups.SecurityGroupController()
+        self.controller = secgroups_v2.SecurityGroupController()
         if self.controller.security_group_api.id_is_uuid:
             id1 = '11111111-1111-1111-1111-111111111111'
             id2 = '22222222-2222-2222-2222-222222222222'
@@ -823,7 +844,7 @@ class TestSecurityGroupRules(test.TestCase):
 
         self.parent_security_group = db2
 
-        self.controller = security_groups.SecurityGroupRulesController()
+        self.controller = secgroups_v2.SecurityGroupRulesController()
 
     def test_create_by_cidr(self):
         rule = security_group_rule_template(cidr='10.2.3.124/24',
@@ -982,13 +1003,21 @@ class TestSecurityGroupRules(test.TestCase):
                           req, {'security_group_rule': rule})
 
     def test_create_with_non_existing_parent_group_id(self):
-        rule = security_group_rule_template(group_id='invalid',
+        rule = security_group_rule_template(group_id=None,
                                             parent_group_id=self.invalid_id)
 
         req = fakes.HTTPRequest.blank('/v2/fake/os-security-group-rules')
         self.assertRaises(webob.exc.HTTPNotFound, self.controller.create,
                           req, {'security_group_rule': rule})
 
+    def test_create_with_non_existing_group_id(self):
+        rule = security_group_rule_template(group_id='invalid',
+                                            parent_group_id=self.sg2['id'])
+
+        req = fakes.HTTPRequest.blank('/v2/fake/os-security-group-rules')
+        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
+                          req, {'security_group_rule': rule})
+
     def test_create_with_invalid_protocol(self):
         rule = security_group_rule_template(ip_protocol='invalid-protocol',
                                             cidr='10.2.2.0/24',
@@ -1140,7 +1169,7 @@ class TestSecurityGroupRules(test.TestCase):
         if proto == 'icmp':
             expected_rule['to_port'] = -1
             expected_rule['from_port'] = -1
-        self.assertTrue(security_group_rule == expected_rule)
+        self.assertEqual(expected_rule, security_group_rule)
 
     def test_create_with_no_ports_icmp(self):
         self._test_create_with_no_ports_and_no_group('icmp')
@@ -1169,10 +1198,10 @@ class TestSecurityGroupRules(test.TestCase):
             'ip_protocol': proto, 'to_port': to_port, 'parent_group_id':
              self.sg2['id'], 'ip_range': {}, 'id': security_group_rule['id']
         }
-        self.assertTrue(security_group_rule['ip_protocol'] == proto)
-        self.assertTrue(security_group_rule['from_port'] == from_port)
-        self.assertTrue(security_group_rule['to_port'] == to_port)
-        self.assertTrue(security_group_rule == expected_rule)
+        self.assertEqual(proto, security_group_rule['ip_protocol'])
+        self.assertEqual(from_port, security_group_rule['from_port'])
+        self.assertEqual(to_port, security_group_rule['to_port'])
+        self.assertEqual(expected_rule, security_group_rule)
 
     def test_create_with_ports_icmp(self):
         self._test_create_with_ports('icmp', 0, 1)
@@ -1234,8 +1263,7 @@ class TestSecurityGroupRules(test.TestCase):
             'ip_protocol': 'tcp', 'from_port': '121', 'to_port': '121',
             'parent_group_id': self.sg2['id'], 'group_id': self.sg1['id']
         }
-        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
-                          self.controller.create,
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
                           req, {'security_group_rule': rule})
 
     def test_create_rule_cidr_allow_all(self):
@@ -1291,7 +1319,7 @@ class TestSecurityGroupRulesXMLDeserializer(test.TestCase):
 
     def setUp(self):
         super(TestSecurityGroupRulesXMLDeserializer, self).setUp()
-        self.deserializer = security_groups.SecurityGroupRulesXMLDeserializer()
+        self.deserializer = secgroups_v2.SecurityGroupRulesXMLDeserializer()
 
     def test_create_request(self):
         serial_request = """
@@ -1349,7 +1377,7 @@ class TestSecurityGroupXMLDeserializer(test.TestCase):
 
     def setUp(self):
         super(TestSecurityGroupXMLDeserializer, self).setUp()
-        self.deserializer = security_groups.SecurityGroupXMLDeserializer()
+        self.deserializer = secgroups_v2.SecurityGroupXMLDeserializer()
 
     def test_create_request(self):
         serial_request = """
@@ -1402,9 +1430,9 @@ class TestSecurityGroupXMLSerializer(test.TestCase):
     def setUp(self):
         super(TestSecurityGroupXMLSerializer, self).setUp()
         self.namespace = wsgi.XMLNS_V11
-        self.rule_serializer = security_groups.SecurityGroupRuleTemplate()
-        self.index_serializer = security_groups.SecurityGroupsTemplate()
-        self.default_serializer = security_groups.SecurityGroupTemplate()
+        self.rule_serializer = secgroups_v2.SecurityGroupRuleTemplate()
+        self.index_serializer = secgroups_v2.SecurityGroupsTemplate()
+        self.default_serializer = secgroups_v2.SecurityGroupTemplate()
 
     def _tag(self, elem):
         tagname = elem.tag
@@ -1581,7 +1609,7 @@ def fake_compute_get_all(*args, **kwargs):
     ]
 
     return instance_obj._make_instance_list(args[1],
-                                            instance_obj.InstanceList(),
+                                            objects.InstanceList(),
                                             db_list,
                                             ['metadata', 'system_metadata',
                                              'security_groups', 'info_cache'])
@@ -1609,12 +1637,12 @@ def fake_get_instances_security_groups_bindings(inst, context, servers):
     return result
 
 
-class SecurityGroupsOutputTest(test.TestCase):
+class SecurityGroupsOutputTestV21(test.TestCase):
+    base_url = '/v3/servers'
     content_type = 'application/json'
 
     def setUp(self):
-        super(SecurityGroupsOutputTest, self).setUp()
-        self.controller = security_groups.SecurityGroupController()
+        super(SecurityGroupsOutputTestV21, self).setUp()
         fakes.stub_out_nw_api(self.stubs)
         self.stubs.Set(compute.api.API, 'get', fake_compute_get)
         self.stubs.Set(compute.api.API, 'get_all', fake_compute_get_all)
@@ -1623,6 +1651,10 @@ class SecurityGroupsOutputTest(test.TestCase):
             osapi_compute_extension=[
                 'nova.api.openstack.compute.contrib.select_extensions'],
             osapi_compute_ext_list=['Security_groups'])
+        self.app = self._setup_app()
+
+    def _setup_app(self):
+        return fakes.wsgi_app_v3(init_only=('os-security-groups', 'servers'))
 
     def _make_request(self, url, body=None):
         req = webob.Request.blank(url)
@@ -1631,7 +1663,7 @@ class SecurityGroupsOutputTest(test.TestCase):
             req.body = self._encode_body(body)
         req.content_type = self.content_type
         req.headers['Accept'] = self.content_type
-        res = req.get_response(fakes.wsgi_app(init_only=('servers',)))
+        res = req.get_response(self.app)
         return res
 
     def _encode_body(self, body):
@@ -1647,10 +1679,9 @@ class SecurityGroupsOutputTest(test.TestCase):
         return server.get('security_groups')
 
     def test_create(self):
-        url = '/v2/fake/servers'
         image_uuid = 'c905cedb-7281-47e4-8a62-f26bc5fc4c77'
         server = dict(name='server_test', imageRef=image_uuid, flavorRef=2)
-        res = self._make_request(url, {'server': server})
+        res = self._make_request(self.base_url, {'server': server})
         self.assertEqual(res.status_int, 202)
         server = self._get_server(res.body)
         for i, group in enumerate(self._get_groups(server)):
@@ -1658,7 +1689,7 @@ class SecurityGroupsOutputTest(test.TestCase):
             self.assertEqual(group.get('name'), name)
 
     def test_show(self):
-        url = '/v2/fake/servers/%s' % UUID3
+        url = self.base_url + '/' + UUID3
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -1668,7 +1699,7 @@ class SecurityGroupsOutputTest(test.TestCase):
             self.assertEqual(group.get('name'), name)
 
     def test_detail(self):
-        url = '/v2/fake/servers/detail'
+        url = self.base_url + '/detail'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 200)
@@ -1683,13 +1714,20 @@ class SecurityGroupsOutputTest(test.TestCase):
             raise exception.InstanceNotFound(instance_id='fake')
 
         self.stubs.Set(compute.api.API, 'get', fake_compute_get)
-        url = '/v2/fake/servers/70f6db34-de8d-4fbd-aafb-4065bdfa6115'
+        url = self.base_url + '/70f6db34-de8d-4fbd-aafb-4065bdfa6115'
         res = self._make_request(url)
 
         self.assertEqual(res.status_int, 404)
 
 
-class SecurityGroupsOutputXmlTest(SecurityGroupsOutputTest):
+class SecurityGroupsOutputTestV2(SecurityGroupsOutputTestV21):
+    base_url = '/v2/fake/servers'
+
+    def _setup_app(self):
+        return fakes.wsgi_app(init_only=('servers',))
+
+
+class SecurityGroupsOutputXmlTest(SecurityGroupsOutputTestV2):
     content_type = 'application/xml'
 
     class MinimalCreateServerTemplate(xmlutil.TemplateBuilder):
