@@ -17,12 +17,13 @@ from lxml import etree
 import webob
 
 from nova.api.openstack.compute.contrib import server_groups
+from nova.api.openstack.compute.plugins.v3 import server_groups as sg_v3
+from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
 from nova import context
 import nova.db
 from nova import exception
-from nova.objects import instance as instance_obj
-from nova.objects import instance_group as instance_group_obj
+from nova import objects
 from nova.openstack.common import uuidutils
 from nova import test
 from nova.tests.api.openstack import fakes
@@ -49,7 +50,6 @@ def server_group_resp_template(**kwargs):
     sgroup.setdefault('name', 'test')
     sgroup.setdefault('policies', [])
     sgroup.setdefault('members', [])
-    sgroup.setdefault('metadata', {})
     return sgroup
 
 
@@ -67,10 +67,6 @@ def server_group_db(sg):
         attrs['members'] = members
     else:
         attrs['members'] = []
-    if 'metadata' in attrs:
-        attrs['metadetails'] = attrs.pop('metadata')
-    else:
-        attrs['metadetails'] = {}
     attrs['deleted'] = 0
     attrs['deleted_at'] = None
     attrs['created_at'] = None
@@ -84,20 +80,30 @@ def server_group_db(sg):
     return AttrDict(attrs)
 
 
-class ServerGroupTest(test.TestCase):
+class ServerGroupTestV21(test.TestCase):
+
     def setUp(self):
-        super(ServerGroupTest, self).setUp()
-        self.controller = server_groups.ServerGroupController()
-        self.app = fakes.wsgi_app(init_only=('os-server-groups',))
+        super(ServerGroupTestV21, self).setUp()
+        self._setup_controller()
+        self.app = self._get_app()
+
+    def _setup_controller(self):
+        self.controller = sg_v3.ServerGroupController()
+
+    def _get_app(self):
+        return fakes.wsgi_app_v3(init_only=('os-server-groups',))
+
+    def _get_url(self):
+        return '/v3'
 
     def test_create_server_group_with_no_policies(self):
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         sgroup = server_group_template()
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
     def test_create_server_group_normal(self):
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         sgroup = server_group_template()
         policies = ['anti-affinity']
         sgroup['policies'] = policies
@@ -107,14 +113,14 @@ class ServerGroupTest(test.TestCase):
         self.assertEqual(res_dict['server_group']['policies'], policies)
 
     def _create_instance(self, context):
-        instance = instance_obj.Instance(image_ref=1, node='node1',
+        instance = objects.Instance(image_ref=1, node='node1',
                 reservation_id='a', host='host1', project_id='fake',
                 vm_state='fake', system_metadata={'key': 'value'})
         instance.create(context)
         return instance
 
     def _create_instance_group(self, context, members):
-        ig = instance_group_obj.InstanceGroup(name='fake_name',
+        ig = objects.InstanceGroup(name='fake_name',
                   user_id='fake_user', project_id='fake',
                   members=members)
         ig.create(context)
@@ -129,7 +135,7 @@ class ServerGroupTest(test.TestCase):
     def test_display_members(self):
         ctx = context.RequestContext('fake_user', 'fake')
         (ig_uuid, instances, members) = self._create_groups_and_instances(ctx)
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         res_dict = self.controller.show(req, ig_uuid)
         result_members = res_dict['server_group']['members']
         self.assertEqual(2, len(result_members))
@@ -139,13 +145,13 @@ class ServerGroupTest(test.TestCase):
     def test_display_active_members_only(self):
         ctx = context.RequestContext('fake_user', 'fake')
         (ig_uuid, instances, members) = self._create_groups_and_instances(ctx)
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
 
         # delete an instance
         instances[1].destroy(ctx)
         # check that the instance does not exist
         self.assertRaises(exception.InstanceNotFound,
-                          instance_obj.Instance.get_by_uuid,
+                          objects.Instance.get_by_uuid,
                           ctx, instances[1].uuid)
         res_dict = self.controller.show(req, ig_uuid)
         result_members = res_dict['server_group']['members']
@@ -156,66 +162,66 @@ class ServerGroupTest(test.TestCase):
     def test_create_server_group_with_illegal_name(self):
         # blank name
         sgroup = server_group_template(name='', policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # name with length 256
         sgroup = server_group_template(name='1234567890' * 26,
                                        policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # non-string name
         sgroup = server_group_template(name=12, policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # name with leading spaces
         sgroup = server_group_template(name='  leading spaces',
                                        policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # name with trailing spaces
         sgroup = server_group_template(name='trailing space ',
                                        policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # name with all spaces
         sgroup = server_group_template(name='    ',
                                        policies=['test_policy'])
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
     def test_create_server_group_with_illegal_policies(self):
         # blank policy
         sgroup = server_group_template(name='fake-name', policies='')
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # policy as integer
         sgroup = server_group_template(name='fake-name', policies=7)
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # policy as string
         sgroup = server_group_template(name='fake-name', policies='invalid')
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
         # policy as None
         sgroup = server_group_template(name='fake-name', policies=None)
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
@@ -223,7 +229,15 @@ class ServerGroupTest(test.TestCase):
         sgroup = server_group_template()
         policies = ['anti-affinity', 'affinity']
         sgroup['policies'] = policies
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
+        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
+                          req, {'server_group': sgroup})
+
+    def test_create_server_group_with_duplicate_policies(self):
+        sgroup = server_group_template()
+        policies = ['affinity', 'affinity']
+        sgroup['policies'] = policies
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
@@ -231,18 +245,18 @@ class ServerGroupTest(test.TestCase):
         sgroup = server_group_template()
         policies = ['storage-affinity', 'anti-affinity', 'rack-affinity']
         sgroup['policies'] = policies
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                           req, {'server_group': sgroup})
 
     def test_create_server_group_with_no_body(self):
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create, req, None)
 
     def test_create_server_group_with_no_server_group(self):
         body = {'no-instanceGroup': None}
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create, req, body)
 
@@ -250,7 +264,7 @@ class ServerGroupTest(test.TestCase):
         groups = []
         policies = ['anti-affinity']
         members = []
-        metadata = {'key1': 'value1'}
+        metadata = {}  # always empty
         names = ['default-x', 'test']
         sg1 = server_group_resp_template(id=str(1345),
                                            name=names[0],
@@ -271,7 +285,7 @@ class ServerGroupTest(test.TestCase):
         self.stubs.Set(nova.db, 'instance_group_get_all_by_project_id',
                        return_server_groups)
 
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups')
+        req = fakes.HTTPRequest.blank(self._get_url() + '/os-server-groups')
         res_dict = self.controller.index(req)
         self.assertEqual(res_dict, expected)
 
@@ -280,7 +294,7 @@ class ServerGroupTest(test.TestCase):
         tenant_groups = []
         policies = ['anti-affinity']
         members = []
-        metadata = {'key1': 'value1'}
+        metadata = {}  # always empty
         names = ['default-x', 'test']
         sg1 = server_group_resp_template(id=str(1345),
                                            name=names[0],
@@ -310,7 +324,7 @@ class ServerGroupTest(test.TestCase):
         self.stubs.Set(nova.db, 'instance_group_get_all_by_project_id',
                        return_tenant_server_groups)
 
-        path = '/v2/fake/os-server-groups?all_projects=True'
+        path = self._get_url() + '/os-server-groups?all_projects=True'
 
         req = fakes.HTTPRequest.blank(path, use_admin_context=True)
         res_dict = self.controller.index(req)
@@ -336,17 +350,33 @@ class ServerGroupTest(test.TestCase):
         self.stubs.Set(nova.db, 'instance_group_get',
                        return_server_group)
 
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups/123')
+        req = fakes.HTTPRequest.blank(self._get_url() +
+                                     '/os-server-groups/123')
         resp = self.controller.delete(req, '123')
         self.assertTrue(self.called)
         self.assertEqual(resp.status_int, 204)
 
     def test_delete_non_existing_server_group(self):
-        req = fakes.HTTPRequest.blank('/v2/fake/os-server-groups/invalid')
+        req = fakes.HTTPRequest.blank(self._get_url() +
+                                     '/os-server-groups/invalid')
         self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
                           req, 'invalid')
 
 
+class ServerGroupTestV2(ServerGroupTestV21):
+
+    def _setup_controller(self):
+        ext_mgr = extensions.ExtensionManager()
+        ext_mgr.extensions = {}
+        self.controller = server_groups.ServerGroupController(ext_mgr)
+
+    def _get_app(self):
+        return fakes.wsgi_app(init_only=('os-server-groups',))
+
+    def _get_url(self):
+        return '/v2/fake'
+
+
 class TestServerGroupXMLDeserializer(test.TestCase):
 
     def setUp(self):
@@ -421,7 +451,6 @@ class TestServerGroupXMLSerializer(test.TestCase):
     def _verify_server_group(self, raw_group, tree):
         policies = raw_group['policies']
         members = raw_group['members']
-        metadata = raw_group['metadata']
         self.assertEqual('server_group', self._tag(tree))
         self.assertEqual(raw_group['id'], tree.get('id'))
         self.assertEqual(raw_group['name'], tree.get('name'))
@@ -441,16 +470,7 @@ class TestServerGroupXMLSerializer(test.TestCase):
                     self.assertEqual(members[idx],
                                      gr_child.text)
             elif child_tag == 'metadata':
-                self.assertEqual(len(metadata), len(child))
-                metas = {}
-                for idx, gr_child in enumerate(child):
-                    self.assertEqual(self._tag(gr_child), 'meta')
-                    key = gr_child.get('key')
-                    self.assertTrue(key in ['key1', 'key2'])
-                    metas[key] = gr_child.text
-                self.assertEqual(len(metas), len(metadata))
-                for k in metadata:
-                    self.assertEqual(metadata[k], metas[k])
+                self.assertEqual(0, len(child))
 
     def _verify_server_group_brief(self, raw_group, tree):
         self.assertEqual('server_group', self._tag(tree))
@@ -460,13 +480,11 @@ class TestServerGroupXMLSerializer(test.TestCase):
     def test_group_serializer(self):
         policies = ["policy-1", "policy-2"]
         members = ["1", "2"]
-        metadata = dict(key1="value1", key2="value2")
         raw_group = dict(
             id='890',
             name='name',
             policies=policies,
-            members=members,
-            metadata=metadata)
+            members=members)
         sg_group = dict(server_group=raw_group)
         text = self.default_serializer.serialize(sg_group)
 
@@ -478,19 +496,16 @@ class TestServerGroupXMLSerializer(test.TestCase):
         policies = ["policy-1", "policy-2",
                     "policy-3"]
         members = ["1", "2", "3"]
-        metadata = dict(key1="value1", key2="value2")
         groups = [dict(
                  id='890',
                  name='test',
                  policies=policies[0:2],
-                 members=members[0:2],
-                 metadata=metadata),
+                 members=members[0:2]),
                  dict(
                  id='123',
                  name='default',
                  policies=policies[2:],
-                 members=members[2:],
-                 metadata=metadata)]
+                 members=members[2:])]
         sg_groups = dict(server_groups=groups)
         text = self.index_serializer.serialize(sg_groups)
 
