@@ -70,10 +70,12 @@ class CommonMixin(object):
         self.mox.UnsetStubs()
 
     def _test_action(self, action, body=None, method=None,
-                     compute_api_args_map={}):
+                     compute_api_args_map=None):
         if method is None:
             method = action
 
+        compute_api_args_map = compute_api_args_map or {}
+
         instance = self._stub_instance_get()
 
         args, kwargs = compute_api_args_map.get(action, ((), {}))
@@ -90,6 +92,28 @@ class CommonMixin(object):
         self.mox.VerifyAll()
         self.mox.UnsetStubs()
 
+    def _test_not_implemented_state(self, action, method=None):
+        if method is None:
+            method = action
+
+        instance = self._stub_instance_get()
+        body = {}
+        compute_api_args_map = {}
+        args, kwargs = compute_api_args_map.get(action, ((), {}))
+        getattr(self.compute_api, method)(self.context, instance,
+                                          *args, **kwargs).AndRaise(
+                NotImplementedError())
+
+        self.mox.ReplayAll()
+
+        res = self._make_request('/servers/%s/action' % instance.uuid,
+                                 {action: body})
+        self.assertEqual(501, res.status_int)
+        # Do these here instead of tearDown because this method is called
+        # more than once for the same test case
+        self.mox.VerifyAll()
+        self.mox.UnsetStubs()
+
     def _test_invalid_state(self, action, method=None, body_map=None,
                             compute_api_args_map=None):
         if method is None:
@@ -121,10 +145,11 @@ class CommonMixin(object):
         self.mox.UnsetStubs()
 
     def _test_locked_instance(self, action, method=None, body=None,
-                              compute_api_args_map={}):
+                              compute_api_args_map=None):
         if method is None:
             method = action
 
+        compute_api_args_map = compute_api_args_map or {}
         instance = self._stub_instance_get()
 
         args, kwargs = compute_api_args_map.get(action, ((), {}))
@@ -142,10 +167,37 @@ class CommonMixin(object):
         self.mox.VerifyAll()
         self.mox.UnsetStubs()
 
+    def _test_instance_not_found_in_compute_api(self, action,
+                         method=None, body=None, compute_api_args_map=None):
+        if method is None:
+            method = action
+
+        compute_api_args_map = compute_api_args_map or {}
+
+        instance = self._stub_instance_get()
+
+        args, kwargs = compute_api_args_map.get(action, ((), {}))
+        getattr(self.compute_api, method)(self.context, instance, *args,
+                                          **kwargs).AndRaise(
+                exception.InstanceNotFound(instance_id=instance.uuid))
+
+        self.mox.ReplayAll()
+
+        res = self._make_request('/servers/%s/action' % instance.uuid,
+                                 {action: body})
+        self.assertEqual(404, res.status_int)
+        # Do these here instead of tearDown because this method is called
+        # more than once for the same test case
+        self.mox.VerifyAll()
+        self.mox.UnsetStubs()
+
 
 class CommonTests(CommonMixin, test.NoDBTestCase):
-    def _test_actions(self, actions, method_translations={}, body_map={},
-                      args_map={}):
+    def _test_actions(self, actions, method_translations=None, body_map=None,
+                      args_map=None):
+        method_translations = method_translations or {}
+        body_map = body_map or {}
+        args_map = args_map or {}
         for action in actions:
             method = method_translations.get(action)
             body = body_map.get(action)
@@ -155,7 +207,24 @@ class CommonTests(CommonMixin, test.NoDBTestCase):
             # Re-mock this.
             self.mox.StubOutWithMock(self.compute_api, 'get')
 
-    def _test_actions_with_non_existed_instance(self, actions, body_map={}):
+    def _test_actions_instance_not_found_in_compute_api(self,
+                  actions, method_translations=None, body_map=None,
+                  args_map=None):
+        method_translations = method_translations or {}
+        body_map = body_map or {}
+        args_map = args_map or {}
+        for action in actions:
+            method = method_translations.get(action)
+            body = body_map.get(action)
+            self.mox.StubOutWithMock(self.compute_api, method or action)
+            self._test_instance_not_found_in_compute_api(
+                action, method=method, body=body,
+                compute_api_args_map=args_map)
+            # Re-mock this.
+            self.mox.StubOutWithMock(self.compute_api, 'get')
+
+    def _test_actions_with_non_existed_instance(self, actions, body_map=None):
+        body_map = body_map or {}
         for action in actions:
             self._test_non_existing_instance(action,
                                              body_map=body_map)
@@ -163,7 +232,11 @@ class CommonTests(CommonMixin, test.NoDBTestCase):
             self.mox.StubOutWithMock(self.compute_api, 'get')
 
     def _test_actions_raise_conflict_on_invalid_state(
-            self, actions, method_translations={}, body_map={}, args_map={}):
+            self, actions, method_translations=None, body_map=None,
+            args_map=None):
+        method_translations = method_translations or {}
+        body_map = body_map or {}
+        args_map = args_map or {}
         for action in actions:
             method = method_translations.get(action)
             self.mox.StubOutWithMock(self.compute_api, method or action)
@@ -174,8 +247,11 @@ class CommonTests(CommonMixin, test.NoDBTestCase):
             self.mox.StubOutWithMock(self.compute_api, 'get')
 
     def _test_actions_with_locked_instance(self, actions,
-                                           method_translations={},
-                                           body_map={}, args_map={}):
+                                           method_translations=None,
+                                           body_map=None, args_map=None):
+        method_translations = method_translations or {}
+        body_map = body_map or {}
+        args_map = args_map or {}
         for action in actions:
             method = method_translations.get(action)
             body = body_map.get(action)
