@@ -236,6 +236,7 @@ class ExtensionControllerTest(ExtensionTestCase):
             "SecurityGroupDefaultRules",
             "SecurityGroups",
             "ServerDiagnostics",
+            "ServerListMultiStatus",
             "ServerPassword",
             "ServerStartStop",
             "Services",
@@ -438,7 +439,7 @@ class ExtensionManagerTest(ExtensionTestCase):
     def test_invalid_extensions(self):
         # Don't need the serialization middleware here because we're
         # not testing any serialization
-        app = compute.APIRouter()
+        compute.APIRouter()
         ext_mgr = compute_extensions.ExtensionManager()
         ext_mgr.register(InvalidExtension())
         self.assertTrue(ext_mgr.is_loaded('FOXNSOX'))
@@ -681,7 +682,6 @@ class ExtensionsXMLSerializerTest(test.TestCase):
                 "description": "Adds the capability to share an image.",
                 "links": [{"rel": "describedby",
                             "type": "application/pdf",
-                            "type": "application/vnd.sun.wadl+xml",
                             "href": "http://foo.com/api/ext/cs-pie.pdf"},
                            {"rel": "describedby",
                             "type": "application/vnd.sun.wadl+xml",
