@@ -11,12 +11,13 @@
 #    under the License.
 
 import inspect
+
 import webob
 
 from nova.api.openstack import extensions
 from nova.api.openstack import wsgi
 from nova import exception
-from nova.openstack.common import gettextutils
+from nova import i18n
 from nova import test
 from nova.tests.api.openstack import fakes
 from nova.tests import utils
@@ -124,14 +125,14 @@ class RequestTest(test.NoDBTestCase):
                 compute_nodes[1])
         self.assertEqual(request.get_db_compute_node('id2'),
                 compute_nodes[2])
-        self.assertEqual(request.get_db_compute_node('id3'), None)
+        self.assertIsNone(request.get_db_compute_node('id3'))
         self.assertEqual(request.get_db_compute_nodes(),
                 {'id0': compute_nodes[0],
                  'id1': compute_nodes[1],
                  'id2': compute_nodes[2]})
 
     def test_from_request(self):
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -142,7 +143,7 @@ class RequestTest(test.NoDBTestCase):
     def test_asterisk(self):
         # asterisk should match first available if there
         # are not any other available matches
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -151,7 +152,7 @@ class RequestTest(test.NoDBTestCase):
         self.assertEqual(request.best_match_language(), 'en_GB')
 
     def test_prefix(self):
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -160,7 +161,7 @@ class RequestTest(test.NoDBTestCase):
         self.assertEqual(request.best_match_language(), 'zh_CN')
 
     def test_secondary(self):
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -169,7 +170,7 @@ class RequestTest(test.NoDBTestCase):
         self.assertEqual(request.best_match_language(), 'en_GB')
 
     def test_none_found(self):
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -178,7 +179,7 @@ class RequestTest(test.NoDBTestCase):
         self.assertIs(request.best_match_language(), None)
 
     def test_no_lang_header(self):
-        self.stubs.Set(gettextutils, 'get_available_languages',
+        self.stubs.Set(i18n, 'get_available_languages',
                        fakes.fake_get_available_languages)
 
         request = wsgi.Request.blank('/')
@@ -287,7 +288,7 @@ class JSONDeserializerTest(test.NoDBTestCase):
         self.assertEqual(deserializer.deserialize(data), as_dict)
 
     def test_json_invalid_utf8(self):
-        """ Send invalid utf-8 to JSONDeserializer"""
+        """Send invalid utf-8 to JSONDeserializer."""
         data = """{"server": {"min_count": 1, "flavorRef": "1",
                 "name": "\xf0\x28\x8c\x28",
                 "imageRef": "10bab10c-1304-47d",
@@ -335,7 +336,7 @@ class XMLDeserializerTest(test.NoDBTestCase):
         self.assertEqual(deserializer.deserialize(xml), as_dict)
 
     def test_xml_invalid_utf8(self):
-        """ Send invalid utf-8 to XMLDeserializer"""
+        """Send invalid utf-8 to XMLDeserializer."""
         xml = """ <a><name>\xf0\x28\x8c\x28</name></a> """
         deserializer = wsgi.XMLDeserializer()
         self.assertRaises(exception.MalformedRequestBody,
@@ -343,6 +344,14 @@ class XMLDeserializerTest(test.NoDBTestCase):
 
 
 class ResourceTest(test.NoDBTestCase):
+
+    def get_req_id_header_name(self, request):
+        header_name = 'x-openstack-request-id'
+        if utils.get_api_version(request) < 3:
+                header_name = 'x-compute-request-id'
+
+        return header_name
+
     def test_resource_call_with_method_get(self):
         class Controller(object):
             def index(self, req):
@@ -420,7 +429,7 @@ class ResourceTest(test.NoDBTestCase):
         expected_body = None
         response = req.get_response(app)
         self.assertEqual(response.status_int, 200)
-        #verify no content_type is contained in the request
+        # verify no content_type is contained in the request
         req.content_type = None
         req.body = '{"body": {"key": "value"}}'
         response = req.get_response(app)
@@ -449,7 +458,7 @@ class ResourceTest(test.NoDBTestCase):
     def test_resource_not_authorized(self):
         class Controller(object):
             def index(self, req):
-                raise exception.NotAuthorized()
+                raise exception.Forbidden()
 
         req = webob.Request.blank('/tests')
         app = fakes.TestRouter(Controller())
@@ -644,11 +653,9 @@ class ResourceTest(test.NoDBTestCase):
                 return {'foo': 'bar'}
 
         req = fakes.HTTPRequest.blank('/tests')
-        context = req.environ['nova.context']
         app = fakes.TestRouter(Controller())
         response = req.get_response(app)
-        self.assertEqual(response.headers['x-compute-request-id'],
-                context.request_id)
+        self.assertIn('nova.context', req.environ)
         self.assertEqual(response.body, '{"foo": "bar"}')
         self.assertEqual(response.status_int, 200)
 
@@ -663,7 +670,8 @@ class ResourceTest(test.NoDBTestCase):
         # NOTE(alaski): This test is really to ensure that a str response
         # doesn't error.  Not having a request_id header is a side effect of
         # our wsgi setup, ideally it would be there.
-        self.assertFalse(hasattr(response.headers, 'x-compute-request-id'))
+        expected_header = self.get_req_id_header_name(req)
+        self.assertFalse(hasattr(response.headers, expected_header))
         self.assertEqual(response.body, 'foo')
         self.assertEqual(response.status_int, 200)
 
@@ -673,11 +681,9 @@ class ResourceTest(test.NoDBTestCase):
                 pass
 
         req = fakes.HTTPRequest.blank('/tests')
-        context = req.environ['nova.context']
         app = fakes.TestRouter(Controller())
         response = req.get_response(app)
-        self.assertEqual(response.headers['x-compute-request-id'],
-                context.request_id)
+        self.assertIn('nova.context', req.environ)
         self.assertEqual(response.body, '')
         self.assertEqual(response.status_int, 200)
 
@@ -889,12 +895,12 @@ class ResourceTest(test.NoDBTestCase):
 
         def extension1(req):
             called.append('pre1')
-            resp_obj = yield
+            yield
             called.append('post1')
 
         def extension2(req):
             called.append('pre2')
-            resp_obj = yield
+            yield
             called.append('post2')
 
         extensions = [extension1, extension2]
@@ -994,11 +1000,11 @@ class ResourceTest(test.NoDBTestCase):
         called = []
 
         def extension1(req):
-            resp_obj = yield
+            yield
             called.append(1)
 
         def extension2(req):
-            resp_obj = yield
+            yield
             called.append(2)
 
         ext1 = extension1(None)
@@ -1023,11 +1029,11 @@ class ResourceTest(test.NoDBTestCase):
         called = []
 
         def extension1(req):
-            resp_obj = yield
+            yield
             called.append(1)
 
         def extension2(req):
-            resp_obj = yield
+            yield
             called.append(2)
             yield 'foo'
 
@@ -1220,19 +1226,19 @@ class ValidBodyTest(test.NoDBTestCase):
         self.assertTrue(self.controller.is_valid_body(body, 'foo'))
 
     def test_is_valid_body_none(self):
-        resource = wsgi.Resource(controller=None)
+        wsgi.Resource(controller=None)
         self.assertFalse(self.controller.is_valid_body(None, 'foo'))
 
     def test_is_valid_body_empty(self):
-        resource = wsgi.Resource(controller=None)
+        wsgi.Resource(controller=None)
         self.assertFalse(self.controller.is_valid_body({}, 'foo'))
 
     def test_is_valid_body_no_entity(self):
-        resource = wsgi.Resource(controller=None)
+        wsgi.Resource(controller=None)
         body = {'bar': {}}
         self.assertFalse(self.controller.is_valid_body(body, 'foo'))
 
     def test_is_valid_body_malformed_entity(self):
-        resource = wsgi.Resource(controller=None)
+        wsgi.Resource(controller=None)
         body = {'foo': 'bar'}
         self.assertFalse(self.controller.is_valid_body(body, 'foo'))
