@@ -22,6 +22,7 @@ import mock
 import six
 
 from nova.compute import claims
+from nova import context
 from nova import db
 from nova import exception
 from nova import objects
@@ -45,9 +46,11 @@ class FakeResourceHandler(object):
 class DummyTracker(object):
     icalled = False
     rcalled = False
-    pci_tracker = pci_manager.PciDevTracker()
     ext_resources_handler = FakeResourceHandler()
 
+    def __init__(self):
+        self.new_pci_tracker()
+
     def abort_instance_claim(self, *args, **kwargs):
         self.icalled = True
 
@@ -55,7 +58,8 @@ class DummyTracker(object):
         self.rcalled = True
 
     def new_pci_tracker(self):
-        self.pci_tracker = pci_manager.PciDevTracker()
+        ctxt = context.RequestContext('testuser', 'testproject')
+        self.pci_tracker = pci_manager.PciDevTracker(ctxt)
 
 
 @mock.patch('nova.objects.InstancePCIRequests.get_by_instance_uuid',
