@@ -21,6 +21,7 @@ import inspect
 import mock
 from oslo.config import cfg
 
+from nova import block_device
 from nova.cells import manager
 from nova.compute import api as compute_api
 from nova.compute import cells_api as compute_cells_api
@@ -178,6 +179,19 @@ class CellsComputeAPITestCase(test_compute.ComputeAPITestCase):
 
         self.assertEqual(migrations, response)
 
+    def test_update_block_device_mapping(self):
+        instance_type = {'swap': 1, 'ephemeral_gb': 1}
+        instance = self._create_fake_instance_obj()
+        bdms = [block_device.BlockDeviceDict({'source_type': 'image',
+                                              'destination_type': 'local',
+                                              'image_id': 'fake-image',
+                                              'boot_index': 0})]
+        self.compute_api._update_block_device_mapping(
+            instance_type, instance.uuid, bdms)
+        bdms = db.block_device_mapping_get_all_by_instance(
+            self.context, instance['uuid'])
+        self.assertEqual(0, len(bdms))
+
     @mock.patch('nova.cells.messaging._TargetedMessage')
     def test_rebuild_sig(self, mock_msg):
         # TODO(belliott) Cells could benefit from better testing to ensure API
