@@ -16,8 +16,10 @@ from oslo.config import cfg
 
 from nova.compute import power_state
 from nova import context
+from nova import objects
 from nova.objects import instance as instance_obj
 from nova.openstack.common import importutils
+from nova.tests.compute import eventlet_utils
 from nova.tests import fake_instance
 from nova.tests.virt.xenapi import stubs
 from nova.virt.xenapi import vm_utils
@@ -37,20 +39,22 @@ class ComputeXenTestCase(stubs.XenAPITestBaseNoDB):
 
         stubs.stubout_session(self.stubs, stubs.FakeSessionForVMTests)
         self.compute = importutils.import_object(CONF.compute_manager)
+        # execute power syncing synchronously for testing:
+        self.compute._sync_power_pool = eventlet_utils.SyncPool()
 
     def test_sync_power_states_instance_not_found(self):
         db_instance = fake_instance.fake_db_instance()
         ctxt = context.get_admin_context()
         instance_list = instance_obj._make_instance_list(ctxt,
-                instance_obj.InstanceList(), [db_instance], None)
+                objects.InstanceList(), [db_instance], None)
         instance = instance_list[0]
 
-        self.mox.StubOutWithMock(instance_obj.InstanceList, 'get_by_host')
+        self.mox.StubOutWithMock(objects.InstanceList, 'get_by_host')
         self.mox.StubOutWithMock(self.compute.driver, 'get_num_instances')
         self.mox.StubOutWithMock(vm_utils, 'lookup')
         self.mox.StubOutWithMock(self.compute, '_sync_instance_power_state')
 
-        instance_obj.InstanceList.get_by_host(ctxt,
+        objects.InstanceList.get_by_host(ctxt,
                 self.compute.host, use_slave=True).AndReturn(instance_list)
         self.compute.driver.get_num_instances().AndReturn(1)
         vm_utils.lookup(self.compute.driver._session, instance['name'],
