@@ -19,7 +19,7 @@ from nova.compute import manager as compute_manager
 from nova import context
 from nova import db
 from nova import exception
-from nova.objects import external_event as external_event_obj
+from nova import objects
 from nova import test
 from nova.virt import fake
 from nova.virt import virtapi
@@ -42,17 +42,9 @@ class VirtAPIBaseTest(test.NoDBTestCase, test.APICoverage):
                           getattr(self.virtapi, method), self.context,
                           *args, **kwargs)
 
-    def test_instance_update(self):
-        self.assertExpected('instance_update', 'fake-uuid',
-                            dict(host='foohost'))
-
     def test_provider_fw_rule_get_all(self):
         self.assertExpected('provider_fw_rule_get_all')
 
-    def test_agent_build_get_by_triple(self):
-        self.assertExpected('agent_build_get_by_triple',
-                            'fake-hv', 'gnu/hurd', 'fake-arch')
-
     def test_wait_for_instance_event(self):
         self.assertExpected('wait_for_instance_event',
                             'instance', ['event'])
@@ -73,13 +65,7 @@ class FakeVirtAPITest(VirtAPIBaseTest):
             self.assertTrue(run)
             return
 
-        if method == 'instance_update':
-            # NOTE(danms): instance_update actually becomes the other variant
-            # in FakeVirtAPI
-            db_method = 'instance_update_and_get_original'
-        else:
-            db_method = method
-        self.mox.StubOutWithMock(db, db_method)
+        self.mox.StubOutWithMock(db, method)
 
         if method in ('aggregate_metadata_add', 'aggregate_metadata_delete',
                       'security_group_rule_get_by_security_group'):
@@ -91,7 +77,7 @@ class FakeVirtAPITest(VirtAPIBaseTest):
         else:
             e_args = args
 
-        getattr(db, db_method)(self.context, *e_args, **kwargs).AndReturn(
+        getattr(db, method)(self.context, *e_args, **kwargs).AndReturn(
                 'it worked')
         self.mox.ReplayAll()
         result = getattr(self.virtapi, method)(self.context, *args, **kwargs)
@@ -107,11 +93,6 @@ class FakeCompute(object):
         self.instance_events.prepare_for_instance_event.side_effect = \
             self._prepare_for_instance_event
 
-    def _instance_update(self, context, instance_uuid, **kwargs):
-        # NOTE(danms): Fake this behavior from compute/manager::ComputeManager
-        return self.conductor_api.instance_update(context,
-                                                  instance_uuid, kwargs)
-
     def _event_waiter(self):
         event = mock.MagicMock()
         event.status = 'completed'
@@ -144,9 +125,9 @@ class ComputeVirtAPITest(VirtAPIBaseTest):
 
     def test_wait_for_instance_event(self):
         and_i_ran = ''
-        event_1_tag = external_event_obj.InstanceExternalEvent.make_key(
+        event_1_tag = objects.InstanceExternalEvent.make_key(
             'event1')
-        event_2_tag = external_event_obj.InstanceExternalEvent.make_key(
+        event_2_tag = objects.InstanceExternalEvent.make_key(
             'event2', 'tag')
         events = {
             'event1': event_1_tag,
