@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
 # Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
@@ -48,7 +46,7 @@ def stub_out(stubs, funcs):
     for func in funcs:
         func_name = '_'.join(func.__name__.split('_')[1:])
         stubs.Set(db, func_name, func)
-        stubs.Set(db.sqlalchemy.api, func_name, func)
+        stubs.Set(db.api, func_name, func)
 
 
 fixed_ip_fields = {'id': 0,
@@ -103,7 +101,8 @@ def stub_out_db_network_api(stubs):
     virtual_interfacees = [virtual_interface_fields]
     networks = [network_fields]
 
-    def fake_floating_ip_allocate_address(context, project_id, pool):
+    def fake_floating_ip_allocate_address(context, project_id, pool,
+                                          auto_assigned=False):
         ips = filter(lambda i: i['fixed_ip_id'] is None and
                                i['project_id'] is None and
                                i['pool'] == pool,
@@ -111,6 +110,7 @@ def stub_out_db_network_api(stubs):
         if not ips:
             raise exception.NoMoreFloatingIps()
         ips[0]['project_id'] = project_id
+        ips[0]['auto_assigned'] = auto_assigned
         return FakeModel(ips[0])
 
     def fake_floating_ip_deallocate(context, address):
@@ -187,7 +187,7 @@ def stub_out_db_network_api(stubs):
         ip['id'] = max([i['id'] for i in fixed_ips] or [-1]) + 1
         for key in values:
             ip[key] = values[key]
-        return ip['address']
+        return ip
 
     def fake_fixed_ip_disassociate(context, address):
         ips = filter(lambda i: i['address'] == address,
@@ -228,7 +228,7 @@ def stub_out_db_network_api(stubs):
                         continue
                     fixed_ip_fields['virtual_interface'] = FakeModel(vif[0])
 
-    def fake_instance_type_get(context, id):
+    def fake_flavor_get(context, id):
         if flavor_fields['id'] == id:
             return FakeModel(flavor_fields)
 
@@ -282,10 +282,6 @@ def stub_out_db_network_api(stubs):
         nets = filter(lambda n: n['host'] == host, networks)
         return [FakeModel(n) for n in nets]
 
-    def fake_network_get_all_by_instance(context, instance_id):
-        nets = filter(lambda n: n['instance_id'] == instance_id, networks)
-        return [FakeModel(n) for n in nets]
-
     def fake_network_set_host(context, network_id, host_id):
         nets = filter(lambda n: n['id'] == network_id, networks)
         for net in nets:
@@ -318,7 +314,7 @@ def stub_out_db_network_api(stubs):
              fake_fixed_ip_get_by_instance,
              fake_fixed_ip_get_by_address,
              fake_fixed_ip_update,
-             fake_instance_type_get,
+             fake_flavor_get,
              fake_virtual_interface_create,
              fake_virtual_interface_delete_by_instance,
              fake_virtual_interface_get_by_instance,
@@ -327,7 +323,6 @@ def stub_out_db_network_api(stubs):
              fake_network_get,
              fake_network_get_all,
              fake_network_get_all_by_host,
-             fake_network_get_all_by_instance,
              fake_network_set_host,
              fake_network_update,
              fake_project_get_networks]
@@ -374,7 +369,7 @@ def stub_out_db_instance_api(stubs, injected=True):
                          name='m1.large',
                          memory_mb=8192,
                          vcpus=4,
-                         vcpu_weight=None,
+                         vcpu_weight=10,
                          root_gb=80,
                          ephemeral_gb=80,
                          flavorid=4,
@@ -392,63 +387,27 @@ def stub_out_db_instance_api(stubs, injected=True):
                  rxtx_factor=1.0,
                  swap=0)}
 
-    flat_network_fields = {'id': 'fake_flat',
-                           'bridge': 'xenbr0',
-                           'label': 'fake_flat_network',
-                           'netmask': '255.255.255.0',
-                           'cidr_v6': 'fe80::a00:0/120',
-                           'netmask_v6': '120',
-                           'gateway': '10.0.0.1',
-                           'gateway_v6': 'fe80::a00:1',
-                           'broadcast': '10.0.0.255',
-                           'dns': '10.0.0.2',
-                           'ra_server': None,
-                           'injected': injected}
-
-    vlan_network_fields = {'id': 'fake_vlan',
-                           'bridge': 'br111',
-                           'label': 'fake_vlan_network',
-                           'netmask': '255.255.255.0',
-                           'cidr_v6': 'fe80::a00:0/120',
-                           'netmask_v6': '120',
-                           'gateway': '10.0.0.1',
-                           'gateway_v6': 'fe80::a00:1',
-                           'broadcast': '10.0.0.255',
-                           'dns': '10.0.0.2',
-                           'ra_server': None,
-                           'vlan': 111,
-                           'injected': False}
-
     fixed_ip_fields = {'address': '10.0.0.3',
                        'address_v6': 'fe80::a00:3',
                        'network_id': 'fake_flat'}
 
-    def fake_instance_type_get_all(context, inactive=0, filters=None):
+    def fake_flavor_get_all(context, inactive=0, filters=None):
         return INSTANCE_TYPES.values()
 
-    def fake_instance_type_get_by_name(context, name):
+    def fake_flavor_get_by_name(context, name):
         return INSTANCE_TYPES[name]
 
-    def fake_instance_type_get(context, id):
+    def fake_flavor_get(context, id):
         for name, inst_type in INSTANCE_TYPES.iteritems():
             if str(inst_type['id']) == str(id):
                 return inst_type
         return None
 
-    def fake_network_get_all_by_instance(context, instance_id):
-        # Even instance numbers are on vlan networks
-        if instance_id % 2 == 0:
-            return [FakeModel(vlan_network_fields)]
-        else:
-            return [FakeModel(flat_network_fields)]
-
     def fake_fixed_ip_get_by_instance(context, instance_id):
         return [FakeModel(fixed_ip_fields)]
 
-    funcs = [fake_network_get_all_by_instance,
-             fake_instance_type_get_all,
-             fake_instance_type_get_by_name,
-             fake_instance_type_get,
-             fake_network_get_all_by_instance,
+    funcs = [fake_flavor_get_all,
+             fake_flavor_get_by_name,
+             fake_flavor_get,
              fake_fixed_ip_get_by_instance]
     stub_out(stubs, funcs)
