@@ -17,7 +17,7 @@ import uuid
 from oslo.config import cfg
 
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
 
@@ -25,8 +25,8 @@ from nova.openstack.common import timeutils
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
-CONF.import_opt('cinder_cross_az_attach',
-                'nova.volume.cinder')
+CONF.import_opt('cross_az_attach',
+                'nova.volume.cinder', group='cinder')
 
 
 class fake_volume():
@@ -162,7 +162,7 @@ class API(object):
         return v.vol
 
     def get(self, context, volume_id):
-        if volume_id == 87654321:
+        if str(volume_id) == '87654321':
             return {'id': volume_id,
                     'attach_time': '13:56:24',
                     'attach_status': 'attached',
@@ -190,7 +190,7 @@ class API(object):
         if volume['attach_status'] == 'attached':
             msg = _("already attached")
             raise exception.InvalidVolume(reason=msg)
-        if instance and not CONF.cinder_cross_az_attach:
+        if instance and not CONF.cinder.cross_az_attach:
             if instance['availability_zone'] != volume['availability_zone']:
                 msg = _("Instance and volume not in same availability_zone")
                 raise exception.InvalidVolume(reason=msg)
