@@ -19,7 +19,7 @@ import re
 from lxml import etree
 import six
 
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import importutils
 from nova.openstack.common import jsonutils
 from nova import test
@@ -197,7 +197,8 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
             if isinstance(expected, six.string_types):
                 # NOTE(danms): Ignore whitespace in this comparison
                 expected = expected.strip()
-                result = result.strip()
+                if isinstance(result, six.string_types):
+                    result = result.strip()
             if expected != result:
                 raise NoMatch(
                         _('Values do not match:\n'
@@ -219,8 +220,8 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
         return subs
 
     def _verify_response(self, name, subs, response, exp_code):
-        self.assertEqual(response.status, exp_code)
-        response_data = response.read()
+        self.assertEqual(response.status_code, exp_code)
+        response_data = response.content
         response_data = self._pretty_data(response_data)
         if not os.path.exists(self._get_template(name)):
             self._write_template(name, response_data)
@@ -266,14 +267,16 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
             text = r'(\\"|[^"])*'
         else:
             text = r'[^<]*'
-        # NOTE(treinish): Could result in a false positive, but it
-        # shouldn't be an issue for this case.
-        timestamp_re = ('\d{4}-[0,1]\d-[0-3]\d[ ,T]'
-                        '\d{2}:\d{2}:\d{2}'
-                        '(Z|(\+|-)\d{2}:\d{2}|\.\d{6}|)')
+        isotime_re = '\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}Z'
+        strtime_re = '\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}\.\d{6}'
+        xmltime_re = ('\d{4}-[0,1]\d-[0-3]\d '
+                      '\d{2}:\d{2}:\d{2}'
+                      '(\.\d{6})?(\+00:00)?')
         return {
-            'timestamp': timestamp_re,
-            'timestamp_or_none': r'None|%s' % timestamp_re,
+            'isotime': isotime_re,
+            'strtime': strtime_re,
+            'strtime_or_none': r'None|%s' % strtime_re,
+            'xmltime': xmltime_re,
             'password': '[0-9a-zA-Z]{1,12}',
             'ip': '[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}',
             'ip6': '([0-9a-zA-Z]{1,4}:){1,7}:?[0-9a-zA-Z]{1,4}',
@@ -286,7 +289,7 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
                            '[a-zA-Z0-9\n/+=]*'
                            '-----END RSA PRIVATE KEY-----',
             'public_key': 'ssh-rsa[ a-zA-Z0-9/+=]*'
-                          'Generated by Nova',
+                          'Generated-by-Nova',
             'fingerprint': '([0-9a-f]{2}:){15}[0-9a-f]{2}',
             'host': self._get_host(),
             'host_name': '[0-9a-z]{32}',
