@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import mock
+
 from nova.compute import api as compute_api
 from nova.compute import manager as compute_manager
 from nova.servicegroup import api as service_group_api
@@ -22,14 +24,9 @@ from nova.tests.integrated.v3 import test_servers
 class EvacuateJsonTest(test_servers.ServersSampleBase):
     extension_name = "os-evacuate"
 
-    def test_server_evacuate(self):
-        uuid = self._post_server()
-
-        req_subs = {
-            'host': self.compute.host,
-            "adminPass": "MySecretPass",
-            "onSharedStorage": 'False'
-        }
+    def _test_evacuate(self, req_subs, server_req, server_resp,
+                       expected_resp_code):
+        self.uuid = self._post_server()
 
         def fake_service_is_up(self, service):
             """Simulate validation of instance host is down."""
@@ -55,7 +52,40 @@ class EvacuateJsonTest(test_servers.ServersSampleBase):
                        '_check_instance_exists',
                        fake_check_instance_exists)
 
-        response = self._do_post('servers/%s/action' % uuid,
-                                 'server-evacuate-req', req_subs)
+        response = self._do_post('servers/%s/action' % self.uuid,
+                                 server_req, req_subs)
         subs = self._get_regexes()
-        self._verify_response('server-evacuate-resp', subs, response, 202)
+        self._verify_response(server_resp, subs, response, expected_resp_code)
+
+    @mock.patch('nova.conductor.manager.ComputeTaskManager.rebuild_instance')
+    def test_server_evacuate(self, rebuild_mock):
+        # Note (wingwj): The host can't be the same one
+        req_subs = {
+            'host': 'testHost',
+            "adminPass": "MySecretPass",
+            "onSharedStorage": 'False'
+        }
+        self._test_evacuate(req_subs, 'server-evacuate-req',
+                            'server-evacuate-resp', 200)
+        rebuild_mock.assert_called_once_with(mock.ANY, instance=mock.ANY,
+                orig_image_ref=mock.ANY, image_ref=mock.ANY,
+                injected_files=mock.ANY, new_pass="MySecretPass",
+                orig_sys_metadata=mock.ANY, bdms=mock.ANY, recreate=mock.ANY,
+                on_shared_storage=False, preserve_ephemeral=mock.ANY,
+                host='testHost')
+
+    @mock.patch('nova.conductor.manager.ComputeTaskManager.rebuild_instance')
+    def test_server_evacuate_find_host(self, rebuild_mock):
+        req_subs = {
+            "adminPass": "MySecretPass",
+            "onSharedStorage": 'False'
+        }
+        self._test_evacuate(req_subs, 'server-evacuate-find-host-req',
+                            'server-evacuate-find-host-resp', 200)
+
+        rebuild_mock.assert_called_once_with(mock.ANY, instance=mock.ANY,
+                orig_image_ref=mock.ANY, image_ref=mock.ANY,
+                injected_files=mock.ANY, new_pass="MySecretPass",
+                orig_sys_metadata=mock.ANY, bdms=mock.ANY, recreate=mock.ANY,
+                on_shared_storage=False, preserve_ephemeral=mock.ANY,
+                host=None)
