@@ -68,5 +68,5 @@ class MigrationsSamplesJsonTest(api_sample_base.ApiSampleTestBaseV3):
         response = self._do_get('os-migrations')
         subs = self._get_regexes()
 
-        self.assertEqual(response.status, 200)
+        self.assertEqual(response.status_code, 200)
         self._verify_response('migrations-get', subs, response, 200)
