@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import mock
+
 from nova import db
 from nova.openstack.common import jsonutils
 from nova.tests.integrated.v3 import api_sample_base
@@ -33,6 +35,7 @@ fake_db_dev_1 = {
     'dev_id': 'pci_0000_04_10_0',
     'label': 'label_8086_1520',
     'instance_uuid': '69ba1044-0766-4ec0-b60d-09595de034a1',
+    'request_id': None,
     'extra_info': '{"key1": "value1", "key2": "value2"}'
     }
 
@@ -51,6 +54,7 @@ fake_db_dev_2 = {
     'dev_id': 'pci_0000_04_10_1',
     'label': 'label_8086_1520',
     'instance_uuid': 'd5b446a6-a1b4-4d01-b4f0-eac37b3a62fc',
+    'request_id': None,
     'extra_info': '{"key3": "value3", "key4": "value4"}'
     }
 
@@ -83,19 +87,23 @@ class ExtendedHyervisorPciSampleJsonTest(api_sample_base.ApiSampleTestBaseV3):
                                   "current_workload": 0,
                                   "disk_available_least": 0,
                                   "host_ip": "1.1.1.1",
+                                  "state": "up",
+                                  "status": "enabled",
                                   "free_disk_gb": 1028,
                                   "free_ram_mb": 7680,
                                   "hypervisor_hostname": "fake-mini",
                                   "hypervisor_type": "fake",
-                                  "hypervisor_version": 1,
+                                  "hypervisor_version": 1000,
                                   "id": 1,
                                   "local_gb": 1028,
                                   "local_gb_used": 0,
                                   "memory_mb": 8192,
                                   "memory_mb_used": 512,
                                   "running_vms": 0,
-                                  "service": {"host": '043b3cacf6f34c90a724'
-                                                      '5151fc8ebcda'},
+                                  "service": {"host": '043b3cacf6f34c90a'
+                                                      '7245151fc8ebcda',
+                                              "disabled": False,
+                                              "disabled_reason": None},
                                   "vcpus": 1,
                                   "vcpus_used": 0,
                                   "service_id": 2,
@@ -110,13 +118,12 @@ class ExtendedHyervisorPciSampleJsonTest(api_sample_base.ApiSampleTestBaseV3):
                                                             ' "0x1"]]',
                                             "key1": "value1"}}]}
 
-    def test_pci_show(self):
-        def fake_compute_node_get(context, id):
-            self.fake_compute_node['pci_stats'] = jsonutils.dumps(
-                self.fake_compute_node['pci_stats'])
-            return self.fake_compute_node
-
-        self.stubs.Set(db, 'compute_node_get', fake_compute_node_get)
+    @mock.patch("nova.servicegroup.API.service_is_up", return_value=True)
+    @mock.patch("nova.db.compute_node_get")
+    def test_pci_show(self, mock_db, mock_service):
+        self.fake_compute_node['pci_stats'] = jsonutils.dumps(
+            self.fake_compute_node['pci_stats'])
+        mock_db.return_value = self.fake_compute_node
         hypervisor_id = 1
         response = self._do_get('os-hypervisors/%s' % hypervisor_id)
         subs = {
@@ -126,13 +133,13 @@ class ExtendedHyervisorPciSampleJsonTest(api_sample_base.ApiSampleTestBaseV3):
         self._verify_response('hypervisors-pci-show-resp',
                               subs, response, 200)
 
-    def test_pci_detail(self):
-        def fake_compute_node_get_all(context):
-            self.fake_compute_node['pci_stats'] = jsonutils.dumps(
-                self.fake_compute_node['pci_stats'])
-            return [self.fake_compute_node]
+    @mock.patch("nova.servicegroup.API.service_is_up", return_value=True)
+    @mock.patch("nova.db.compute_node_get_all")
+    def test_pci_detail(self, mock_db, mock_service):
+        self.fake_compute_node['pci_stats'] = jsonutils.dumps(
+            self.fake_compute_node['pci_stats'])
 
-        self.stubs.Set(db, 'compute_node_get_all', fake_compute_node_get_all)
+        mock_db.return_value = [self.fake_compute_node]
         hypervisor_id = 1
         subs = {
             'hypervisor_id': hypervisor_id
