@@ -25,12 +25,12 @@ class RescueJsonTest(test_servers.ServersSampleBase):
         }
         response = self._do_post('servers/%s/action' % uuid,
                                  'server-rescue-req', req_subs)
-        self._verify_response('server-rescue', req_subs, response, 202)
+        self._verify_response('server-rescue', req_subs, response, 200)
 
     def _unrescue(self, uuid):
         response = self._do_post('servers/%s/action' % uuid,
                                  'server-unrescue-req', {})
-        self.assertEqual(response.status, 202)
+        self.assertEqual(response.status_code, 202)
 
     def test_server_rescue(self):
         uuid = self._post_server()
@@ -46,6 +46,26 @@ class RescueJsonTest(test_servers.ServersSampleBase):
 
         self._verify_response('server-get-resp-rescue', subs, response, 200)
 
+    def test_server_rescue_with_image_ref_specified(self):
+        uuid = self._post_server()
+
+        req_subs = {
+            'password': 'MySecretPass',
+            'image_ref': '2341-Abc'
+        }
+        response = self._do_post('servers/%s/action' % uuid,
+                                 'server-rescue-req-with-image-ref', req_subs)
+        self._verify_response('server-rescue', req_subs, response, 200)
+
+        # Do a server get to make sure that the 'RESCUE' state is set
+        response = self._do_get('servers/%s' % uuid)
+        subs = self._get_regexes()
+        subs['hostid'] = '[a-f0-9]+'
+        subs['id'] = uuid
+        subs['status'] = 'RESCUE'
+
+        self._verify_response('server-get-resp-rescue', subs, response, 200)
+
     def test_server_unrescue(self):
         uuid = self._post_server()
 
