@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # Copyright 2012 Nebula, Inc.
 # Copyright 2013 IBM Corp.
 #
@@ -46,7 +45,7 @@ class ServersMetadataJsonTest(test_servers.ServersSampleBase):
         response = self._do_post('servers/%s/metadata' % uuid,
                                  'server-metadata-all-req',
                                  subs)
-        self._verify_response('server-metadata-all-resp', subs, response, 201)
+        self._verify_response('server-metadata-all-resp', subs, response, 200)
 
     def test_metadata_get_all(self):
         # Test getting all metadata for a server.
@@ -77,9 +76,5 @@ class ServersMetadataJsonTest(test_servers.ServersSampleBase):
         subs = {'value': 'Foo Value'}
         uuid = self._create_and_set(subs)
         response = self._do_delete('servers/%s/metadata/foo' % uuid)
-        self.assertEqual(response.status, 204)
-        self.assertEqual(response.read(), '')
-
-
-class ServersMetadataXmlTest(ServersMetadataJsonTest):
-    ctype = 'xml'
+        self.assertEqual(response.status_code, 204)
+        self.assertEqual(response.content, '')
