@@ -16,17 +16,26 @@ Tests For Scheduler Host Filters.
 """
 
 from oslo.config import cfg
+import six
 
+from nova.compute import arch
+from nova.compute import hvtype
+from nova.compute import vm_mode
 from nova import context
 from nova import db
+from nova import objects
+from nova.objects import base as obj_base
 from nova.openstack.common import jsonutils
 from nova.pci import pci_stats
 from nova.scheduler import filters
 from nova.scheduler.filters import extra_specs_ops
+from nova.scheduler.filters import ram_filter
 from nova import servicegroup
 from nova import test
+from nova.tests import fake_instance
 from nova.tests.scheduler import fakes
 from nova import utils
+from nova.virt import hardware
 
 CONF = cfg.CONF
 
@@ -448,15 +457,15 @@ class HostFiltersTestCase(test.NoDBTestCase):
         service = {'disabled': False}
         host = fakes.FakeHostState('fake_host', 'fake_node',
                 {'service': service})
-        #True since empty
+        # True since empty
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
         fakes.FakeInstance(context=self.context,
                            params={'host': 'fake_host', 'instance_type_id': 1})
-        #True since same type
+        # True since same type
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
-        #False since different type
+        # False since different type
         self.assertFalse(filt_cls.host_passes(host, filter2_properties))
-        #False since node not homogeneous
+        # False since node not homogeneous
         fakes.FakeInstance(context=self.context,
                            params={'host': 'fake_host', 'instance_type_id': 2})
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
@@ -472,19 +481,19 @@ class HostFiltersTestCase(test.NoDBTestCase):
         service = {'disabled': False}
         host = fakes.FakeHostState('fake_host', 'fake_node',
                 {'service': service})
-        #True since no aggregates
+        # True since no aggregates
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
-        #True since type matches aggregate, metadata
+        # True since type matches aggregate, metadata
         self._create_aggregate_with_host(name='fake_aggregate',
                 hosts=['fake_host'], metadata={'instance_type': 'fake1'})
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
-        #False since type matches aggregate, metadata
+        # False since type matches aggregate, metadata
         self.assertFalse(filt_cls.host_passes(host, filter2_properties))
 
     def test_ram_filter_fails_on_memory(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['RamFilter']()
-        self.flags(ram_allocation_ratio=1.0)
+        ram_filter.RamFilter.ram_allocation_ratio = 1.0
         filter_properties = {'instance_type': {'memory_mb': 1024}}
         service = {'disabled': False}
         host = fakes.FakeHostState('host1', 'node1',
@@ -495,7 +504,7 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_ram_filter_passes(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['RamFilter']()
-        self.flags(ram_allocation_ratio=1.0)
+        ram_filter.RamFilter.ram_allocation_ratio = 1.0
         filter_properties = {'instance_type': {'memory_mb': 1024}}
         service = {'disabled': False}
         host = fakes.FakeHostState('host1', 'node1',
@@ -506,7 +515,7 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_ram_filter_oversubscribe(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['RamFilter']()
-        self.flags(ram_allocation_ratio=2.0)
+        ram_filter.RamFilter.ram_allocation_ratio = 2.0
         filter_properties = {'instance_type': {'memory_mb': 1024}}
         service = {'disabled': False}
         host = fakes.FakeHostState('host1', 'node1',
@@ -651,14 +660,15 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_passes_same_inst_props_and_version(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'_architecture': 'x86_64',
-                                    'hypervisor_type': 'kvm',
-                                    'vm_mode': 'hvm',
+        img_props = {'properties': {'_architecture': arch.X86_64,
+                                    'hypervisor_type': hvtype.KVM,
+                                    'vm_mode': vm_mode.HVM,
                                     'hypervisor_version_requires': '>=6.0,<6.2'
         }}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'kvm', 'hvm')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
@@ -666,12 +676,13 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_fails_different_inst_props(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'arm',
-                                    'hypervisor_type': 'qemu',
-                                    'vm_mode': 'hvm'}}
+        img_props = {'properties': {'architecture': arch.ARMV7,
+                                    'hypervisor_type': hvtype.QEMU,
+                                    'vm_mode': vm_mode.HVM}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'kvm', 'hvm')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
@@ -679,14 +690,15 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_fails_different_hyper_version(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'hypervisor_type': 'kvm',
-                                    'vm_mode': 'hvm',
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'hypervisor_type': hvtype.KVM,
+                                    'vm_mode': vm_mode.HVM,
                                     'hypervisor_version_requires': '>=6.2'}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
         capabilities = {'enabled': True,
-                        'supported_instances': [('x86_64', 'kvm', 'hvm')],
+                        'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
@@ -694,11 +706,12 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_passes_partial_inst_props(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'vm_mode': 'hvm'}}
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'vm_mode': vm_mode.HVM}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'kvm', 'hvm')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
@@ -706,11 +719,12 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_fails_partial_inst_props(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'vm_mode': 'hvm'}}
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'vm_mode': vm_mode.HVM}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'xen', 'xen')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.XEN, vm_mode.XEN)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
@@ -720,7 +734,8 @@ class HostFiltersTestCase(test.NoDBTestCase):
         filt_cls = self.class_map['ImagePropertiesFilter']()
         filter_properties = {'request_spec': {}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'kvm', 'hvm')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
@@ -728,9 +743,9 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_fails_without_host_props(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'hypervisor_type': 'kvm',
-                                    'vm_mode': 'hvm'}}
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'hypervisor_type': hvtype.KVM,
+                                    'vm_mode': vm_mode.HVM}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
         capabilities = {'enabled': True,
@@ -741,26 +756,28 @@ class HostFiltersTestCase(test.NoDBTestCase):
     def test_image_properties_filter_passes_without_hyper_version(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'hypervisor_type': 'kvm',
-                                    'vm_mode': 'hvm',
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'hypervisor_type': hvtype.KVM,
+                                    'vm_mode': vm_mode.HVM,
                                     'hypervisor_version_requires': '>=6.0'}}
         filter_properties = {'request_spec': {'image': img_props}}
         capabilities = {'enabled': True,
-                        'supported_instances': [('x86_64', 'kvm', 'hvm')]}
+                        'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)]}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
 
     def test_image_properties_filter_fails_with_unsupported_hyper_ver(self):
         self._stub_service_is_up(True)
         filt_cls = self.class_map['ImagePropertiesFilter']()
-        img_props = {'properties': {'architecture': 'x86_64',
-                                    'hypervisor_type': 'kvm',
-                                    'vm_mode': 'hvm',
+        img_props = {'properties': {'architecture': arch.X86_64,
+                                    'hypervisor_type': hvtype.KVM,
+                                    'vm_mode': vm_mode.HVM,
                                     'hypervisor_version_requires': '>=6.0'}}
         filter_properties = {'request_spec': {'image': img_props}}
         capabilities = {'enabled': True,
-                        'supported_instances': [('x86_64', 'kvm', 'hvm')],
+                        'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
                         'hypervisor_version': 5000}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
@@ -772,7 +789,8 @@ class HostFiltersTestCase(test.NoDBTestCase):
         img_props = {'properties': {'vm_mode': 'pv'}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'xapi', 'xen')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.XEN, vm_mode.XEN)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
@@ -784,7 +802,48 @@ class HostFiltersTestCase(test.NoDBTestCase):
         img_props = {'properties': {'vm_mode': 'hv'}}
         filter_properties = {'request_spec': {'image': img_props}}
         hypervisor_version = utils.convert_version_to_int('6.0.0')
-        capabilities = {'supported_instances': [('x86_64', 'kvm', 'hvm')],
+        capabilities = {'supported_instances':
+                        [(arch.X86_64, hvtype.KVM, vm_mode.HVM)],
+                        'hypervisor_version': hypervisor_version}
+        host = fakes.FakeHostState('host1', 'node1', capabilities)
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_image_properties_filter_xen_arch_compat(self):
+        # if an old image has 'x86_32' for arch it should be treated as i686
+        self._stub_service_is_up(True)
+        filt_cls = self.class_map['ImagePropertiesFilter']()
+        img_props = {'properties': {'architecture': 'x86_32'}}
+        filter_properties = {'request_spec': {'image': img_props}}
+        hypervisor_version = utils.convert_version_to_int('6.0.0')
+        capabilities = {'supported_instances':
+                        [(arch.I686, hvtype.KVM, vm_mode.HVM)],
+                        'hypervisor_version': hypervisor_version}
+        host = fakes.FakeHostState('host1', 'node1', capabilities)
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_image_properties_filter_xen_hvtype_compat(self):
+        # if an old image has 'xapi' for hvtype it should be treated as xen
+        self._stub_service_is_up(True)
+        filt_cls = self.class_map['ImagePropertiesFilter']()
+        img_props = {'properties': {'hypervisor_type': 'xapi'}}
+        filter_properties = {'request_spec': {'image': img_props}}
+        hypervisor_version = utils.convert_version_to_int('6.0.0')
+        capabilities = {'supported_instances':
+                        [(arch.I686, hvtype.XEN, vm_mode.HVM)],
+                        'hypervisor_version': hypervisor_version}
+        host = fakes.FakeHostState('host1', 'node1', capabilities)
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_image_properties_filter_baremetal_vmmode_compat(self):
+        # if an old image has 'baremetal' for vmmode it should be
+        # treated as hvm
+        self._stub_service_is_up(True)
+        filt_cls = self.class_map['ImagePropertiesFilter']()
+        img_props = {'properties': {'vm_mode': 'baremetal'}}
+        filter_properties = {'request_spec': {'image': img_props}}
+        hypervisor_version = utils.convert_version_to_int('6.0.0')
+        capabilities = {'supported_instances':
+                        [(arch.I686, hvtype.BAREMETAL, vm_mode.HVM)],
                         'hypervisor_version': hypervisor_version}
         host = fakes.FakeHostState('host1', 'node1', capabilities)
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
@@ -806,6 +865,28 @@ class HostFiltersTestCase(test.NoDBTestCase):
         assertion = self.assertTrue if passes else self.assertFalse
         assertion(filt_cls.host_passes(host, filter_properties))
 
+    def test_compute_filter_pass_cpu_info_as_text_type(self):
+        cpu_info = """ { "vendor": "Intel", "model": "core2duo",
+        "arch": "i686","features": ["lahf_lm", "rdtscp"], "topology":
+        {"cores": 1, "threads":1, "sockets": 1}} """
+
+        cpu_info = six.text_type(cpu_info)
+
+        self._do_test_compute_filter_extra_specs(
+                ecaps={'cpu_info': cpu_info},
+            especs={'capabilities:cpu_info:vendor': 'Intel'},
+            passes=True)
+
+    def test_compute_filter_fail_cpu_info_as_text_type_not_valid(self):
+        cpu_info = "cpu_info"
+
+        cpu_info = six.text_type(cpu_info)
+
+        self._do_test_compute_filter_extra_specs(
+                ecaps={'cpu_info': cpu_info},
+            especs={'capabilities:cpu_info:vendor': 'Intel'},
+            passes=False)
+
     def test_compute_filter_passes_extra_specs_simple(self):
         self._do_test_compute_filter_extra_specs(
                 ecaps={'stats': {'opt1': 1, 'opt2': 2}},
@@ -1537,7 +1618,9 @@ class HostFiltersTestCase(test.NoDBTestCase):
 
     def test_pci_passthrough_pass(self):
         filt_cls = self.class_map['PciPassthroughFilter']()
-        requests = [{'count': 1, 'spec': [{'vendor_id': '8086'}]}]
+        request = objects.InstancePCIRequest(count=1,
+            spec=[{'vendor_id': '8086'}])
+        requests = objects.InstancePCIRequests(requests=[request])
         filter_properties = {'pci_requests': requests}
         self.stubs.Set(pci_stats.PciDeviceStats, 'support_requests',
                        self._fake_pci_support_requests)
@@ -1546,11 +1629,13 @@ class HostFiltersTestCase(test.NoDBTestCase):
             attribute_dict={'pci_stats': pci_stats.PciDeviceStats()})
         self.pci_request_result = True
         self.assertTrue(filt_cls.host_passes(host, filter_properties))
-        self.assertEqual(self.pci_requests, requests)
+        self.assertEqual(self.pci_requests, requests.requests)
 
     def test_pci_passthrough_fail(self):
         filt_cls = self.class_map['PciPassthroughFilter']()
-        requests = [{'count': 1, 'spec': [{'vendor_id': '8086'}]}]
+        request = objects.InstancePCIRequest(count=1,
+            spec=[{'vendor_id': '8086'}])
+        requests = objects.InstancePCIRequests(requests=[request])
         filter_properties = {'pci_requests': requests}
         self.stubs.Set(pci_stats.PciDeviceStats, 'support_requests',
                        self._fake_pci_support_requests)
@@ -1559,7 +1644,7 @@ class HostFiltersTestCase(test.NoDBTestCase):
             attribute_dict={'pci_stats': pci_stats.PciDeviceStats()})
         self.pci_request_result = False
         self.assertFalse(filt_cls.host_passes(host, filter_properties))
-        self.assertEqual(self.pci_requests, requests)
+        self.assertEqual(self.pci_requests, requests.requests)
 
     def test_pci_passthrough_no_pci_request(self):
         filt_cls = self.class_map['PciPassthroughFilter']()
@@ -1697,3 +1782,226 @@ class HostFiltersTestCase(test.NoDBTestCase):
                                    attribute_dict={'metrics': metrics})
         filt_cls = self.class_map['MetricsFilter']()
         self.assertFalse(filt_cls.host_passes(host, None))
+
+    def test_aggregate_filter_num_iops_value(self):
+        self.flags(max_io_ops_per_host=7)
+        filt_cls = self.class_map['AggregateIoOpsFilter']()
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'num_io_ops': 7})
+        filter_properties = {'context': self.context}
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+        self._create_aggregate_with_host(
+            name='fake_aggregate',
+            hosts=['host1'],
+            metadata={'max_io_ops_per_host': 8})
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_aggregate_filter_num_iops_value_error(self):
+        self.flags(max_io_ops_per_host=8)
+        filt_cls = self.class_map['AggregateIoOpsFilter']()
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'num_io_ops': 7})
+        self._create_aggregate_with_host(
+            name='fake_aggregate',
+            hosts=['host1'],
+            metadata={'max_io_ops_per_host': 'XXX'})
+        filter_properties = {'context': self.context}
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_aggregate_disk_filter_value_error(self):
+        self._stub_service_is_up(True)
+        filt_cls = self.class_map['AggregateDiskFilter']()
+        self.flags(disk_allocation_ratio=1.0)
+        filter_properties = {
+            'context': self.context,
+            'instance_type': {'root_gb': 1,
+                              'ephemeral_gb': 1,
+                              'swap': 1024}}
+        service = {'disabled': False}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'free_disk_mb': 3 * 1024,
+                                    'total_usable_disk_gb': 1,
+                                   'service': service})
+        self._create_aggregate_with_host(name='fake_aggregate',
+                hosts=['host1'],
+                metadata={'disk_allocation_ratio': 'XXX'})
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_aggregate_disk_filter_default_value(self):
+        self._stub_service_is_up(True)
+        filt_cls = self.class_map['AggregateDiskFilter']()
+        self.flags(disk_allocation_ratio=1.0)
+        filter_properties = {
+            'context': self.context,
+            'instance_type': {'root_gb': 2,
+                              'ephemeral_gb': 1,
+                              'swap': 1024}}
+        service = {'disabled': False}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'free_disk_mb': 3 * 1024,
+                                    'total_usable_disk_gb': 1,
+                                   'service': service})
+        # Uses global conf.
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+
+        # Uses an aggregate with ratio
+        self._create_aggregate_with_host(
+            name='fake_aggregate',
+            hosts=['host1'],
+            metadata={'disk_allocation_ratio': '2'})
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_filter_aggregate_num_instances_value(self):
+        self.flags(max_instances_per_host=4)
+        filt_cls = self.class_map['AggregateNumInstancesFilter']()
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'num_instances': 5})
+        filter_properties = {'context': self.context}
+        # No aggregate defined for that host.
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+        self._create_aggregate_with_host(
+            name='fake_aggregate',
+            hosts=['host1'],
+            metadata={'max_instances_per_host': 6})
+        # Aggregate defined for that host.
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_filter_aggregate_num_instances_value_error(self):
+        self.flags(max_instances_per_host=6)
+        filt_cls = self.class_map['AggregateNumInstancesFilter']()
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'num_instances': 5})
+        filter_properties = {'context': self.context}
+        self._create_aggregate_with_host(
+            name='fake_aggregate',
+            hosts=['host1'],
+            metadata={'max_instances_per_host': 'XXX'})
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_pass(self):
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 512),
+                   hardware.VirtNUMATopologyCell(1, set([3]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_numa_instance_no_numa_host_fail(self):
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 512),
+                   hardware.VirtNUMATopologyCell(1, set([3]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1', {})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_numa_host_no_numa_instance_pass(self):
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = None
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_fail_fit(self):
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 512),
+                   hardware.VirtNUMATopologyCell(1, set([2]), 512),
+                   hardware.VirtNUMATopologyCell(2, set([3]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_fail_memory(self):
+        self.flags(ram_allocation_ratio=1)
+
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 1024),
+                   hardware.VirtNUMATopologyCell(1, set([3]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_fail_cpu(self):
+        self.flags(cpu_allocation_ratio=1)
+
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 512),
+                   hardware.VirtNUMATopologyCell(1, set([3, 4, 5]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertFalse(filt_cls.host_passes(host, filter_properties))
+
+    def test_numa_topology_filter_pass_set_limit(self):
+        self.flags(cpu_allocation_ratio=21)
+        self.flags(ram_allocation_ratio=1.3)
+
+        instance_topology = hardware.VirtNUMAInstanceTopology(
+            cells=[hardware.VirtNUMATopologyCell(0, set([1]), 512),
+                   hardware.VirtNUMATopologyCell(1, set([3]), 512)])
+        instance = fake_instance.fake_instance_obj(self.context)
+        instance.numa_topology = (
+                objects.InstanceNUMATopology.obj_from_topology(
+                    instance_topology))
+        filter_properties = {
+            'request_spec': {
+                'instance_properties': jsonutils.to_primitive(
+                    obj_base.obj_to_primitive(instance))}}
+        host = fakes.FakeHostState('host1', 'node1',
+                                   {'numa_topology': fakes.NUMA_TOPOLOGY})
+        filt_cls = self.class_map['NUMATopologyFilter']()
+        self.assertTrue(filt_cls.host_passes(host, filter_properties))
+        limits_topology = hardware.VirtNUMALimitTopology.from_json(
+                host.limits['numa_topology'])
+        self.assertEqual(limits_topology.cells[0].cpu_limit, 42)
+        self.assertEqual(limits_topology.cells[1].cpu_limit, 42)
+        self.assertEqual(limits_topology.cells[0].memory_limit, 665)
+        self.assertEqual(limits_topology.cells[1].memory_limit, 665)
