@@ -21,8 +21,8 @@ from oslo.config import cfg
 
 from nova.compute import flavors
 from nova.compute import utils as compute_utils
-from nova.conductor import api as conductor_api
 from nova import db
+from nova import exception
 from nova import notifications
 from nova import rpc
 from nova.scheduler import utils as scheduler_utils
@@ -81,7 +81,6 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
 
         self.mox.StubOutWithMock(rpc, 'get_notifier')
         notifier = self.mox.CreateMockAnything()
-        rpc.get_notifier('conductor', CONF.host).AndReturn(notifier)
         rpc.get_notifier(service).AndReturn(notifier)
 
         old_ref = 'old_ref'
@@ -94,7 +93,6 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
                                       service=service)
             compute_utils.add_instance_fault_from_exc(
                     self.context,
-                    mox.IsA(conductor_api.LocalAPI),
                     new_ref, exc_info, mox.IsA(tuple))
 
             payload = dict(request_spec=request_spec,
@@ -190,6 +188,18 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
     def test_populate_filter_props_force_nodes_no_retry(self):
         self._test_populate_filter_props(force_nodes=['force-node'])
 
+    @mock.patch.object(scheduler_utils, '_max_attempts')
+    def test_populate_retry_exception_at_max_attempts(self, _max_attempts):
+        _max_attempts.return_value = 2
+        msg = 'The exception text was preserved!'
+        filter_properties = dict(retry=dict(num_attempts=2, hosts=[],
+                                            exc=[msg]))
+        nvh = self.assertRaises(exception.NoValidHost,
+                                scheduler_utils.populate_retry,
+                                filter_properties, 'fake-uuid')
+        # make sure 'msg' is a substring of the complete exception text
+        self.assertIn(msg, nvh.message)
+
     def _check_parse_options(self, opts, sep, converter, expected):
         good = scheduler_utils.parse_options(opts,
                                              sep=sep,
@@ -218,3 +228,9 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
                                   '=',
                                   float,
                                   [('bar', -2.1)])
+
+    def test_validate_filters_configured(self):
+        self.flags(scheduler_default_filters='FakeFilter1,FakeFilter2')
+        self.assertTrue(scheduler_utils.validate_filter('FakeFilter1'))
+        self.assertTrue(scheduler_utils.validate_filter('FakeFilter2'))
+        self.assertFalse(scheduler_utils.validate_filter('FakeFilter3'))
