@@ -19,7 +19,6 @@
 import base64
 import hashlib
 import hmac
-import json
 import re
 
 try:
@@ -27,6 +26,7 @@ try:
 except ImportError:
     import pickle
 
+import mock
 from oslo.config import cfg
 import webob
 
@@ -41,7 +41,8 @@ from nova import db
 from nova.db.sqlalchemy import api
 from nova import exception
 from nova.network import api as network_api
-from nova.objects import instance as instance_obj
+from nova import objects
+from nova.openstack.common import jsonutils
 from nova import test
 from nova.tests import fake_block_device
 from nova.tests import fake_instance
@@ -80,8 +81,8 @@ INSTANCE = fake_instance.fake_db_instance(**
 
 
 def fake_inst_obj(context):
-    return instance_obj.Instance._from_db_object(
-        context, instance_obj.Instance(), INSTANCE,
+    return objects.Instance._from_db_object(
+        context, objects.Instance(), INSTANCE,
         expected_attrs=['metadata', 'system_metadata',
                         'info_cache'])
 
@@ -96,9 +97,10 @@ def return_non_existing_address(*args, **kwarg):
 
 
 def fake_InstanceMetadata(stubs, inst_data, address=None,
-                          sgroups=None, content=[], extra_md={},
+                          sgroups=None, content=None, extra_md=None,
                           vd_driver=None, network_info=None):
-
+    content = content or []
+    extra_md = extra_md or {}
     if sgroups is None:
         sgroups = [dict(test_security_group.fake_secgroup,
                         name='default')]
@@ -191,12 +193,12 @@ class MetadataTestCase(test.TestCase):
     def test_format_instance_mapping(self):
         # Make sure that _format_instance_mappings works.
         ctxt = None
-        instance_ref0 = instance_obj.Instance(**{'id': 0,
+        instance_ref0 = objects.Instance(**{'id': 0,
                          'uuid': 'e5fe5518-0288-4fa3-b0c4-c79764101b85',
                          'root_device_name': None,
                          'default_ephemeral_device': None,
                          'default_swap_device': None})
-        instance_ref1 = instance_obj.Instance(**{'id': 0,
+        instance_ref1 = objects.Instance(**{'id': 0,
                          'uuid': 'b65cee2f-8c69-4aeb-be2f-f79742548fc2',
                          'root_device_name': '/dev/sda1',
                          'default_ephemeral_device': None,
@@ -239,7 +241,7 @@ class MetadataTestCase(test.TestCase):
                     'swap': '/dev/sdc',
                     'ebs0': '/dev/sdh'}
 
-        capi = conductor_api.LocalAPI()
+        conductor_api.LocalAPI()
 
         self.assertEqual(base._format_instance_mapping(ctxt,
                          instance_ref0), block_device._DEFAULT_MAPPINGS)
@@ -415,7 +417,7 @@ class OpenStackMetadataTestCase(test.TestCase):
         mdjson = mdinst.lookup("/openstack/2012-08-10/meta_data.json")
         mdjson = mdinst.lookup("/openstack/latest/meta_data.json")
 
-        mddict = json.loads(mdjson)
+        mddict = jsonutils.loads(mdjson)
 
         self.assertEqual(mddict['uuid'], self.instance['uuid'])
         self.assertIn('files', mddict)
@@ -432,7 +434,7 @@ class OpenStackMetadataTestCase(test.TestCase):
         # there is as expected, and that /content lists them.
         for (path, content) in content:
             fent = [f for f in mddict['files'] if f['path'] == path]
-            self.assertTrue((len(fent) == 1))
+            self.assertEqual(1, len(fent))
             fent = fent[0]
             found = mdinst.lookup("/openstack%s" % fent['content_path'])
             self.assertEqual(found, content)
@@ -445,7 +447,7 @@ class OpenStackMetadataTestCase(test.TestCase):
         mdinst = fake_InstanceMetadata(self.stubs, inst, extra_md=extra)
 
         mdjson = mdinst.lookup("/openstack/2012-08-10/meta_data.json")
-        mddict = json.loads(mdjson)
+        mddict = jsonutils.loads(mdjson)
 
         for key, val in extra.iteritems():
             self.assertEqual(mddict[key], val)
@@ -483,24 +485,25 @@ class OpenStackMetadataTestCase(test.TestCase):
 
         # verify that 2013-04-04 has the 'random' field
         mdjson = mdinst.lookup("/openstack/2013-04-04/meta_data.json")
-        mddict = json.loads(mdjson)
+        mddict = jsonutils.loads(mdjson)
 
         self.assertIn("random_seed", mddict)
         self.assertEqual(len(base64.b64decode(mddict["random_seed"])), 512)
 
         # verify that older version do not have it
         mdjson = mdinst.lookup("/openstack/2012-08-10/meta_data.json")
-        self.assertNotIn("random_seed", json.loads(mdjson))
+        self.assertNotIn("random_seed", jsonutils.loads(mdjson))
 
     def test_no_dashes_in_metadata(self):
         # top level entries in meta_data should not contain '-' in their name
         inst = self.instance.obj_clone()
         mdinst = fake_InstanceMetadata(self.stubs, inst)
-        mdjson = json.loads(mdinst.lookup("/openstack/latest/meta_data.json"))
+        mdjson = jsonutils.loads(
+            mdinst.lookup("/openstack/latest/meta_data.json"))
 
         self.assertEqual([], [k for k in mdjson.keys() if k.find("-") != -1])
 
-    def test_vendor_data_presense(self):
+    def test_vendor_data_presence(self):
         inst = self.instance.obj_clone()
         mdinst = fake_InstanceMetadata(self.stubs, inst)
 
@@ -532,7 +535,7 @@ class OpenStackMetadataTestCase(test.TestCase):
 
         # verify that 2013-10-17 has the vendor_data.json file
         vdpath = "/openstack/2013-10-17/vendor_data.json"
-        vd = json.loads(mdinst.lookup(vdpath))
+        vd = jsonutils.loads(mdinst.lookup(vdpath))
 
         # the instance should be passed through, and our class copies the
         # uuid through to 'inst_uuid'.
@@ -580,7 +583,8 @@ class MetadataHandlerTestCase(test.TestCase):
         self.assertEqual(response.body, expected)
 
     def test_root_metadata_proxy_enabled(self):
-        CONF.set_override("service_neutron_metadata_proxy", True)
+        self.flags(service_metadata_proxy=True,
+                   group='neutron')
 
         expected = "\n".join(base.VERSIONS) + "\nlatest"
         response = fake_request(self.stubs, self.mdinst, "/")
@@ -591,11 +595,24 @@ class MetadataHandlerTestCase(test.TestCase):
 
     def test_version_root(self):
         response = fake_request(self.stubs, self.mdinst, "/2009-04-04")
+        response_ctype = response.headers['Content-Type']
+        self.assertTrue(response_ctype.startswith("text/plain"))
         self.assertEqual(response.body, 'meta-data/\nuser-data')
 
         response = fake_request(self.stubs, self.mdinst, "/9999-99-99")
         self.assertEqual(response.status_int, 404)
 
+    def test_json_data(self):
+        response = fake_request(self.stubs, self.mdinst,
+                                "/openstack/latest/meta_data.json")
+        response_ctype = response.headers['Content-Type']
+        self.assertTrue(response_ctype.startswith("application/json"))
+
+        response = fake_request(self.stubs, self.mdinst,
+                                "/openstack/latest/vendor_data.json")
+        response_ctype = response.headers['Content-Type']
+        self.assertTrue(response_ctype.startswith("application/json"))
+
     def test_user_data_non_existing_fixed_address(self):
         self.stubs.Set(network_api.API, 'get_fixed_ip_by_address',
                        return_non_existing_address)
@@ -631,6 +648,8 @@ class MetadataHandlerTestCase(test.TestCase):
                                 headers={'X-Forwarded-For': expected_addr})
 
         self.assertEqual(response.status_int, 200)
+        response_ctype = response.headers['Content-Type']
+        self.assertTrue(response_ctype.startswith("text/plain"))
         self.assertEqual(response.body,
                          base64.b64decode(self.instance['user_data']))
 
@@ -641,6 +660,21 @@ class MetadataHandlerTestCase(test.TestCase):
                                 headers=None)
         self.assertEqual(response.status_int, 500)
 
+    @mock.patch('nova.utils.constant_time_compare')
+    def test_by_instance_id_uses_constant_time_compare(self, mock_compare):
+        mock_compare.side_effect = test.TestingException
+
+        req = webob.Request.blank('/')
+        hnd = handler.MetadataRequestHandler()
+
+        req.headers['X-Instance-ID'] = 'fake-inst'
+        req.headers['X-Tenant-ID'] = 'fake-proj'
+
+        self.assertRaises(test.TestingException,
+                          hnd._handle_instance_id_request, req)
+
+        self.assertEqual(1, mock_compare.call_count)
+
     def test_user_data_with_neutron_instance_id(self):
         expected_instance_id = 'a-b-c-d'
 
@@ -655,7 +689,7 @@ class MetadataHandlerTestCase(test.TestCase):
                                 (expected_instance_id, instance_id))
 
         signed = hmac.new(
-            CONF.neutron_metadata_proxy_shared_secret,
+            CONF.neutron.metadata_proxy_shared_secret,
             expected_instance_id,
             hashlib.sha256).hexdigest()
 
@@ -670,7 +704,8 @@ class MetadataHandlerTestCase(test.TestCase):
         self.assertEqual(response.status_int, 200)
 
         # now enable the service
-        self.flags(service_neutron_metadata_proxy=True)
+        self.flags(service_metadata_proxy=True,
+                   group='neutron')
         response = fake_request(
             self.stubs, self.mdinst,
             relpath="/2009-04-04/user-data",
@@ -682,6 +717,8 @@ class MetadataHandlerTestCase(test.TestCase):
                      'X-Instance-ID-Signature': signed})
 
         self.assertEqual(response.status_int, 200)
+        response_ctype = response.headers['Content-Type']
+        self.assertTrue(response_ctype.startswith("text/plain"))
         self.assertEqual(response.body,
                          base64.b64decode(self.instance['user_data']))
 
@@ -737,7 +774,7 @@ class MetadataHandlerTestCase(test.TestCase):
 
         # unexpected Instance-ID
         signed = hmac.new(
-            CONF.neutron_metadata_proxy_shared_secret,
+            CONF.neutron.metadata_proxy_shared_secret,
            'z-z-z-z',
            hashlib.sha256).hexdigest()
 
@@ -752,6 +789,28 @@ class MetadataHandlerTestCase(test.TestCase):
                      'X-Instance-ID-Signature': signed})
         self.assertEqual(response.status_int, 500)
 
+    def test_get_metadata(self):
+        def _test_metadata_path(relpath):
+            # recursively confirm a http 200 from all meta-data elements
+            # available at relpath.
+            response = fake_request(self.stubs, self.mdinst,
+                                    relpath=relpath)
+            for item in response.body.split('\n'):
+                if 'public-keys' in relpath:
+                    # meta-data/public-keys/0=keyname refers to
+                    # meta-data/public-keys/0
+                    item = item.split('=')[0]
+                if item.endswith('/'):
+                    path = relpath + '/' + item
+                    _test_metadata_path(path)
+                    continue
+
+                path = relpath + '/' + item
+                response = fake_request(self.stubs, self.mdinst, relpath=path)
+                self.assertEqual(response.status_int, 200, message=path)
+
+        _test_metadata_path('/2009-04-04/meta-data')
+
 
 class MetadataPasswordTestCase(test.TestCase):
     def setUp(self):
@@ -777,21 +836,18 @@ class MetadataPasswordTestCase(test.TestCase):
         self.assertRaises(webob.exc.HTTPBadRequest,
                           password.handle_password, request, self.mdinst)
 
-    def _try_set_password(self, val='bar'):
+    @mock.patch('nova.objects.Instance.get_by_uuid')
+    def _try_set_password(self, get_by_uuid, val='bar'):
         request = webob.Request.blank('')
         request.method = 'POST'
         request.body = val
-        self.stubs.Set(db, 'instance_get_by_uuid',
-                       lambda *a, **kw: {'system_metadata': []})
+        get_by_uuid.return_value = self.instance
 
-        def fake_instance_update(context, uuid, updates):
-            self.assertIn('system_metadata', updates)
-            self.assertIn('password_0', updates['system_metadata'])
-            return self.instance, self.instance
+        with mock.patch.object(self.instance, 'save') as save:
+            password.handle_password(request, self.mdinst)
+            save.assert_called_once_with()
 
-        self.stubs.Set(db, 'instance_update_and_get_original',
-                       fake_instance_update)
-        password.handle_password(request, self.mdinst)
+        self.assertIn('password_0', self.instance.system_metadata)
 
     def test_set_password(self):
         self.mdinst.password = ''
@@ -806,4 +862,4 @@ class MetadataPasswordTestCase(test.TestCase):
         self.mdinst.password = ''
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self._try_set_password,
-                          'a' * (password.MAX_SIZE + 1))
+                          val=('a' * (password.MAX_SIZE + 1)))
