@@ -21,13 +21,12 @@
 import os
 
 import mox
-
 from oslo.config import cfg
+from oslo.db import exception as db_exc
 from testtools import matchers
 
 from nova import exception
-from nova.objects import flavor as flavor_obj
-from nova.openstack.common.db import exception as db_exc
+from nova import objects
 from nova.tests.image import fake as fake_image
 from nova.tests import utils
 from nova.tests.virt.baremetal.db import base as bm_db_base
@@ -285,15 +284,12 @@ class PXEClassMethodsTestCase(BareMetalPXETestCase):
         flavor = utils.get_test_flavor()
         # Raises an exception when options are neither specified
         # on the instance nor in configuration file
-        CONF.baremetal.deploy_kernel = None
-        CONF.baremetal.deploy_ramdisk = None
         self.assertRaises(exception.NovaException,
                 pxe.get_tftp_image_info,
                 self.instance, flavor)
 
         # Test that other non-true values also raise an exception
-        CONF.baremetal.deploy_kernel = ""
-        CONF.baremetal.deploy_ramdisk = ""
+        self.flags(deploy_kernel='', deploy_ramdisk='', group='baremetal')
         self.assertRaises(exception.NovaException,
                 pxe.get_tftp_image_info,
                 self.instance, flavor)
@@ -313,8 +309,8 @@ class PXEClassMethodsTestCase(BareMetalPXETestCase):
 
         # Here, we confirm both that all four values were set
         # and that the proper paths are getting set for all of them
-        CONF.baremetal.deploy_kernel = 'cccc'
-        CONF.baremetal.deploy_ramdisk = 'dddd'
+        self.flags(deploy_kernel='cccc', deploy_ramdisk='dddd',
+                   group='baremetal')
         base = os.path.join(CONF.baremetal.tftp_root, self.instance['uuid'])
         res = pxe.get_tftp_image_info(self.instance, flavor)
         expected = {
@@ -414,6 +410,9 @@ class PXEPrivateMethodsTestCase(BareMetalPXETestCase):
         net = pxe.build_network_config(net_info)
         admin_password = 'fake password'
 
+        self.mox.StubOutWithMock(os.path, 'exists')
+        os.path.exists(mox.IgnoreArg()).AndReturn(True)
+
         self.mox.StubOutWithMock(disk_api, 'inject_data')
         disk_api.inject_data(
                 admin_password=admin_password,
@@ -438,15 +437,15 @@ class PXEPublicMethodsTestCase(BareMetalPXETestCase):
 
     def test_cache_images(self):
         self._create_node()
-        self.mox.StubOutWithMock(flavor_obj.Flavor, 'get_by_id')
+        self.mox.StubOutWithMock(objects.Flavor, 'get_by_id')
         self.mox.StubOutWithMock(pxe, "get_tftp_image_info")
         self.mox.StubOutWithMock(self.driver, "_cache_tftp_images")
         self.mox.StubOutWithMock(self.driver, "_cache_image")
         self.mox.StubOutWithMock(self.driver, "_inject_into_image")
 
-        flavor_obj.Flavor.get_by_id(self.context,
-                                    self.instance['instance_type_id']
-                                    ).AndReturn({})
+        objects.Flavor.get_by_id(self.context,
+                                 self.instance['instance_type_id']
+                                 ).AndReturn({})
         pxe.get_tftp_image_info(self.instance, {}).AndReturn([])
         self.driver._cache_tftp_images(self.context, self.instance, [])
         self.driver._cache_image(self.context, self.instance, [])
@@ -500,9 +499,9 @@ class PXEPublicMethodsTestCase(BareMetalPXETestCase):
         iqn = "iqn-%s" % self.instance['uuid']
         pxe_config = 'this is a fake pxe config'
         pxe_path = pxe.get_pxe_config_file_path(self.instance)
-        image_path = pxe.get_image_file_path(self.instance)
+        pxe.get_image_file_path(self.instance)
 
-        self.mox.StubOutWithMock(flavor_obj.Flavor, 'get_by_id')
+        self.mox.StubOutWithMock(objects.Flavor, 'get_by_id')
         self.mox.StubOutWithMock(pxe, 'get_tftp_image_info')
         self.mox.StubOutWithMock(pxe, 'get_partition_sizes')
         self.mox.StubOutWithMock(bm_utils, 'random_alnum')
@@ -510,9 +509,9 @@ class PXEPublicMethodsTestCase(BareMetalPXETestCase):
         self.mox.StubOutWithMock(bm_utils, 'write_to_file')
         self.mox.StubOutWithMock(bm_utils, 'create_link_without_raise')
 
-        flavor_obj.Flavor.get_by_id(self.context,
-                                    self.instance['instance_type_id']
-                                    ).AndReturn({})
+        objects.Flavor.get_by_id(self.context,
+                                 self.instance['instance_type_id']
+                                 ).AndReturn({})
         pxe.get_tftp_image_info(self.instance, {}).AndReturn(image_info)
         pxe.get_partition_sizes(self.instance).AndReturn((0, 0, 0))
         bm_utils.random_alnum(32).AndReturn('alnum')
@@ -534,7 +533,7 @@ class PXEPublicMethodsTestCase(BareMetalPXETestCase):
 
     def test_activate_and_deactivate_bootloader(self):
         self._create_node()
-        flavor = flavor_obj.Flavor(
+        flavor = objects.Flavor(
             context=self.context,
             extra_specs={
                 'baremetal:deploy_kernel_id': 'eeee',
@@ -542,13 +541,13 @@ class PXEPublicMethodsTestCase(BareMetalPXETestCase):
                 })
         self.instance['uuid'] = 'fake-uuid'
 
-        self.mox.StubOutWithMock(flavor_obj.Flavor, 'get_by_id')
+        self.mox.StubOutWithMock(objects.Flavor, 'get_by_id')
         self.mox.StubOutWithMock(bm_utils, 'write_to_file')
         self.mox.StubOutWithMock(bm_utils, 'create_link_without_raise')
         self.mox.StubOutWithMock(bm_utils, 'unlink_without_raise')
         self.mox.StubOutWithMock(bm_utils, 'rmtree_without_raise')
 
-        flavor_obj.Flavor.get_by_id(
+        objects.Flavor.get_by_id(
             self.context, self.instance['instance_type_id']).AndReturn(
                 flavor)
 
