@@ -587,11 +587,14 @@ class HyperVAPITestCase(HyperVAPIBaseTestCase):
                                    constants.HYPERV_VM_STATE_ENABLED)
 
     def test_suspend(self):
-        self._test_vm_state_change(self._conn.suspend, None,
+        self._test_vm_state_change(lambda i: self._conn.suspend(self._context,
+                                                                i),
+                                   None,
                                    constants.HYPERV_VM_STATE_SUSPENDED)
 
     def test_suspend_already_suspended(self):
-        self._test_vm_state_change(self._conn.suspend,
+        self._test_vm_state_change(lambda i: self._conn.suspend(self._context,
+                                                                i),
                                    constants.HYPERV_VM_STATE_SUSPENDED,
                                    constants.HYPERV_VM_STATE_SUSPENDED)
 
@@ -985,6 +988,7 @@ class HyperVAPITestCase(HyperVAPIBaseTestCase):
         vmutils.VMUtils.create_vm(mox.Func(self._check_vm_name), mox.IsA(int),
                                   mox.IsA(int), mox.IsA(bool),
                                   CONF.hyperv.dynamic_memory_ratio,
+                                  mox.IsA(str),
                                   mox.IsA(list))
 
         if not boot_from_volume:
