@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 #  Copyright 2013 Cloudbase Solutions Srl
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,8 +14,8 @@
 
 import mock
 
+from nova.openstack.common import units
 from nova import test
-
 from nova.virt.hyperv import constants
 from nova.virt.hyperv import vhdutilsv2
 
@@ -28,7 +26,7 @@ class VHDUtilsV2TestCase(test.NoDBTestCase):
     _FAKE_VHD_PATH = "C:\\fake_path.vhdx"
     _FAKE_PARENT_VHD_PATH = "C:\\fake_parent_path.vhdx"
     _FAKE_FORMAT = 3
-    _FAKE_MAK_INTERNAL_SIZE = 1 << 30
+    _FAKE_MAK_INTERNAL_SIZE = units.Gi
     _FAKE_TYPE = 3
     _FAKE_JOB_PATH = 'fake_job_path'
     _FAKE_RET_VAL = 0
@@ -37,6 +35,11 @@ class VHDUtilsV2TestCase(test.NoDBTestCase):
     _FAKE_LOG_SIZE = 1048576
     _FAKE_LOGICAL_SECTOR_SIZE = 4096
     _FAKE_METADATA_SIZE = 1048576
+    _FAKE_VHD_INFO = {'ParentPath': _FAKE_PARENT_VHD_PATH,
+                      'Format': _FAKE_FORMAT,
+                      'BlockSize': _FAKE_BLOCK_SIZE,
+                      'LogicalSectorSize': _FAKE_LOGICAL_SECTOR_SIZE,
+                      'Type': _FAKE_TYPE}
 
     def setUp(self):
         self._vhdutils = vhdutilsv2.VHDUtilsV2()
@@ -100,12 +103,12 @@ class VHDUtilsV2TestCase(test.NoDBTestCase):
 
         vhd_info = self._vhdutils.get_vhd_info(self._FAKE_VHD_PATH)
 
-        self.assertEquals(self._FAKE_VHD_PATH, vhd_info['Path'])
-        self.assertEquals(self._FAKE_PARENT_VHD_PATH, vhd_info['ParentPath'])
-        self.assertEquals(self._FAKE_FORMAT, vhd_info['Format'])
-        self.assertEquals(self._FAKE_MAK_INTERNAL_SIZE,
-                          vhd_info['MaxInternalSize'])
-        self.assertEquals(self._FAKE_TYPE, vhd_info['Type'])
+        self.assertEqual(self._FAKE_VHD_PATH, vhd_info['Path'])
+        self.assertEqual(self._FAKE_PARENT_VHD_PATH, vhd_info['ParentPath'])
+        self.assertEqual(self._FAKE_FORMAT, vhd_info['Format'])
+        self.assertEqual(self._FAKE_MAK_INTERNAL_SIZE,
+                         vhd_info['MaxInternalSize'])
+        self.assertEqual(self._FAKE_TYPE, vhd_info['Type'])
 
     def test_create_dynamic_vhd(self):
         self._vhdutils.get_vhd_info = mock.MagicMock(
@@ -168,13 +171,16 @@ class VHDUtilsV2TestCase(test.NoDBTestCase):
         self.mock_get.assert_called_once_with(self._FAKE_VHD_PATH,
                                               self._FAKE_MAK_INTERNAL_SIZE)
 
-    def test_get_vhdx_internal_size(self):
-        self._vhdutils.get_vhd_info = mock.MagicMock(
-            return_value={'ParentPath': self._FAKE_PARENT_VHD_PATH,
-                          'Format': self._FAKE_FORMAT,
-                          'BlockSize': self._FAKE_BLOCK_SIZE,
-                          'LogicalSectorSize': self._FAKE_LOGICAL_SECTOR_SIZE,
-                          'Type': self._FAKE_TYPE})
+    def _test_get_vhdx_internal_size(self, vhd_type):
+        self._vhdutils.get_vhd_info = mock.MagicMock()
+        self._vhdutils.get_vhd_parent_path = mock.Mock(
+            return_value=self._FAKE_PARENT_VHD_PATH)
+
+        if vhd_type == 4:
+            self._vhdutils.get_vhd_info.side_effect = [
+                {'Type': vhd_type}, self._FAKE_VHD_INFO]
+        else:
+            self._vhdutils.get_vhd_info.return_value = self._FAKE_VHD_INFO
         self._vhdutils._get_vhdx_log_size = mock.MagicMock(
             return_value=self._FAKE_LOG_SIZE)
         self._vhdutils._get_vhdx_metadata_size_and_offset = mock.MagicMock(
@@ -191,6 +197,12 @@ class VHDUtilsV2TestCase(test.NoDBTestCase):
         self.assertEqual(self._FAKE_MAK_INTERNAL_SIZE - self._FAKE_BLOCK_SIZE,
                          internal_size)
 
+    def test_get_vhdx_internal_size_dynamic(self):
+        self._test_get_vhdx_internal_size(3)
+
+    def test_get_vhdx_internal_size_differencing(self):
+        self._test_get_vhdx_internal_size(4)
+
     def test_get_vhdx_current_header(self):
         VHDX_HEADER_OFFSETS = [64 * 1024, 128 * 1024]
         fake_sequence_numbers = ['\x01\x00\x00\x00\x00\x00\x00\x00',
