@@ -38,6 +38,9 @@ class VMUtilsTestCase(test.NoDBTestCase):
     _FAKE_VHD_PATH = "fake_vhd_path"
     _FAKE_DVD_PATH = "fake_dvd_path"
     _FAKE_VOLUME_DRIVE_PATH = "fake_volume_drive_path"
+    _FAKE_VM_UUID = "04e79212-39bc-4065-933c-50f6d48a57f6"
+    _FAKE_INSTANCE = {"name": _FAKE_VM_NAME,
+                      "uuid": _FAKE_VM_UUID}
     _FAKE_SNAPSHOT_PATH = "fake_snapshot_path"
     _FAKE_RES_DATA = "fake_res_data"
     _FAKE_HOST_RESOURCE = "fake_host_resource"
@@ -147,6 +150,27 @@ class VMUtilsTestCase(test.NoDBTestCase):
         else:
             self.assertFalse(mock_s.DynamicMemoryEnabled)
 
+    def test_soft_shutdown_vm(self):
+        mock_vm = self._lookup_vm()
+        mock_shutdown = mock.MagicMock()
+        mock_shutdown.InitiateShutdown.return_value = (self._FAKE_RET_VAL, )
+        mock_vm.associators.return_value = [mock_shutdown]
+
+        with mock.patch.object(self._vmutils, 'check_ret_val') as mock_check:
+            self._vmutils.soft_shutdown_vm(self._FAKE_VM_NAME)
+
+            mock_shutdown.InitiateShutdown.assert_called_once_with(
+                Force=False, Reason=mock.ANY)
+            mock_check.assert_called_once_with(self._FAKE_RET_VAL, None)
+
+    def test_soft_shutdown_vm_no_component(self):
+        mock_vm = self._lookup_vm()
+        mock_vm.associators.return_value = []
+
+        with mock.patch.object(self._vmutils, 'check_ret_val') as mock_check:
+            self._vmutils.soft_shutdown_vm(self._FAKE_VM_NAME)
+            self.assertFalse(mock_check.called)
+
     @mock.patch('nova.virt.hyperv.vmutils.VMUtils._get_vm_disks')
     def test_get_vm_storage_paths(self, mock_get_vm_disks):
         self._lookup_vm()
@@ -191,89 +215,6 @@ class VMUtilsTestCase(test.NoDBTestCase):
 
         return [mock_rasd1, mock_rasd2]
 
-    def test_list_instance_notes(self):
-        vs = mock.MagicMock()
-        attrs = {'ElementName': 'fake_name',
-                 'Notes': '4f54fb69-d3a2-45b7-bb9b-b6e6b3d893b3'}
-        vs.configure_mock(**attrs)
-        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs]
-        response = self._vmutils.list_instance_notes()
-
-        self.assertEqual([(attrs['ElementName'], [attrs['Notes']])], response)
-        self._vmutils._conn.Msvm_VirtualSystemSettingData.assert_called_with(
-            ['ElementName', 'Notes'],
-            SettingType=self._vmutils._VIRTUAL_SYSTEM_CURRENT_SETTINGS)
-
-    @mock.patch('nova.virt.hyperv.vmutils.VMUtils.check_ret_val')
-    def test_modify_virtual_system(self, mock_check_ret_val):
-        mock_vs_man_svc = mock.MagicMock()
-        mock_vmsetting = mock.MagicMock()
-        fake_path = 'fake path'
-        fake_job_path = 'fake job path'
-        fake_ret_val = 'fake return value'
-
-        mock_vs_man_svc.ModifyVirtualSystem.return_value = (0, fake_job_path,
-                                                            fake_ret_val)
-
-        self._vmutils._modify_virtual_system(vs_man_svc=mock_vs_man_svc,
-                                             vm_path=fake_path,
-                                             vmsetting=mock_vmsetting)
-
-        mock_vs_man_svc.ModifyVirtualSystem.assert_called_once_with(
-            ComputerSystem=fake_path,
-            SystemSettingData=mock_vmsetting.GetText_(1))
-        mock_check_ret_val.assert_called_once_with(fake_ret_val, fake_job_path)
-
-    @mock.patch('nova.virt.hyperv.vmutils.VMUtils.check_ret_val')
-    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._get_wmi_obj')
-    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._modify_virtual_system')
-    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._get_vm_setting_data')
-    def test_create_vm_obj(self, mock_get_vm_setting_data,
-                           mock_modify_virtual_system,
-                           mock_get_wmi_obj, mock_check_ret_val):
-        mock_vs_man_svc = mock.MagicMock()
-        mock_vs_gs_data = mock.MagicMock()
-        fake_vm_path = 'fake vm path'
-        fake_job_path = 'fake job path'
-        fake_ret_val = 'fake return value'
-        _conn = self._vmutils._conn.Msvm_VirtualSystemGlobalSettingData
-
-        _conn.new.return_value = mock_vs_gs_data
-        mock_vs_man_svc.DefineVirtualSystem.return_value = (fake_vm_path,
-                                                            fake_job_path,
-                                                            fake_ret_val)
-
-        response = self._vmutils._create_vm_obj(vs_man_svc=mock_vs_man_svc,
-                                                vm_name='fake vm',
-                                                notes='fake notes')
-
-        _conn.new.assert_called_once_with()
-        self.assertEqual(mock_vs_gs_data.ElementName, 'fake vm')
-        mock_vs_man_svc.DefineVirtualSystem.assert_called_once_with(
-            [], None, mock_vs_gs_data.GetText_(1))
-        mock_check_ret_val.assert_called_once_with(fake_ret_val, fake_job_path)
-
-        mock_get_wmi_obj.assert_called_with(fake_vm_path)
-        mock_get_vm_setting_data.assert_called_once_with(mock_get_wmi_obj())
-        mock_modify_virtual_system.assert_called_once_with(
-            mock_vs_man_svc, fake_vm_path, mock_get_vm_setting_data())
-
-        self.assertEqual(mock_get_vm_setting_data().Notes,
-                         '\n'.join('fake notes'))
-        self.assertEqual(response, mock_get_wmi_obj())
-
-    def test_list_instances(self):
-        vs = mock.MagicMock()
-        attrs = {'ElementName': 'fake_name'}
-        vs.configure_mock(**attrs)
-        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs]
-        response = self._vmutils.list_instances()
-
-        self.assertEqual([(attrs['ElementName'])], response)
-        self._vmutils._conn.Msvm_VirtualSystemSettingData.assert_called_with(
-            ['ElementName'],
-            SettingType=self._vmutils._VIRTUAL_SYSTEM_CURRENT_SETTINGS)
-
     @mock.patch.object(vmutils.VMUtils, '_set_vm_vcpus')
     @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
     @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
@@ -293,7 +234,8 @@ class VMUtilsTestCase(test.NoDBTestCase):
 
         self._vmutils.create_vm(self._FAKE_VM_NAME, self._FAKE_MEMORY_MB,
                                 self._FAKE_VCPUS_NUM, False,
-                                self._FAKE_DYNAMIC_MEMORY_RATIO)
+                                self._FAKE_DYNAMIC_MEMORY_RATIO,
+                                mock.sentinel.instance_path)
 
         self.assertTrue(getattr(mock_svc, self._DEFINE_SYSTEM).called)
         mock_set_mem.assert_called_with(mock_vm, mock_s, self._FAKE_MEMORY_MB,
@@ -497,6 +439,29 @@ class VMUtilsTestCase(test.NoDBTestCase):
                                             self._FAKE_VM_PATH)
         self._assert_remove_resources(mock_svc)
 
+    def test_set_disk_host_resource(self):
+        self._lookup_vm()
+        mock_rasds = self._create_mock_disks()
+
+        self._vmutils._get_vm_disks = mock.MagicMock(
+            return_value=([mock_rasds[0]], [mock_rasds[1]]))
+        self._vmutils._modify_virt_resource = mock.MagicMock()
+        self._vmutils._get_disk_resource_address = mock.MagicMock(
+            return_value=self._FAKE_ADDRESS)
+
+        self._vmutils.set_disk_host_resource(
+            self._FAKE_VM_NAME,
+            self._FAKE_CTRL_PATH,
+            self._FAKE_ADDRESS,
+            mock.sentinel.fake_new_mounted_disk_path)
+        self._vmutils._get_disk_resource_address.assert_called_with(
+            mock_rasds[0])
+        self._vmutils._modify_virt_resource.assert_called_with(
+            mock_rasds[0], self._FAKE_VM_PATH)
+        self.assertEqual(
+            mock.sentinel.fake_new_mounted_disk_path,
+            mock_rasds[0].HostResource[0])
+
     @mock.patch.object(vmutils, 'wmi', create=True)
     @mock.patch.object(vmutils.VMUtils, 'check_ret_val')
     def test_take_vm_snapshot(self, mock_check_ret_val, mock_wmi):
@@ -571,25 +536,140 @@ class VMUtilsTestCase(test.NoDBTestCase):
         getattr(mock_svc, self._REMOVE_RESOURCE).assert_called_with(
             [self._FAKE_RES_PATH], self._FAKE_VM_PATH)
 
-    def test_set_disk_host_resource(self):
-        self._lookup_vm()
-        mock_rasds = self._create_mock_disks()
+    def test_get_active_instances(self):
+        fake_vm = mock.MagicMock()
 
-        self._vmutils._get_vm_disks = mock.MagicMock(
-            return_value=([mock_rasds[0]], [mock_rasds[1]]))
+        type(fake_vm).ElementName = mock.PropertyMock(
+            side_effect=['active_vm', 'inactive_vm'])
+        type(fake_vm).EnabledState = mock.PropertyMock(
+            side_effect=[constants.HYPERV_VM_STATE_ENABLED,
+                         constants.HYPERV_VM_STATE_DISABLED])
+        self._vmutils.list_instances = mock.MagicMock(
+            return_value=[mock.sentinel.fake_vm_name] * 2)
+        self._vmutils._lookup_vm = mock.MagicMock(side_effect=[fake_vm] * 2)
+        active_instances = self._vmutils.get_active_instances()
+
+        self.assertEqual(['active_vm'], active_instances)
+
+    def _test_get_vm_serial_port_connection(self, new_connection=None):
+        old_serial_connection = 'old_serial_connection'
+
+        mock_vm = self._lookup_vm()
+        mock_vmsettings = [mock.MagicMock()]
+        mock_vm.associators.return_value = mock_vmsettings
+
+        fake_serial_port = mock.MagicMock()
+
+        fake_serial_port.ResourceSubType = (
+            self._vmutils._SERIAL_PORT_RES_SUB_TYPE)
+        fake_serial_port.Connection = [old_serial_connection]
+        mock_rasds = [fake_serial_port]
+        mock_vmsettings[0].associators.return_value = mock_rasds
         self._vmutils._modify_virt_resource = mock.MagicMock()
-        self._vmutils._get_disk_resource_address = mock.MagicMock(
-            return_value=self._FAKE_ADDRESS)
+        fake_modify = self._vmutils._modify_virt_resource
 
-        self._vmutils.set_disk_host_resource(
-            self._FAKE_VM_NAME,
-            self._FAKE_CTRL_PATH,
-            self._FAKE_ADDRESS,
-            mock.sentinel.fake_new_mounted_disk_path)
-        self._vmutils._get_disk_resource_address.assert_called_with(
-            mock_rasds[0])
-        self._vmutils._modify_virt_resource.assert_called_with(
-            mock_rasds[0], self._FAKE_VM_PATH)
-        self.assertEqual(
-            mock.sentinel.fake_new_mounted_disk_path,
-            mock_rasds[0].HostResource[0])
+        ret_val = self._vmutils.get_vm_serial_port_connection(
+            self._FAKE_VM_NAME, update_connection=new_connection)
+
+        if new_connection:
+            self.assertEqual(new_connection, ret_val)
+            fake_modify.assert_called_once_with(fake_serial_port,
+                                                mock_vm.path_())
+        else:
+            self.assertEqual(old_serial_connection, ret_val)
+
+    def test_set_vm_serial_port_connection(self):
+        self._test_get_vm_serial_port_connection('new_serial_connection')
+
+    def test_get_vm_serial_port_connection(self):
+        self._test_get_vm_serial_port_connection()
+
+    def test_list_instance_notes(self):
+        vs = mock.MagicMock()
+        attrs = {'ElementName': 'fake_name',
+                 'Notes': '4f54fb69-d3a2-45b7-bb9b-b6e6b3d893b3'}
+        vs.configure_mock(**attrs)
+        vs2 = mock.MagicMock(ElementName='fake_name2', Notes=None)
+        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs,
+                                                                          vs2]
+        response = self._vmutils.list_instance_notes()
+
+        self.assertEqual([(attrs['ElementName'], [attrs['Notes']])], response)
+        self._vmutils._conn.Msvm_VirtualSystemSettingData.assert_called_with(
+            ['ElementName', 'Notes'],
+            SettingType=self._vmutils._VIRTUAL_SYSTEM_CURRENT_SETTINGS)
+
+    @mock.patch('nova.virt.hyperv.vmutils.VMUtils.check_ret_val')
+    def test_modify_virtual_system(self, mock_check_ret_val):
+        mock_vs_man_svc = mock.MagicMock()
+        mock_vmsetting = mock.MagicMock()
+        fake_path = 'fake path'
+        fake_job_path = 'fake job path'
+        fake_ret_val = 'fake return value'
+
+        mock_vs_man_svc.ModifyVirtualSystem.return_value = (0, fake_job_path,
+                                                            fake_ret_val)
+
+        self._vmutils._modify_virtual_system(vs_man_svc=mock_vs_man_svc,
+                                             vm_path=fake_path,
+                                             vmsetting=mock_vmsetting)
+
+        mock_vs_man_svc.ModifyVirtualSystem.assert_called_once_with(
+            ComputerSystem=fake_path,
+            SystemSettingData=mock_vmsetting.GetText_(1))
+        mock_check_ret_val.assert_called_once_with(fake_ret_val, fake_job_path)
+
+    @mock.patch('nova.virt.hyperv.vmutils.VMUtils.check_ret_val')
+    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._get_wmi_obj')
+    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._modify_virtual_system')
+    @mock.patch('nova.virt.hyperv.vmutils.VMUtils._get_vm_setting_data')
+    def test_create_vm_obj(self, mock_get_vm_setting_data,
+                           mock_modify_virtual_system,
+                           mock_get_wmi_obj, mock_check_ret_val):
+        mock_vs_man_svc = mock.MagicMock()
+        mock_vs_gs_data = mock.MagicMock()
+        fake_vm_path = 'fake vm path'
+        fake_job_path = 'fake job path'
+        fake_ret_val = 'fake return value'
+        _conn = self._vmutils._conn.Msvm_VirtualSystemGlobalSettingData
+
+        _conn.new.return_value = mock_vs_gs_data
+        mock_vs_man_svc.DefineVirtualSystem.return_value = (fake_vm_path,
+                                                            fake_job_path,
+                                                            fake_ret_val)
+
+        response = self._vmutils._create_vm_obj(
+            vs_man_svc=mock_vs_man_svc,
+            vm_name='fake vm',
+            notes='fake notes', dynamic_memory_ratio=1.0,
+            instance_path=mock.sentinel.instance_path)
+
+        _conn.new.assert_called_once_with()
+        self.assertEqual(mock_vs_gs_data.ElementName, 'fake vm')
+        mock_vs_man_svc.DefineVirtualSystem.assert_called_once_with(
+            [], None, mock_vs_gs_data.GetText_(1))
+        mock_check_ret_val.assert_called_once_with(fake_ret_val, fake_job_path)
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_gs_data.ExternalDataRoot)
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_gs_data.SnapshotDataRoot)
+        mock_get_wmi_obj.assert_called_with(fake_vm_path)
+        mock_get_vm_setting_data.assert_called_once_with(mock_get_wmi_obj())
+        mock_modify_virtual_system.assert_called_once_with(
+            mock_vs_man_svc, fake_vm_path, mock_get_vm_setting_data())
+
+        self.assertEqual(mock_get_vm_setting_data().Notes,
+                         '\n'.join('fake notes'))
+        self.assertEqual(response, mock_get_wmi_obj())
+
+    def test_list_instances(self):
+        vs = mock.MagicMock()
+        attrs = {'ElementName': 'fake_name'}
+        vs.configure_mock(**attrs)
+        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs]
+        response = self._vmutils.list_instances()
+
+        self.assertEqual([(attrs['ElementName'])], response)
+        self._vmutils._conn.Msvm_VirtualSystemSettingData.assert_called_with(
+            ['ElementName'],
+            SettingType=self._vmutils._VIRTUAL_SYSTEM_CURRENT_SETTINGS)
