@@ -124,7 +124,9 @@ class VMUtilsV2TestCase(test_vmutils.VMUtilsTestCase):
         attrs = {'ElementName': 'fake_name',
                  'Notes': ['4f54fb69-d3a2-45b7-bb9b-b6e6b3d893b3']}
         vs.configure_mock(**attrs)
-        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs]
+        vs2 = mock.MagicMock(ElementName='fake_name2', Notes=None)
+        self._vmutils._conn.Msvm_VirtualSystemSettingData.return_value = [vs,
+                                                                          vs2]
         response = self._vmutils.list_instance_notes()
 
         self.assertEqual([(attrs['ElementName'], attrs['Notes'])], response)
@@ -135,12 +137,13 @@ class VMUtilsV2TestCase(test_vmutils.VMUtilsTestCase):
     @mock.patch('nova.virt.hyperv.vmutilsv2.VMUtilsV2.check_ret_val')
     @mock.patch('nova.virt.hyperv.vmutilsv2.VMUtilsV2._get_wmi_obj')
     def _test_create_vm_obj(self, mock_get_wmi_obj, mock_check_ret_val,
-                            vm_path):
+                            vm_path, dynamic_memory_ratio=1.0):
         mock_vs_man_svc = mock.MagicMock()
         mock_vs_data = mock.MagicMock()
         mock_job = mock.MagicMock()
         fake_job_path = 'fake job path'
         fake_ret_val = 'fake return value'
+        fake_vm_name = 'fake_vm_name'
         _conn = self._vmutils._conn.Msvm_VirtualSystemSettingData
 
         mock_check_ret_val.return_value = mock_job
@@ -150,24 +153,39 @@ class VMUtilsV2TestCase(test_vmutils.VMUtilsTestCase):
                                                      fake_ret_val)
         mock_job.associators.return_value = ['fake vm path']
 
-        response = self._vmutils._create_vm_obj(vs_man_svc=mock_vs_man_svc,
-                                                vm_name='fake vm',
-                                                notes='fake notes')
+        response = self._vmutils._create_vm_obj(
+            vs_man_svc=mock_vs_man_svc,
+            vm_name=fake_vm_name,
+            notes='fake notes',
+            dynamic_memory_ratio=dynamic_memory_ratio,
+            instance_path=mock.sentinel.instance_path)
 
         if not vm_path:
             mock_job.associators.assert_called_once_with(
                 self._vmutils._AFFECTED_JOB_ELEMENT_CLASS)
 
         _conn.new.assert_called_once_with()
-        self.assertEqual(mock_vs_data.ElementName, 'fake vm')
+        self.assertEqual(mock_vs_data.ElementName, fake_vm_name)
         mock_vs_man_svc.DefineSystem.assert_called_once_with(
             ResourceSettings=[], ReferenceConfiguration=None,
             SystemSettings=mock_vs_data.GetText_(1))
         mock_check_ret_val.assert_called_once_with(fake_ret_val, fake_job_path)
 
+        if dynamic_memory_ratio > 1:
+            self.assertFalse(mock_vs_data.VirtualNumaEnabled)
+
         mock_get_wmi_obj.assert_called_with('fake vm path')
 
         self.assertEqual(mock_vs_data.Notes, 'fake notes')
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_data.ConfigurationDataRoot)
+        self.assertEqual(mock.sentinel.instance_path, mock_vs_data.LogDataRoot)
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_data.SnapshotDataRoot)
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_data.SuspendDataRoot)
+        self.assertEqual(mock.sentinel.instance_path,
+                         mock_vs_data.SwapFileDataRoot)
         self.assertEqual(response, mock_get_wmi_obj())
 
     def test_create_vm_obj(self):
@@ -176,6 +194,9 @@ class VMUtilsV2TestCase(test_vmutils.VMUtilsTestCase):
     def test_create_vm_obj_no_vm_path(self):
         self._test_create_vm_obj(vm_path=None)
 
+    def test_create_vm_obj_dynamic_memory(self):
+        self._test_create_vm_obj(vm_path=None, dynamic_memory_ratio=1.1)
+
     def test_list_instances(self):
         vs = mock.MagicMock()
         attrs = {'ElementName': 'fake_name'}
