@@ -15,6 +15,7 @@
 import contextlib
 
 import mock
+import mox
 
 from nova import block_device
 from nova import context
@@ -381,7 +382,11 @@ class TestDriverBlockDevice(test.NoDBTestCase):
             self.volume_api.attach(elevated_context, fake_volume['id'],
                                    'fake_uuid', bdm_dict['device_name'],
                                    mode=access_mode).AndReturn(None)
-        driver_bdm._bdm_obj.save(self.context).AndReturn(None)
+        # NOTE(mriedem): save() is called with the elevated context within
+        # attach() and with the original context from the update_db decorator
+        # so we ignore which arg it is in test.
+        driver_bdm._bdm_obj.save(
+            mox.IgnoreArg()).MultipleTimes().AndReturn(None)
         return instance, expected_conn_info
 
     def test_volume_attach(self):
