@@ -13,26 +13,31 @@
 #    under the License.
 
 import base64
-import fixtures
 import sys
 import traceback
 
+import fixtures
 import mock
 import netaddr
 import six
 
 from nova.compute import manager
+from nova.console import type as ctype
 from nova import exception
+from nova import objects
 from nova.openstack.common import importutils
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
+from nova.openstack.common import timeutils
 from nova import test
+from nova.tests import fake_block_device
 from nova.tests.image import fake as fake_image
 from nova.tests import utils as test_utils
 from nova.tests.virt.libvirt import fake_libvirt_utils
-from nova.tests.virt.libvirt import test_libvirt
+from nova.virt import block_device as driver_block_device
 from nova.virt import event as virtevent
 from nova.virt import fake
+from nova.virt import libvirt
 from nova.virt.libvirt import imagebackend
 
 LOG = logging.getLogger(__name__)
@@ -42,17 +47,17 @@ def catch_notimplementederror(f):
     """Decorator to simplify catching drivers raising NotImplementedError
 
     If a particular call makes a driver raise NotImplementedError, we
-    log it so that we can extract this information afterwards to
-    automatically generate a hypervisor/feature support matrix.
+    log it so that we can extract this information afterwards as needed.
     """
     def wrapped_func(self, *args, **kwargs):
         try:
             return f(self, *args, **kwargs)
         except NotImplementedError:
             frame = traceback.extract_tb(sys.exc_info()[2])[-1]
-            LOG.error('%(driver)s does not implement %(method)s' % {
-                                               'driver': type(self.connection),
-                                               'method': frame[2]})
+            LOG.error("%(driver)s does not implement %(method)s "
+                      "required for test %(test)s" %
+                      {'driver': type(self.connection),
+                       'method': frame[2], 'test': f.__name__})
 
     wrapped_func.__name__ = f.__name__
     wrapped_func.__doc__ = f.__doc__
@@ -119,7 +124,7 @@ class _FakeDriverBackendTestCase(object):
             pass
 
         self.stubs.Set(nova.virt.libvirt.driver.LibvirtDriver,
-                       'get_instance_disk_info',
+                       '_get_instance_disk_info',
                        fake_get_instance_disk_info)
 
         self.stubs.Set(nova.virt.libvirt.driver.disk,
@@ -260,6 +265,12 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.connection.snapshot(self.ctxt, instance_ref, img_ref['id'],
                                  lambda *args, **kwargs: None)
 
+    @catch_notimplementederror
+    def test_post_interrupted_snapshot_cleanup(self):
+        instance_ref, network_info = self._get_running_instance()
+        self.connection.post_interrupted_snapshot_cleanup(self.ctxt,
+                instance_ref)
+
     @catch_notimplementederror
     def test_reboot(self):
         reboot_type = "SOFT"
@@ -376,7 +387,7 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
     @catch_notimplementederror
     def test_suspend(self):
         instance_ref, network_info = self._get_running_instance()
-        self.connection.suspend(instance_ref)
+        self.connection.suspend(self.ctxt, instance_ref)
 
     @catch_notimplementederror
     def test_resume_unsuspended_instance(self):
@@ -386,7 +397,7 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
     @catch_notimplementederror
     def test_resume_suspended_instance(self):
         instance_ref, network_info = self._get_running_instance()
-        self.connection.suspend(instance_ref)
+        self.connection.suspend(self.ctxt, instance_ref)
         self.connection.resume(self.ctxt, instance_ref, network_info)
 
     @catch_notimplementederror
@@ -419,21 +430,25 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
             "driver_volume_type": "fake",
             "serial": "fake_serial",
         }
-        self.connection.attach_volume(None, connection_info, instance_ref,
-                                      '/dev/sda')
-        self.connection.detach_volume(connection_info, instance_ref,
-                                      '/dev/sda')
+        self.assertIsNone(
+            self.connection.attach_volume(None, connection_info, instance_ref,
+                                          '/dev/sda'))
+        self.assertIsNone(
+            self.connection.detach_volume(connection_info, instance_ref,
+                                      '/dev/sda'))
 
     @catch_notimplementederror
     def test_swap_volume(self):
         instance_ref, network_info = self._get_running_instance()
-        self.connection.attach_volume(None, {'driver_volume_type': 'fake'},
-                                      instance_ref,
-                                      '/dev/sda')
-        self.connection.swap_volume({'driver_volume_type': 'fake'},
-                                    {'driver_volume_type': 'fake'},
-                                    instance_ref,
-                                    '/dev/sda')
+        self.assertIsNone(
+            self.connection.attach_volume(None, {'driver_volume_type': 'fake'},
+                                          instance_ref,
+                                          '/dev/sda'))
+        self.assertIsNone(
+            self.connection.swap_volume({'driver_volume_type': 'fake'},
+                                        {'driver_volume_type': 'fake'},
+                                        instance_ref,
+                                        '/dev/sda', 2))
 
     @catch_notimplementederror
     def test_attach_detach_different_power_states(self):
@@ -450,24 +465,29 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
             'root_device_name': None,
             'swap': None,
             'ephemerals': [],
-            'block_device_mapping': [
-                test_libvirt.mocked_bdm(1, {
-                        'instance_uuid': instance_ref['uuid'],
-                        'connection_info': {'driver_volume_type': 'fake'},
-                        'mount_device': '/dev/sda',
+            'block_device_mapping': driver_block_device.convert_volumes([
+                fake_block_device.FakeDbBlockDeviceDict(
+                       {'id': 1, 'instance_uuid': instance_ref['uuid'],
+                        'device_name': '/dev/sda',
+                        'source_type': 'volume',
+                        'destination_type': 'volume',
                         'delete_on_termination': False,
-                        'virtual_name': None,
                         'snapshot_id': None,
                         'volume_id': 'abcdedf',
                         'volume_size': None,
                         'no_device': None
                         }),
-                ]
+                ])
         }
-        self.connection.power_on(self.ctxt, instance_ref, network_info, bdm)
-        self.connection.detach_volume(connection_info,
-                                      instance_ref,
-                                      '/dev/sda')
+        bdm['block_device_mapping'][0]['connection_info'] = (
+            {'driver_volume_type': 'fake'})
+        with mock.patch.object(
+                driver_block_device.DriverVolumeBlockDevice, 'save'):
+            self.connection.power_on(
+                    self.ctxt, instance_ref, network_info, bdm)
+            self.connection.detach_volume(connection_info,
+                                          instance_ref,
+                                          '/dev/sda')
 
     @catch_notimplementederror
     def test_get_info(self):
@@ -490,6 +510,12 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         instance_ref, network_info = self._get_running_instance(obj=True)
         self.connection.get_diagnostics(instance_ref)
 
+    @catch_notimplementederror
+    def test_get_instance_diagnostics(self):
+        instance_ref, network_info = self._get_running_instance(obj=True)
+        instance_ref['launched_at'] = timeutils.utcnow()
+        self.connection.get_instance_diagnostics(instance_ref)
+
     @catch_notimplementederror
     def test_block_stats(self):
         instance_ref, network_info = self._get_running_instance()
@@ -514,27 +540,27 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
     def test_get_vnc_console(self):
         instance, network_info = self._get_running_instance(obj=True)
         vnc_console = self.connection.get_vnc_console(self.ctxt, instance)
-        self.assertIn('internal_access_path', vnc_console)
-        self.assertIn('host', vnc_console)
-        self.assertIn('port', vnc_console)
+        self.assertIsInstance(vnc_console, ctype.ConsoleVNC)
 
     @catch_notimplementederror
     def test_get_spice_console(self):
         instance_ref, network_info = self._get_running_instance()
         spice_console = self.connection.get_spice_console(self.ctxt,
-                instance_ref)
-        self.assertIn('internal_access_path', spice_console)
-        self.assertIn('host', spice_console)
-        self.assertIn('port', spice_console)
-        self.assertIn('tlsPort', spice_console)
+                                                          instance_ref)
+        self.assertIsInstance(spice_console, ctype.ConsoleSpice)
 
     @catch_notimplementederror
     def test_get_rdp_console(self):
         instance_ref, network_info = self._get_running_instance()
         rdp_console = self.connection.get_rdp_console(self.ctxt, instance_ref)
-        self.assertIn('internal_access_path', rdp_console)
-        self.assertIn('host', rdp_console)
-        self.assertIn('port', rdp_console)
+        self.assertIsInstance(rdp_console, ctype.ConsoleRDP)
+
+    @catch_notimplementederror
+    def test_get_serial_console(self):
+        instance_ref, network_info = self._get_running_instance()
+        serial_console = self.connection.get_serial_console(self.ctxt,
+                                                            instance_ref)
+        self.assertIsInstance(serial_console, ctype.ConsoleSerial)
 
     @catch_notimplementederror
     def test_get_console_pool_info(self):
@@ -594,12 +620,12 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
                 'supported_instances']
         for key in keys:
             self.assertIn(key, host_status)
+        self.assertIsInstance(host_status['hypervisor_version'], int)
 
     @catch_notimplementederror
     def test_get_host_stats(self):
         host_status = self.connection.get_host_stats()
         self._check_available_resource_fields(host_status)
-        self.assertIsInstance(host_status['hypervisor_version'], int)
 
     @catch_notimplementederror
     def test_get_available_resource(self):
@@ -718,6 +744,13 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.assertRaises(NotImplementedError, self.connection.set_bootable,
                           'instance', True)
 
+    @catch_notimplementederror
+    def test_get_instance_disk_info(self):
+        # This should be implemented by any driver that supports live migrate.
+        instance_ref, network_info = self._get_running_instance()
+        self.connection.get_instance_disk_info(instance_ref['name'],
+                                               block_device_info={})
+
 
 class AbstractDriverTestCase(_VirtDriverTestCase, test.TestCase):
     def setUp(self):
@@ -775,8 +808,7 @@ class LibvirtConnTestCase(_VirtDriverTestCase, test.TestCase):
         # Previous status of the service: disabled: False
         service_mock.configure_mock(disabled_reason='None',
                                     disabled=False)
-        from nova.objects import service as service_obj
-        with mock.patch.object(service_obj.Service, "get_by_compute_host",
+        with mock.patch.object(objects.Service, "get_by_compute_host",
                                return_value=service_mock):
             self.connection._set_host_enabled(False, 'ERROR!')
             self.assertTrue(service_mock.disabled)
@@ -789,8 +821,7 @@ class LibvirtConnTestCase(_VirtDriverTestCase, test.TestCase):
         # Previous status of the service: disabled: True, 'AUTO: ERROR'
         service_mock.configure_mock(disabled_reason='AUTO: ERROR',
                                     disabled=True)
-        from nova.objects import service as service_obj
-        with mock.patch.object(service_obj.Service, "get_by_compute_host",
+        with mock.patch.object(objects.Service, "get_by_compute_host",
                                return_value=service_mock):
             self.connection._set_host_enabled(True)
             self.assertFalse(service_mock.disabled)
@@ -803,8 +834,7 @@ class LibvirtConnTestCase(_VirtDriverTestCase, test.TestCase):
         # Previous status of the service: disabled: True, 'Manually disabled'
         service_mock.configure_mock(disabled_reason='Manually disabled',
                                     disabled=True)
-        from nova.objects import service as service_obj
-        with mock.patch.object(service_obj.Service, "get_by_compute_host",
+        with mock.patch.object(objects.Service, "get_by_compute_host",
                                return_value=service_mock):
             self.connection._set_host_enabled(True)
             self.assertTrue(service_mock.disabled)
@@ -817,9 +847,26 @@ class LibvirtConnTestCase(_VirtDriverTestCase, test.TestCase):
         # Previous status of the service: disabled: True, 'Manually disabled'
         service_mock.configure_mock(disabled_reason='Manually disabled',
                                     disabled=True)
-        from nova.objects import service as service_obj
-        with mock.patch.object(service_obj.Service, "get_by_compute_host",
+        with mock.patch.object(objects.Service, "get_by_compute_host",
                                return_value=service_mock):
             self.connection._set_host_enabled(False, 'ERROR!')
             self.assertTrue(service_mock.disabled)
             self.assertEqual(service_mock.disabled_reason, 'Manually disabled')
+
+    @catch_notimplementederror
+    @mock.patch.object(libvirt.driver.LibvirtDriver, '_unplug_vifs')
+    def test_unplug_vifs_with_destroy_vifs_false(self, unplug_vifs_mock):
+        instance_ref, network_info = self._get_running_instance()
+        self.connection.cleanup(self.ctxt, instance_ref, network_info,
+                                destroy_vifs=False)
+        self.assertEqual(unplug_vifs_mock.call_count, 0)
+
+    @catch_notimplementederror
+    @mock.patch.object(libvirt.driver.LibvirtDriver, '_unplug_vifs')
+    def test_unplug_vifs_with_destroy_vifs_true(self, unplug_vifs_mock):
+        instance_ref, network_info = self._get_running_instance()
+        self.connection.cleanup(self.ctxt, instance_ref, network_info,
+                                destroy_vifs=True)
+        self.assertEqual(unplug_vifs_mock.call_count, 1)
+        unplug_vifs_mock.assert_called_once_with(instance_ref,
+                                            network_info, True)
