@@ -12,9 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.db import exception as db_exc
+from sqlalchemy import MetaData, Table, exists
+from sqlalchemy import sql
+
 from nova.openstack.common import log as logging
-from sqlalchemy import and_, MetaData, select, Table, exists
-from sqlalchemy import exc
 
 LOG = logging.getLogger(__name__)
 
@@ -26,7 +28,7 @@ def upgrade(migrate_engine):
     nodes = Table('bm_nodes', meta, autoload=True)
     ifs = Table('bm_interfaces', meta, autoload=True)
 
-    q = select([nodes.c.id, nodes.c.prov_mac_address],
+    q = sql.select([nodes.c.id, nodes.c.prov_mac_address],
                from_obj=nodes)
 
     # Iterate all elements before starting insert since IntegrityError
@@ -39,7 +41,9 @@ def upgrade(migrate_engine):
     for node_id, address in node_address.iteritems():
         try:
             i.execute({'bm_node_id': node_id, 'address': address})
-        except exc.IntegrityError:
+        except db_exc.DBError:
+            # TODO(ekudryashova): replace by DBReferenceError when db layer
+            # raise it.
             # The address is registered in both bm_nodes and bm_interfaces.
             # It is expected.
             pass
@@ -52,8 +56,9 @@ def downgrade(migrate_engine):
     nodes = Table('bm_nodes', meta, autoload=True)
     ifs = Table('bm_interfaces', meta, autoload=True)
 
-    subq = exists().where(and_(ifs.c.bm_node_id == nodes.c.id,
-                               ifs.c.address == nodes.c.prov_mac_address))
+    subq = exists().where(sql.and_(
+        ifs.c.bm_node_id == nodes.c.id,
+        ifs.c.address == nodes.c.prov_mac_address))
 
     ifs.delete().where(subq).execute()
 
