@@ -22,13 +22,17 @@ A driver for Bare-metal platform.
 
 from oslo.config import cfg
 
+from nova.compute import arch
 from nova.compute import flavors
+from nova.compute import hvtype
 from nova.compute import power_state
 from nova.compute import task_states
+from nova.compute import vm_mode
 from nova import context as nova_context
 from nova import exception
+from nova.i18n import _
+from nova.i18n import _LW
 from nova.openstack.common import excutils
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import importutils
 from nova.openstack.common import jsonutils
 from nova.openstack.common import lockutils
@@ -53,8 +57,7 @@ opts = [
                help='A list of additional capabilities corresponding to '
                'flavor_extra_specs for this compute '
                'host to advertise. Valid entries are name=value, pairs '
-               'For example, "key1:val1, key2:val2"',
-               deprecated_name='instance_type_extra_specs'),
+               'For example, "key1:val1, key2:val2"'),
     cfg.StrOpt('driver',
                default='nova.virt.baremetal.pxe.PXE',
                help='Baremetal driver back-end (pxe or tilera)'),
@@ -136,15 +139,19 @@ class BareMetalDriver(driver.ComputeDriver):
             keyval[0] = keyval[0].strip()
             keyval[1] = keyval[1].strip()
             extra_specs[keyval[0]] = keyval[1]
-        if 'cpu_arch' not in extra_specs:
-            LOG.warning(
-                    _('cpu_arch is not found in flavor_extra_specs'))
-            extra_specs['cpu_arch'] = ''
+
         self.extra_specs = extra_specs
 
-        self.supported_instances = [
-                (extra_specs['cpu_arch'], 'baremetal', 'baremetal'),
-                ]
+        if 'cpu_arch' not in extra_specs:
+            LOG.info(
+                _('cpu_arch is not found in flavor_extra_specs'))
+            self.supported_instances = []
+        else:
+            self.supported_instances = [(
+                arch.canonicalize(extra_specs['cpu_arch']),
+                hvtype.BAREMETAL,
+                vm_mode.HVM
+            ), ]
 
     @classmethod
     def instance(cls):
@@ -153,7 +160,11 @@ class BareMetalDriver(driver.ComputeDriver):
         return cls._instance
 
     def init_host(self, host):
-        return
+        LOG.warn(_LW('The baremetal driver is deprecated in Juno and will be '
+                     'removed before the next release. Please plan to '
+                     'transition to Ironic as soon as possible. See '
+                     'https://wiki.openstack.org/wiki/Ironic for more '
+                     'information'))
 
     def get_hypervisor_type(self):
         return 'baremetal'
@@ -212,6 +223,9 @@ class BareMetalDriver(driver.ComputeDriver):
         self.firewall_driver.unfilter_instance(
                 instance, network_info)
 
+    def deallocate_networks_on_reschedule(self, instance):
+        return True
+
     def macs_for_instance(self, instance):
         context = nova_context.get_admin_context()
         node_uuid = self._require_node(instance)
@@ -222,10 +236,8 @@ class BareMetalDriver(driver.ComputeDriver):
     def _set_default_ephemeral_device(self, instance):
         flavor = flavors.extract_flavor(instance)
         if flavor['ephemeral_gb']:
-            self.virtapi.instance_update(
-                nova_context.get_admin_context(), instance['uuid'],
-                {'default_ephemeral_device':
-                    '/dev/sda1'})
+            instance.default_ephemeral_device = '/dev/sda1'
+            instance.save()
 
     def spawn(self, context, instance, image_meta, injected_files,
               admin_password, network_info=None, block_device_info=None):
@@ -362,7 +374,8 @@ class BareMetalDriver(driver.ComputeDriver):
                 "for instance %r") % instance['uuid'])
         _update_state(ctx, node, instance, state)
 
-    def destroy(self, context, instance, network_info, block_device_info=None):
+    def destroy(self, context, instance, network_info, block_device_info=None,
+                destroy_disks=True, migrate_data=None):
         context = nova_context.get_admin_context()
 
         try:
@@ -395,12 +408,13 @@ class BareMetalDriver(driver.ComputeDriver):
                                 "baremetal database: %s") % e)
 
     def cleanup(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True):
+                destroy_disks=True, migrate_data=None, destroy_vifs=True):
         """Cleanup after instance being destroyed."""
         pass
 
-    def power_off(self, instance, node=None):
+    def power_off(self, instance, timeout=0, retry_interval=0, node=None):
         """Power off the specified instance."""
+        # TODO(PhilDay): Add support for timeout (clean shutdown)
         if not node:
             node = _get_baremetal_node_by_instance_uuid(instance['uuid'])
         pm = get_power_manager(node=node, instance=instance)
