@@ -16,10 +16,11 @@ import functools
 import operator
 
 from nova import block_device
-from nova.objects import block_device as block_device_obj
+from nova.i18n import _
+from nova.i18n import _LI
+from nova import objects
+from nova.objects import base as obj_base
 from nova.openstack.common import excutils
-from nova.openstack.common.gettextutils import _
-from nova.openstack.common.gettextutils import _LI
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.volume import encryptors
@@ -83,10 +84,10 @@ class DriverBlockDevice(dict):
     def __init__(self, bdm):
         # TODO(ndipanov): Remove this check when we have all the rpc methods
         # use objects for block devices.
-        if isinstance(bdm, block_device_obj.BlockDeviceMapping):
+        if isinstance(bdm, obj_base.NovaObject):
             self.__dict__['_bdm_obj'] = bdm
         else:
-            self.__dict__['_bdm_obj'] = block_device_obj.BlockDeviceMapping()
+            self.__dict__['_bdm_obj'] = objects.BlockDeviceMapping()
             self._bdm_obj.update(block_device.BlockDeviceDict(bdm))
             self._bdm_obj.obj_reset_changes()
 
@@ -101,15 +102,13 @@ class DriverBlockDevice(dict):
         if name in self._proxy_as_attr:
             return getattr(self._bdm_obj, name)
         else:
-            raise AttributeError("Cannot access %s on DriverBlockDevice "
-                                  "class" % name)
+            super(DriverBlockDevice, self).__getattr__(name)
 
     def __setattr__(self, name, value):
         if name in self._proxy_as_attr:
             return setattr(self._bdm_obj, name, value)
         else:
-            raise AttributeError("Cannot access %s on DriverBlockDevice "
-                                  "class" % name)
+            super(DriverBlockDevice, self).__setattr__(name, value)
 
     def _transform(self):
         """Transform bdm to the format that is passed to drivers."""
@@ -126,15 +125,19 @@ class DriverBlockDevice(dict):
     def attach(self, **kwargs):
         """Make the device available to be used by VMs.
 
-        To be overriden in subclasses with the connecting logic for
+        To be overridden in subclasses with the connecting logic for
         the type of device the subclass represents.
         """
         raise NotImplementedError()
 
-    def save(self, context):
+    def save(self, context=None):
         for attr_name, key_name in self._update_on_save.iteritems():
             setattr(self._bdm_obj, attr_name, self[key_name or attr_name])
-        self._bdm_obj.save(context)
+
+        if context:
+            self._bdm_obj.save(context)
+        else:
+            self._bdm_obj.save()
 
 
 class DriverSwapBlockDevice(DriverBlockDevice):
@@ -261,8 +264,14 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
         mode = 'rw'
         if 'data' in connection_info:
             mode = connection_info['data'].get('access_mode', 'rw')
-        volume_api.attach(context, volume_id, instance['uuid'],
-                          self['mount_device'], mode=mode)
+        if volume['attach_status'] == "detached":
+            # NOTE(mriedem): save our current state so connection_info is in
+            # the database before the volume status goes to 'in-use' because
+            # after that we can detach and connection_info is required for
+            # detach.
+            self.save(context)
+            volume_api.attach(context, volume_id, instance['uuid'],
+                              self['mount_device'], mode=mode)
 
     @update_db
     def refresh_connection_info(self, context, instance,
@@ -280,7 +289,7 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
         self._preserve_multipath_id(connection_info)
         self['connection_info'] = connection_info
 
-    def save(self, context):
+    def save(self, context=None):
         # NOTE(ndipanov): we might want to generalize this by adding it to the
         # _update_on_save and adding a transformation function.
         try:
@@ -297,7 +306,7 @@ class DriverSnapshotBlockDevice(DriverVolumeBlockDevice):
     _proxy_as_attr = set(['volume_size', 'volume_id', 'snapshot_id'])
 
     def attach(self, context, instance, volume_api,
-               virt_driver, wait_func=None):
+               virt_driver, wait_func=None, do_check_attach=True):
 
         if not self.volume_id:
             snapshot = volume_api.get_snapshot(context,
@@ -310,8 +319,9 @@ class DriverSnapshotBlockDevice(DriverVolumeBlockDevice):
             self.volume_id = vol['id']
 
         # Call the volume attach now
-        super(DriverSnapshotBlockDevice, self).attach(context, instance,
-                                                      volume_api, virt_driver)
+        super(DriverSnapshotBlockDevice, self).attach(
+            context, instance, volume_api, virt_driver,
+            do_check_attach=do_check_attach)
 
 
 class DriverImageBlockDevice(DriverVolumeBlockDevice):
@@ -320,7 +330,7 @@ class DriverImageBlockDevice(DriverVolumeBlockDevice):
     _proxy_as_attr = set(['volume_size', 'volume_id', 'image_id'])
 
     def attach(self, context, instance, volume_api,
-               virt_driver, wait_func=None):
+               virt_driver, wait_func=None, do_check_attach=True):
         if not self.volume_id:
             vol = volume_api.create(context, self.volume_size,
                                     '', '', image_id=self.image_id)
@@ -329,8 +339,29 @@ class DriverImageBlockDevice(DriverVolumeBlockDevice):
 
             self.volume_id = vol['id']
 
-        super(DriverImageBlockDevice, self).attach(context, instance,
-                                                   volume_api, virt_driver)
+        super(DriverImageBlockDevice, self).attach(
+            context, instance, volume_api, virt_driver,
+            do_check_attach=do_check_attach)
+
+
+class DriverBlankBlockDevice(DriverVolumeBlockDevice):
+
+    _valid_source = 'blank'
+    _proxy_as_attr = set(['volume_size', 'volume_id', 'image_id'])
+
+    def attach(self, context, instance, volume_api,
+               virt_driver, wait_func=None, do_check_attach=True):
+        if not self.volume_id:
+            vol_name = instance.uuid + '-blank-vol'
+            vol = volume_api.create(context, self.volume_size, vol_name, '')
+            if wait_func:
+                wait_func(context, vol['id'])
+
+            self.volume_id = vol['id']
+
+        super(DriverBlankBlockDevice, self).attach(
+            context, instance, volume_api, virt_driver,
+            do_check_attach=do_check_attach)
 
 
 def _convert_block_devices(device_type, block_device_mapping):
@@ -364,6 +395,9 @@ convert_snapshots = functools.partial(_convert_block_devices,
 convert_images = functools.partial(_convert_block_devices,
                                      DriverImageBlockDevice)
 
+convert_blanks = functools.partial(_convert_block_devices,
+                                   DriverBlankBlockDevice)
+
 
 def attach_block_devices(block_device_mapping, *attach_args, **attach_kwargs):
     def _log_and_attach(bdm):
@@ -417,7 +451,7 @@ def get_swap(transformed_list):
     if not all(isinstance(device, DriverSwapBlockDevice) or
                'swap_size' in device
                 for device in transformed_list):
-        return transformed_list
+        return None
     try:
         return transformed_list.pop()
     except IndexError:
@@ -426,7 +460,7 @@ def get_swap(transformed_list):
 
 _IMPLEMENTED_CLASSES = (DriverSwapBlockDevice, DriverEphemeralBlockDevice,
                         DriverVolumeBlockDevice, DriverSnapshotBlockDevice,
-                        DriverImageBlockDevice)
+                        DriverImageBlockDevice, DriverBlankBlockDevice)
 
 
 def is_implemented(bdm):
@@ -437,3 +471,9 @@ def is_implemented(bdm):
         except _NotTransformable:
             pass
     return False
+
+
+def is_block_device_mapping(bdm):
+    return (bdm.source_type in ('image', 'volume', 'snapshot', 'blank')
+            and bdm.destination_type == 'volume'
+            and is_implemented(bdm))
