@@ -16,7 +16,7 @@
 import os
 import time
 
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import importutils
 from nova.openstack.common import log as logging
 from nova import utils
@@ -37,36 +37,36 @@ class Mount(object):
 
     @staticmethod
     def instance_for_format(imgfile, mountdir, partition, imgfmt):
-        LOG.debug(_("Instance for format imgfile=%(imgfile)s "
-                    "mountdir=%(mountdir)s partition=%(partition)s "
-                    "imgfmt=%(imgfmt)s"),
+        LOG.debug("Instance for format imgfile=%(imgfile)s "
+                  "mountdir=%(mountdir)s partition=%(partition)s "
+                  "imgfmt=%(imgfmt)s",
                   {'imgfile': imgfile, 'mountdir': mountdir,
                    'partition': partition, 'imgfmt': imgfmt})
         if imgfmt == "raw":
-            LOG.debug(_("Using LoopMount"))
+            LOG.debug("Using LoopMount")
             return importutils.import_object(
                 "nova.virt.disk.mount.loop.LoopMount",
                 imgfile, mountdir, partition)
         else:
-            LOG.debug(_("Using NbdMount"))
+            LOG.debug("Using NbdMount")
             return importutils.import_object(
                 "nova.virt.disk.mount.nbd.NbdMount",
                 imgfile, mountdir, partition)
 
     @staticmethod
     def instance_for_device(imgfile, mountdir, partition, device):
-        LOG.debug(_("Instance for device imgfile=%(imgfile)s "
-                    "mountdir=%(mountdir)s partition=%(partition)s "
-                    "device=%(device)s"),
+        LOG.debug("Instance for device imgfile=%(imgfile)s "
+                  "mountdir=%(mountdir)s partition=%(partition)s "
+                  "device=%(device)s",
                   {'imgfile': imgfile, 'mountdir': mountdir,
                    'partition': partition, 'device': device})
         if "loop" in device:
-            LOG.debug(_("Using LoopMount"))
+            LOG.debug("Using LoopMount")
             return importutils.import_object(
                 "nova.virt.disk.mount.loop.LoopMount",
                 imgfile, mountdir, partition, device)
         else:
-            LOG.debug(_("Using NbdMount"))
+            LOG.debug("Using NbdMount")
             return importutils.import_object(
                 "nova.virt.disk.mount.nbd.NbdMount",
                 imgfile, mountdir, partition, device)
@@ -135,7 +135,7 @@ class Mount(object):
     def map_dev(self):
         """Map partitions of the device to the file system namespace."""
         assert(os.path.exists(self.device))
-        LOG.debug(_("Map dev %s"), self.device)
+        LOG.debug("Map dev %s", self.device)
         automapped_path = '/dev/%sp%s' % (os.path.basename(self.device),
                                               self.partition)
 
@@ -179,7 +179,7 @@ class Mount(object):
         """Remove partitions of the device from the file system namespace."""
         if not self.mapped:
             return
-        LOG.debug(_("Unmap dev %s"), self.device)
+        LOG.debug("Unmap dev %s", self.device)
         if self.partition and not self.automapped:
             utils.execute('kpartx', '-d', self.device, run_as_root=True)
         self.mapped = False
@@ -187,7 +187,7 @@ class Mount(object):
 
     def mnt_dev(self):
         """Mount the device into the file system."""
-        LOG.debug(_("Mount %(dev)s on %(dir)s") %
+        LOG.debug("Mount %(dev)s on %(dir)s",
                   {'dev': self.mapped_device, 'dir': self.mount_dir})
         _out, err = utils.trycmd('mount', self.mapped_device, self.mount_dir,
                                  discard_warnings=True, run_as_root=True)
@@ -204,7 +204,7 @@ class Mount(object):
         if not self.mounted:
             return
         self.flush_dev()
-        LOG.debug(_("Umount %s") % self.mapped_device)
+        LOG.debug("Umount %s", self.mapped_device)
         utils.execute('umount', self.mapped_device, run_as_root=True)
         self.mounted = False
 
@@ -218,7 +218,7 @@ class Mount(object):
             status = self.get_dev() and self.map_dev() and self.mnt_dev()
         finally:
             if not status:
-                LOG.debug(_("Fail to mount, tearing back down"))
+                LOG.debug("Fail to mount, tearing back down")
                 self.do_teardown()
         return status
 
