@@ -83,6 +83,40 @@ class FakeInstance(object):
         return getattr(self, key)
 
 
+class Resources(object):
+    vcpus = 0
+    memory_mb = 0
+    local_gb = 0
+    vcpus_used = 0
+    memory_mb_used = 0
+    local_gb_used = 0
+
+    def __init__(self, vcpus=8, memory_mb=8000, local_gb=500):
+        self.vcpus = vcpus
+        self.memory_mb = memory_mb
+        self.local_gb = local_gb
+
+    def claim(self, vcpus=0, mem=0, disk=0):
+        self.vcpus_used += vcpus
+        self.memory_mb_used += mem
+        self.local_gb_used += disk
+
+    def release(self, vcpus=0, mem=0, disk=0):
+        self.vcpus_used -= vcpus
+        self.memory_mb_used -= mem
+        self.local_gb_used -= disk
+
+    def dump(self):
+        return {
+            'vcpus': self.vcpus,
+            'memory_mb': self.memory_mb,
+            'local_gb': self.local_gb,
+            'vcpus_used': self.vcpus_used,
+            'memory_mb_used': self.memory_mb_used,
+            'local_gb_used': self.local_gb_used
+        }
+
+
 class FakeDriver(driver.ComputeDriver):
     capabilities = {
         "has_imagecache": True,
@@ -100,13 +134,11 @@ class FakeDriver(driver.ComputeDriver):
     def __init__(self, virtapi, read_only=False):
         super(FakeDriver, self).__init__(virtapi)
         self.instances = {}
+        self.resources = Resources(
+            vcpus=self.vcpus,
+            memory_mb=self.memory_mb,
+            local_gb=self.local_gb)
         self.host_status_base = {
-          'vcpus': self.vcpus,
-          'memory_mb': self.memory_mb,
-          'local_gb': self.local_gb,
-          'vcpus_used': 0,
-          'memory_mb_used': 0,
-          'local_gb_used': 100000000000,
           'hypervisor_type': 'fake',
           'hypervisor_version': utils.convert_version_to_int('1.0'),
           'hypervisor_hostname': CONF.host,
@@ -141,6 +173,10 @@ class FakeDriver(driver.ComputeDriver):
               admin_password, network_info=None, block_device_info=None):
         name = instance['name']
         state = power_state.RUNNING
+        self.resources.claim(
+            vcpus=instance.get('vcpus') or 0,
+            mem=instance.get('memory_mb') or 0,
+            disk=instance['root_gb'] or 0)
         fake_instance = FakeInstance(name, state, instance['uuid'])
         self.instances[name] = fake_instance
 
@@ -211,7 +247,7 @@ class FakeDriver(driver.ComputeDriver):
     def unpause(self, instance):
         pass
 
-    def suspend(self, instance):
+    def suspend(self, context, instance):
         pass
 
     def resume(self, context, instance, network_info, block_device_info=None):
@@ -221,6 +257,10 @@ class FakeDriver(driver.ComputeDriver):
                 destroy_disks=True, migrate_data=None):
         key = instance['name']
         if key in self.instances:
+            self.resources.release(
+                vcpus=instance.get('vcpus') or 0,
+                mem=instance.get('memory_mb') or 0,
+                disk=instance.get('root_gb') or 0)
             del self.instances[key]
         else:
             LOG.warning(_("Key '%(key)s' not in instances '%(inst)s'") %
@@ -453,6 +493,7 @@ class FakeDriver(driver.ComputeDriver):
         stats = []
         for nodename in _FAKE_NODES:
             host_status = self.host_status_base.copy()
+            host_status.update(self.resources.dump())
             host_status['hypervisor_hostname'] = nodename
             host_status['host_hostname'] = nodename
             host_status['host_name_label'] = nodename
