@@ -21,12 +21,14 @@ from nova.compute import power_state
 
 HYPERV_VM_STATE_ENABLED = 2
 HYPERV_VM_STATE_DISABLED = 3
+HYPERV_VM_STATE_SHUTTING_DOWN = 4
 HYPERV_VM_STATE_REBOOT = 10
 HYPERV_VM_STATE_PAUSED = 32768
 HYPERV_VM_STATE_SUSPENDED = 32769
 
 HYPERV_POWER_STATE = {
     HYPERV_VM_STATE_DISABLED: power_state.SHUTDOWN,
+    HYPERV_VM_STATE_SHUTTING_DOWN: power_state.SHUTDOWN,
     HYPERV_VM_STATE_ENABLED: power_state.RUNNING,
     HYPERV_VM_STATE_PAUSED: power_state.PAUSED,
     HYPERV_VM_STATE_SUSPENDED: power_state.SUSPENDED
