@@ -16,12 +16,13 @@
 import os
 import shutil
 
-from nova.openstack.common.gettextutils import _
+from oslo.config import cfg
+
+from nova.i18n import _
 from nova.openstack.common import log as logging
 from nova import utils
 from nova.virt.hyperv import constants
-
-from oslo.config import cfg
+from nova.virt.hyperv import vmutils
 
 LOG = logging.getLogger(__name__)
 
@@ -39,6 +40,8 @@ CONF = cfg.CONF
 CONF.register_opts(hyperv_opts, 'hyperv')
 CONF.import_opt('instances_path', 'nova.compute.manager')
 
+ERROR_INVALID_NAME = 123
+
 
 class PathUtils(object):
     def open(self, path, mode):
@@ -92,23 +95,34 @@ class PathUtils(object):
 
     def _check_create_dir(self, path):
         if not self.exists(path):
-            LOG.debug(_('Creating directory: %s') % path)
+            LOG.debug('Creating directory: %s', path)
             self.makedirs(path)
 
     def _check_remove_dir(self, path):
         if self.exists(path):
-            LOG.debug(_('Removing directory: %s') % path)
+            LOG.debug('Removing directory: %s', path)
             self.rmtree(path)
 
     def _get_instances_sub_dir(self, dir_name, remote_server=None,
                                create_dir=True, remove_dir=False):
         instances_path = self.get_instances_dir(remote_server)
         path = os.path.join(instances_path, dir_name)
-        if remove_dir:
-            self._check_remove_dir(path)
-        if create_dir:
-            self._check_create_dir(path)
-        return path
+        try:
+            if remove_dir:
+                self._check_remove_dir(path)
+            if create_dir:
+                self._check_create_dir(path)
+            return path
+        except WindowsError as ex:
+            if ex.winerror == ERROR_INVALID_NAME:
+                raise vmutils.HyperVException(_(
+                    "Cannot access \"%(instances_path)s\", make sure the "
+                    "path exists and that you have the proper permissions. "
+                    "In particular Nova-Compute must not be executed with the "
+                    "builtin SYSTEM account or other accounts unable to "
+                    "authenticate on a remote host.") %
+                    {'instances_path': instances_path})
+            raise
 
     def get_instance_migr_revert_dir(self, instance_name, create_dir=False,
                                      remove_dir=False):
@@ -150,8 +164,9 @@ class PathUtils(object):
         instance_path = self.get_instance_dir(instance_name)
         return os.path.join(instance_path, 'root.' + format_ext.lower())
 
-    def get_configdrive_path(self, instance_name, format_ext):
-        instance_path = self.get_instance_dir(instance_name)
+    def get_configdrive_path(self, instance_name, format_ext,
+                             remote_server=None):
+        instance_path = self.get_instance_dir(instance_name, remote_server)
         return os.path.join(instance_path, 'configdrive.' + format_ext.lower())
 
     def get_ephemeral_vhd_path(self, instance_name, format_ext):
@@ -165,3 +180,18 @@ class PathUtils(object):
         dir_name = os.path.join('export', instance_name)
         return self._get_instances_sub_dir(dir_name, create_dir=True,
                                            remove_dir=True)
+
+    def get_vm_console_log_paths(self, vm_name, remote_server=None):
+        instance_dir = self.get_instance_dir(vm_name,
+                                             remote_server)
+        console_log_path = os.path.join(instance_dir, 'console.log')
+        return console_log_path, console_log_path + '.1'
+
+    def copy_configdrive(self, instance_name, dest_host):
+        local_configdrive_path = self.get_configdrive_path(
+                        instance_name, constants.IDE_DVD_FORMAT)
+        remote_configdrive_path = self.get_configdrive_path(
+                instance_name, constants.IDE_DVD_FORMAT,
+                remote_server=dest_host)
+        self.copyfile(local_configdrive_path,
+                                 remote_configdrive_path)
