@@ -22,22 +22,22 @@ imagecache_opts = [
     cfg.IntOpt('image_cache_manager_interval',
                default=2400,
                help='Number of seconds to wait between runs of the image '
-                    'cache manager'),
+                    'cache manager. Set to -1 to disable. '
+                    'Setting this to 0 will disable, but this will change in '
+                    'the K release to mean "run at the default rate".'),
+    # TODO(gilliard): Clean the above message after the K release
     cfg.StrOpt('image_cache_subdirectory_name',
                default='_base',
                help='Where cached images are stored under $instances_path. '
                     'This is NOT the full path - just a folder name. '
-                    'For per-compute-host cached images, set to _base_$my_ip',
-               deprecated_name='base_dir_name'),
+                    'For per-compute-host cached images, set to _base_$my_ip'),
     cfg.BoolOpt('remove_unused_base_images',
                 default=True,
-                help='Should unused base images be removed?',
-                deprecated_group='libvirt'),
+                help='Should unused base images be removed?'),
     cfg.IntOpt('remove_unused_original_minimum_age_seconds',
                default=(24 * 3600),
                help='Unused unresized base images younger than this will not '
-                    'be removed',
-               deprecated_group='libvirt'),
+                    'be removed'),
     ]
 
 CONF = cfg.CONF
@@ -78,26 +78,24 @@ class ImageCacheManager(object):
             # NOTE(mikal): "instance name" here means "the name of a directory
             # which might contain an instance" and therefore needs to include
             # historical permutations as well as the current one.
-            instance_names.add(instance['name'])
-            instance_names.add(instance['uuid'])
-
-            if (instance['task_state'] in self.resize_states or
-                    instance['vm_state'] == vm_states.RESIZED):
-                instance_names.add(instance['name'] + '_resize')
-                instance_names.add(instance['uuid'] + '_resize')
+            instance_names.add(instance.name)
+            instance_names.add(instance.uuid)
+            if (instance.task_state in self.resize_states or
+                    instance.vm_state == vm_states.RESIZED):
+                instance_names.add(instance.name + '_resize')
+                instance_names.add(instance.uuid + '_resize')
 
             for image_key in ['image_ref', 'kernel_id', 'ramdisk_id']:
-                try:
-                    image_ref_str = str(instance[image_key])
-                except KeyError:
+                image_ref_str = getattr(instance, image_key)
+                if image_ref_str is None:
                     continue
                 local, remote, insts = used_images.get(image_ref_str,
                                                             (0, 0, []))
-                if instance['host'] == CONF.host:
+                if instance.host == CONF.host:
                     local += 1
                 else:
                     remote += 1
-                insts.append(instance['name'])
+                insts.append(instance.name)
                 used_images[image_ref_str] = (local, remote, insts)
 
                 image_popularity.setdefault(image_ref_str, 0)
@@ -118,7 +116,7 @@ class ImageCacheManager(object):
                 'originals': []}
 
     def _age_and_verify_cached_images(self, context, all_instances, base_dir):
-        """Ages and verfies cached images."""
+        """Ages and verifies cached images."""
 
         raise NotImplementedError()
 
