@@ -24,9 +24,9 @@ import os
 from oslo.config import cfg
 
 from nova import exception
-from nova.image import glance
+from nova.i18n import _
+from nova import image
 from nova.openstack.common import fileutils
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import imageutils
 from nova.openstack.common import log as logging
 from nova import utils
@@ -41,6 +41,7 @@ image_opts = [
 
 CONF = cfg.CONF
 CONF.register_opts(image_opts)
+IMAGE_API = image.API()
 
 
 def qemu_img_info(path):
@@ -48,10 +49,16 @@ def qemu_img_info(path):
     # TODO(mikal): this code should not be referring to a libvirt specific
     # flag.
     if not os.path.exists(path) and CONF.libvirt.images_type != 'rbd':
-        return imageutils.QemuImgInfo()
+        msg = (_("Path does not exist %(path)s") % {'path': path})
+        raise exception.InvalidDiskInfo(reason=msg)
 
     out, err = utils.execute('env', 'LC_ALL=C', 'LANG=C',
                              'qemu-img', 'info', path)
+    if not out:
+        msg = (_("Failed to run qemu-img info on %(path)s : %(error)s") %
+               {'path': path, 'error': err})
+        raise exception.InvalidDiskInfo(reason=msg)
+
     return imageutils.QemuImgInfo(out)
 
 
@@ -62,14 +69,8 @@ def convert_image(source, dest, out_format, run_as_root=False):
 
 
 def fetch(context, image_href, path, _user_id, _project_id, max_size=0):
-    # TODO(vish): Improve context handling and add owner and auth data
-    #             when it is added to glance.  Right now there is no
-    #             auth checking in glance, so we assume that access was
-    #             checked before we got here.
-    (image_service, image_id) = glance.get_remote_image_service(context,
-                                                                image_href)
     with fileutils.remove_path_on_error(path):
-        image_service.download(context, image_id, dst_path=path)
+        IMAGE_API.download(context, image_href, dest_path=path)
 
 
 def fetch_to_raw(context, image_href, path, user_id, project_id, max_size=0):
