@@ -244,7 +244,8 @@ class NWFilterFirewall(base_firewall.FirewallDriver):
             doc = etree.fromstring(xml)
             u = doc.find("./uuid").text
         except Exception as e:
-            LOG.debug("Cannot find UUID for filter '%s': '%s'" % (name, e))
+            LOG.debug(u"Cannot find UUID for filter '%(name)s': '%(e)s'",
+                {'name': name, 'e': e})
             u = uuid.uuid4().hex
 
         LOG.debug("UUID for filter '%s' is '%s'" % (name, u))
