@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 Michael Still and Canonical Inc
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,13 +13,18 @@
 #    under the License.
 
 
-import json
 import os
 import time
 
-from nova import utils
 from oslo.config import cfg
 
+from nova.i18n import _
+from nova.openstack.common import jsonutils
+from nova.openstack.common import log as logging
+from nova import utils
+
+LOG = logging.getLogger(__name__)
+
 
 CONF = cfg.CONF
 TWENTY_FOUR_HOURS = 3600 * 24
@@ -36,7 +39,7 @@ TWENTY_FOUR_HOURS = 3600 * 24
 # or method.  If this needs to be used outside of compute.manager, it should
 # be refactored to eliminate this circular dependency loop.
 def register_storage_use(storage_path, hostname):
-    """Idenfity the id of this instance storage."""
+    """Identify the id of this instance storage."""
 
     # NOTE(morganfainberg): config option import is avoided here since it is
     # explicitly imported from compute.manager and may cause issues with
@@ -55,12 +58,16 @@ def register_storage_use(storage_path, hostname):
         id_path = os.path.join(storage_path, 'compute_nodes')
         if os.path.exists(id_path):
             with open(id_path) as f:
-                d = json.loads(f.read())
+                try:
+                    d = jsonutils.loads(f.read())
+                except ValueError:
+                    LOG.warning(_("Cannot decode JSON from %(id_path)s"),
+                                {"id_path": id_path})
 
         d[hostname] = time.time()
 
         with open(id_path, 'w') as f:
-            f.write(json.dumps(d))
+            f.write(jsonutils.dumps(d))
 
     return do_register_storage_use(storage_path, hostname)
 
@@ -89,7 +96,11 @@ def get_storage_users(storage_path):
         id_path = os.path.join(storage_path, 'compute_nodes')
         if os.path.exists(id_path):
             with open(id_path) as f:
-                d = json.loads(f.read())
+                try:
+                    d = jsonutils.loads(f.read())
+                except ValueError:
+                    LOG.warning(_("Cannot decode JSON from %(id_path)s"),
+                                {"id_path": id_path})
 
         recent_users = []
         for node in d:
