@@ -17,66 +17,26 @@
 Management class for host-related functions (start, reboot, etc).
 """
 
+from nova.compute import arch
+from nova.compute import hvtype
+from nova.compute import vm_mode
 from nova import exception
-from nova.openstack.common.gettextutils import _
 from nova.openstack.common import log as logging
 from nova.openstack.common import units
 from nova import utils
+from nova.virt.vmwareapi import ds_util
 from nova.virt.vmwareapi import vim_util
 from nova.virt.vmwareapi import vm_util
 
 LOG = logging.getLogger(__name__)
 
 
-class Host(object):
-    """Implements host related operations."""
-    def __init__(self, session):
-        self._session = session
-
-    def host_power_action(self, host, action):
-        """Reboots or shuts down the host."""
-        host_mor = vm_util.get_host_ref(self._session)
-        LOG.debug(_("%(action)s %(host)s"), {'action': action, 'host': host})
-        if action == "reboot":
-            host_task = self._session._call_method(
-                                    self._session._get_vim(),
-                                    "RebootHost_Task", host_mor,
-                                    force=False)
-        elif action == "shutdown":
-            host_task = self._session._call_method(
-                                    self._session._get_vim(),
-                                    "ShutdownHost_Task", host_mor,
-                                    force=False)
-        elif action == "startup":
-            host_task = self._session._call_method(
-                                    self._session._get_vim(),
-                                    "PowerUpHostFromStandBy_Task", host_mor,
-                                    timeoutSec=60)
-        self._session._wait_for_task(host_task)
-
-    def host_maintenance_mode(self, host, mode):
-        """Start/Stop host maintenance window. On start, it triggers
-        guest VMs evacuation.
-        """
-        host_mor = vm_util.get_host_ref(self._session)
-        LOG.debug(_("Set maintenance mod on %(host)s to %(mode)s"),
-                  {'host': host, 'mode': mode})
-        if mode:
-            host_task = self._session._call_method(
-                                    self._session._get_vim(),
-                                    "EnterMaintenanceMode_Task",
-                                    host_mor, timeout=0,
-                                    evacuatePoweredOffVms=True)
-        else:
-            host_task = self._session._call_method(
-                                    self._session._get_vim(),
-                                    "ExitMaintenanceMode_Task",
-                                    host_mor, timeout=0)
-        self._session._wait_for_task(host_task)
-
-    def set_host_enabled(self, _host, enabled):
-        """Sets the specified host's ability to accept new instances."""
-        pass
+def _get_ds_capacity_and_freespace(session, cluster=None):
+    try:
+        ds = ds_util.get_datastore(session, cluster)
+        return ds.capacity, ds.freespace
+    except exception.DatastoreNotFound:
+        return 0, 0
 
 
 class HostState(object):
@@ -111,10 +71,7 @@ class HostState(object):
         if summary is None:
             return
 
-        try:
-            ds = vm_util.get_datastore_ref_and_name(self._session)
-        except exception.DatastoreNotFound:
-            ds = (None, None, 0, 0)
+        capacity, freespace = _get_ds_capacity_and_freespace(self._session)
 
         data = {}
         data["vcpus"] = summary.hardware.numCpuThreads
@@ -125,8 +82,8 @@ class HostState(object):
                               "sockets": summary.hardware.numCpuPkgs,
                               "threads": summary.hardware.numCpuThreads}
                 }
-        data["disk_total"] = ds[2] / units.Gi
-        data["disk_available"] = ds[3] / units.Gi
+        data["disk_total"] = capacity / units.Gi
+        data["disk_available"] = freespace / units.Gi
         data["disk_used"] = data["disk_total"] - data["disk_available"]
         data["host_memory_total"] = summary.hardware.memorySize / units.Mi
         data["host_memory_free"] = data["host_memory_total"] - \
@@ -135,8 +92,9 @@ class HostState(object):
         data["hypervisor_version"] = utils.convert_version_to_int(
                 str(summary.config.product.version))
         data["hypervisor_hostname"] = self._host_name
-        data["supported_instances"] = [('i686', 'vmware', 'hvm'),
-                                       ('x86_64', 'vmware', 'hvm')]
+        data["supported_instances"] = [
+            (arch.I686, hvtype.VMWARE, vm_mode.HVM),
+            (arch.X86_64, hvtype.VMWARE, vm_mode.HVM)]
 
         self._stats = data
         return data
@@ -164,12 +122,8 @@ class VCState(object):
 
     def update_status(self):
         """Update the current state of the cluster."""
-        # Get the datastore in the cluster
-        try:
-            ds = vm_util.get_datastore_ref_and_name(self._session,
-                                                    self._cluster)
-        except exception.DatastoreNotFound:
-            ds = (None, None, 0, 0)
+        capacity, freespace = _get_ds_capacity_and_freespace(self._session,
+                                                             self._cluster)
 
         # Get cpu, memory stats from the cluster
         stats = vm_util.get_stats_from_cluster(self._session, self._cluster)
@@ -180,8 +134,8 @@ class VCState(object):
                             "model": stats['cpu']['model'],
                             "topology": {"cores": stats['cpu']['cores'],
                                          "threads": stats['cpu']['vcpus']}}
-        data["disk_total"] = ds[2] / units.Gi
-        data["disk_available"] = ds[3] / units.Gi
+        data["disk_total"] = capacity / units.Gi
+        data["disk_available"] = freespace / units.Gi
         data["disk_used"] = data["disk_total"] - data["disk_available"]
         data["host_memory_total"] = stats['mem']['total']
         data["host_memory_free"] = stats['mem']['free']
@@ -189,8 +143,9 @@ class VCState(object):
         data["hypervisor_version"] = utils.convert_version_to_int(
                 str(about_info.version))
         data["hypervisor_hostname"] = self._host_name
-        data["supported_instances"] = [('i686', 'vmware', 'hvm'),
-                                       ('x86_64', 'vmware', 'hvm')]
+        data["supported_instances"] = [
+            (arch.I686, hvtype.VMWARE, vm_mode.HVM),
+            (arch.X86_64, hvtype.VMWARE, vm_mode.HVM)]
 
         self._stats = data
         return data
