@@ -12,8 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-#============================================================================
-#
+
+
 # Parts of this file are based upon xmlrpclib.py, the XML-RPC client
 # interface included in the Python distribution.
 #
@@ -50,15 +50,14 @@ A fake XenAPI SDK.
 
 import base64
 import pickle
+import pprint
 import random
 import uuid
 from xml.sax import saxutils
 import zlib
 
-import pprint
-
 from nova import exception
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
@@ -77,7 +76,7 @@ LOG = logging.getLogger(__name__)
 def log_db_contents(msg=None):
     text = msg or ""
     content = pprint.pformat(_db_content)
-    LOG.debug(_("%(text)s: _db_content => %(content)s"),
+    LOG.debug("%(text)s: _db_content => %(content)s",
               {'text': text, 'content': content})
 
 
@@ -207,11 +206,15 @@ def after_VDI_create(vdi_ref, vdi_rec):
     vdi_rec.setdefault('VBDs', [])
 
 
-def create_vbd(vm_ref, vdi_ref, userdevice=0):
+def create_vbd(vm_ref, vdi_ref, userdevice=0, other_config=None):
+    if other_config is None:
+        other_config = {}
+
     vbd_rec = {'VM': vm_ref,
                'VDI': vdi_ref,
                'userdevice': str(userdevice),
-               'currently_attached': False}
+               'currently_attached': False,
+               'other_config': other_config}
     vbd_ref = _create_object('VBD', vbd_rec)
     after_VBD_create(vbd_ref, vbd_rec)
     return vbd_ref
@@ -223,6 +226,7 @@ def after_VBD_create(vbd_ref, vbd_rec):
     """
     vbd_rec['currently_attached'] = False
     vbd_rec['device'] = ''
+    vbd_rec.setdefault('other_config', {})
 
     vm_ref = vbd_rec['VM']
     vm_rec = _db_content['VM'][vm_ref]
@@ -237,6 +241,14 @@ def after_VBD_create(vbd_ref, vbd_rec):
         vdi_rec['VBDs'].append(vbd_ref)
 
 
+def after_VIF_create(vif_ref, vif_rec):
+    """Create backref from VM to VIF when VIF is created.
+    """
+    vm_ref = vif_rec['VM']
+    vm_rec = _db_content['VM'][vm_ref]
+    vm_rec['VIFs'].append(vif_ref)
+
+
 def after_VM_create(vm_ref, vm_rec):
     """Create read-only fields in the VM record."""
     vm_rec.setdefault('domid', -1)
@@ -246,6 +258,7 @@ def after_VM_create(vm_ref, vm_rec):
     vm_rec.setdefault('memory_dynamic_max', str(8 * units.Gi))
     vm_rec.setdefault('VCPUs_max', str(4))
     vm_rec.setdefault('VBDs', [])
+    vm_rec.setdefault('VIFs', [])
     vm_rec.setdefault('resident_on', '')
 
 
@@ -622,7 +635,7 @@ class SessionBase(object):
         return self.VDI_copy(_1, vdi_to_clone_ref, sr_ref)
 
     def host_compute_free_memory(self, _1, ref):
-        #Always return 12GB available
+        # Always return 12GB available
         return 12 * units.Gi
 
     def _plugin_agent_version(self, method, args):
@@ -699,14 +712,14 @@ class SessionBase(object):
 
         """
         # Driver is not pciback
-        dev_bad1 = ["Slot:\t86:10.0", "Class:\t0604", "Vendor:\t10b5",
+        dev_bad1 = ["Slot:\t0000:86:10.0", "Class:\t0604", "Vendor:\t10b5",
                     "Device:\t8747", "Rev:\tba", "Driver:\tpcieport", "\n"]
         # Driver is pciback but vendor and device are bad
-        dev_bad2 = ["Slot:\t88:00.0", "Class:\t0300", "Vendor:\t0bad",
+        dev_bad2 = ["Slot:\t0000:88:00.0", "Class:\t0300", "Vendor:\t0bad",
                     "Device:\tcafe", "SVendor:\t10de", "SDevice:\t100d",
                     "Rev:\ta1", "Driver:\tpciback", "\n"]
         # Driver is pciback and vendor, device are used for matching
-        dev_good = ["Slot:\t87:00.0", "Class:\t0300", "Vendor:\t10de",
+        dev_good = ["Slot:\t0000:87:00.0", "Class:\t0300", "Vendor:\t10de",
                     "Device:\t11bf", "SVendor:\t10de", "SDevice:\t100d",
                     "Rev:\ta1", "Driver:\tpciback", "\n"]
 
@@ -807,7 +820,7 @@ class SessionBase(object):
             full_params = (self._session,) + params
             meth = getattr(self, methodname, None)
             if meth is None:
-                LOG.debug(_('Raising NotImplemented'))
+                LOG.debug('Raising NotImplemented')
                 raise NotImplementedError(
                     _('xenapi.fake does not have an implementation for %s') %
                     methodname)
@@ -842,16 +855,16 @@ class SessionBase(object):
             if impl is not None:
 
                 def callit(*params):
-                    LOG.debug(_('Calling %(name)s %(impl)s'),
+                    LOG.debug('Calling %(name)s %(impl)s',
                               {'name': name, 'impl': impl})
                     self._check_session(params)
                     return impl(*params)
                 return callit
         if self._is_gettersetter(name, True):
-            LOG.debug(_('Calling getter %s'), name)
+            LOG.debug('Calling getter %s', name)
             return lambda *params: self._getter(name, params)
         elif self._is_gettersetter(name, False):
-            LOG.debug(_('Calling setter %s'), name)
+            LOG.debug('Calling setter %s', name)
             return lambda *params: self._setter(name, params)
         elif self._is_create(name):
             return lambda *params: self._create(name, params)
@@ -915,7 +928,7 @@ class SessionBase(object):
             else:
                 raise Failure(['HANDLE_INVALID', cls, ref])
 
-        LOG.debug(_('Raising NotImplemented'))
+        LOG.debug('Raising NotImplemented')
         raise NotImplementedError(
             _('xenapi.fake does not have an implementation for %s or it has '
             'been called with the wrong number of arguments') % name)
@@ -934,7 +947,7 @@ class SessionBase(object):
                 _db_content[cls][ref][field] = val
                 return
 
-        LOG.debug(_('Raising NotImplemented'))
+        LOG.debug('Raising NotImplemented')
         raise NotImplementedError(
             'xenapi.fake does not have an implementation for %s or it has '
             'been called with the wrong number of arguments or the database '
@@ -1002,7 +1015,7 @@ class SessionBase(object):
                 self._session not in _db_content['session']):
             raise Failure(['HANDLE_INVALID', 'session', self._session])
         if len(params) == 0 or params[0] != self._session:
-            LOG.debug(_('Raising NotImplemented'))
+            LOG.debug('Raising NotImplemented')
             raise NotImplementedError('Call to XenAPI without using .xenapi')
 
     def _check_arg_count(self, params, expected):
