@@ -19,20 +19,24 @@ Management class for host-related functions (start, reboot, etc).
 
 import re
 
+from oslo.config import cfg
+
+from nova.compute import arch
+from nova.compute import hvtype
 from nova.compute import task_states
+from nova.compute import vm_mode
 from nova.compute import vm_states
-from nova import conductor
 from nova import context
 from nova import exception
-from nova.objects import aggregate as aggregate_obj
-from nova.objects import instance as instance_obj
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
+from nova import objects
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.pci import pci_whitelist
 from nova.virt.xenapi import pool_states
 from nova.virt.xenapi import vm_utils
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -41,7 +45,6 @@ class Host(object):
     def __init__(self, session, virtapi):
         self._session = session
         self._virtapi = virtapi
-        self._conductor_api = conductor.API()
 
     def host_power_action(self, _host, action):
         """Reboots or shuts down the host."""
@@ -76,10 +79,10 @@ class Host(object):
                                        ' ping migration to a new host'),
                                      {'name': name, 'host': host})
                             continue
-                    instance = instance_obj.Instance.get_by_uuid(ctxt, uuid)
+                    instance = objects.Instance.get_by_uuid(ctxt, uuid)
                     vm_counter = vm_counter + 1
 
-                    aggregate = aggregate_obj.AggregateList.get_by_host(
+                    aggregate = objects.AggregateList.get_by_host(
                         ctxt, host, key=pool_states.POOL_FLAG)
                     if not aggregate:
                         msg = _('Aggregate for host %(host)s count not be'
@@ -114,22 +117,16 @@ class Host(object):
             raise exception.NoValidHost(reason='Unable to find suitable '
                                                    'host for VMs evacuation')
 
-    def set_host_enabled(self, host, enabled):
-        """Sets the specified host's ability to accept new instances."""
+    def set_host_enabled(self, enabled):
+        """Sets the compute host's ability to accept new instances."""
         # Since capabilities are gone, use service table to disable a node
         # in scheduler
-        status = {'disabled': not enabled,
-                'disabled_reason': 'set by xenapi host_state'
-                }
         cntxt = context.get_admin_context()
-        service = self._conductor_api.service_get_by_args(
-                cntxt,
-                host,
-                'nova-compute')
-        self._conductor_api.service_update(
-                cntxt,
-                service,
-                status)
+        service = objects.Service.get_by_args(cntxt, CONF.host,
+                                              'nova-compute')
+        service.disabled = not enabled
+        service.disabled_reason = 'set by xenapi host_state'
+        service.save()
 
         args = {"enabled": jsonutils.dumps(enabled)}
         response = call_xenhost(self._session, "set_host_enabled", args)
@@ -237,7 +234,7 @@ class HostState(object):
         """Since under Xenserver, a compute node runs on a given host,
         we can get host status information using xenapi.
         """
-        LOG.debug(_("Updating host stats"))
+        LOG.debug("Updating host stats")
         data = call_xenhost(self._session, "host_data", {})
         if data:
             sr_ref = vm_utils.scan_default_sr(self._session)
@@ -246,6 +243,7 @@ class HostState(object):
             used = int(sr_rec["physical_utilisation"])
             data["disk_total"] = total
             data["disk_used"] = used
+            data["disk_allocated"] = int(sr_rec["virtual_allocation"])
             data["disk_available"] = total - used
             data["supported_instances"] = to_supported_instances(
                 data.get("host_capabilities")
@@ -281,8 +279,16 @@ def to_supported_instances(host_capabilities):
     result = []
     for capability in host_capabilities:
         try:
-            ostype, _version, arch = capability.split("-")
-            result.append((arch, 'xapi', ostype))
+            # 'capability'is unicode but we want arch/ostype
+            # to be strings to match the standard constants
+            capability = str(capability)
+
+            ostype, _version, guestarch = capability.split("-")
+
+            guestarch = arch.canonicalize(guestarch)
+            ostype = vm_mode.canonicalize(ostype)
+
+            result.append((guestarch, hvtype.XEN, ostype))
         except ValueError:
             LOG.warning(
                 _("Failed to extract instance support from %s"), capability)
@@ -312,7 +318,7 @@ def call_xenhost(session, method, arg_dict):
 
 def _uuid_find(context, host, name_label):
     """Return instance uuid by name_label."""
-    for i in instance_obj.InstanceList.get_by_host(context, host):
+    for i in objects.InstanceList.get_by_host(context, host):
         if i.name == name_label:
             return i.uuid
     return None
