@@ -17,17 +17,17 @@
 """Eventlet WSGI Services to proxy VNC for XCP protocol."""
 
 import socket
-import webob
 
 import eventlet
 import eventlet.green
 import eventlet.greenio
 import eventlet.wsgi
 from oslo.config import cfg
+import webob
 
 from nova.consoleauth import rpcapi as consoleauth_rpcapi
 from nova import context
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import log as logging
 from nova import version
 from nova import wsgi
