@@ -14,7 +14,7 @@
 #    under the License.
 
 
-from nova.openstack.common.gettextutils import _
+from nova.i18n import _
 from nova.openstack.common import importutils
 from nova.openstack.common import log as logging
 from nova.volume.encryptors import nop
